# coding: utf8

# Copyright 2013-2015 Vincent Jacques <vincent@vincent-jacques.net>

import unittest

from .knot import Knot, String, Segment, End, Bridge, Tunnel


class KnotTestCase(unittest.TestCase):
    def test_1_1(self):
        knot = Knot(1, 1)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(0, 0), End(2, 0)),
                    ],
                    [
                    ],
                ),
            ]
        )

    def test_1_2(self):
        knot = Knot(1, 2)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(2, 0)),
                            Tunnel(0, Segment(End(2, 0), End(3, -1)), Segment(End(3, -1), End(4, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_2_1(self):
        knot = Knot(2, 1)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(0, 0), End(4, 0)),
                    ],
                    [
                    ],
                ),
            ]
        )

    def test_1_3(self):
        knot = Knot(1, 3)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(4, 0), End(5, -1)), Segment(End(5, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(3, 0), End(4, 1)),
                            Segment(End(4, 1), End(4, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(3, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_2_2(self):
        knot = Knot(2, 2)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(2, 0)),
                            Tunnel(1, Segment(End(0, 0), End(1, -1)), Segment(End(1, -1), End(2, 0))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(4, 0)),
                            Tunnel(0, Segment(End(2, 0), End(3, -1)), Segment(End(3, -1), End(4, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_3_1(self):
        knot = Knot(3, 1)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(0, 0), End(6, 0)),
                    ],
                    [
                    ],
                ),
            ]
        )

    def test_1_4(self):
        knot = Knot(1, 4)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(4, 0)),
                            Tunnel(0, Segment(End(4, 0), End(5, -1)), Segment(End(5, -1), End(5, 0))),
                        ),
                        Bridge(
                            Segment(End(5, 0), End(6, 1)),
                            Segment(End(6, 1), End(6, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_2_3(self):
        knot = Knot(2, 3)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(4, 0), End(5, -1)), Segment(End(5, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(3, 0), End(4, 1)),
                            Segment(End(4, 1), End(4, 0)),
                            Tunnel(0, Segment(End(7, 0), End(8, -1)), Segment(End(8, -1), End(9, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(7, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(12, 0))),
                        ),
                        Bridge(
                            Segment(End(9, 0), End(10, 1)),
                            Segment(End(10, 1), End(10, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(3, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_3_2(self):
        knot = Knot(3, 2)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(2, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(6, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(12, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(10, 0)),
                            Tunnel(0, Segment(End(2, 0), End(3, -1)), Segment(End(3, -1), End(4, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_4_1(self):
        knot = Knot(4, 1)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(0, 0), End(8, 0)),
                    ],
                    [
                    ],
                ),
            ]
        )

    def test_1_5(self):
        knot = Knot(1, 5)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(8, 0), End(9, -1)), Segment(End(9, -1), End(10, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(7, 0))),
                        ),
                        Bridge(
                            Segment(End(5, 0), End(6, 1)),
                            Segment(End(6, 1), End(6, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(5, 0))),
                        ),
                        Bridge(
                            Segment(End(7, 0), End(8, 1)),
                            Segment(End(8, 1), End(8, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_2_4(self):
        knot = Knot(2, 4)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(1, Segment(End(0, 0), End(1, -1)), Segment(End(1, -1), End(2, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(4, 0)),
                            Tunnel(1, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(7, 0))),
                        ),
                        Bridge(
                            Segment(End(5, 0), End(6, 1)),
                            Segment(End(6, 1), End(6, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, 0), End(0, 1)),
                            Segment(End(0, 1), End(0, 0)),
                            Tunnel(1, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(6, 0)),
                            Tunnel(0, Segment(End(4, 0), End(5, -1)), Segment(End(5, -1), End(5, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_3_3(self):
        knot = Knot(3, 3)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-2, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_4_2(self):
        knot = Knot(4, 2)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(2, 0)),
                            Tunnel(1, Segment(End(0, 0), End(1, -1)), Segment(End(1, -1), End(2, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(6, 0)),
                            Tunnel(1, Segment(End(4, 0), End(5, -1)), Segment(End(5, -1), End(6, 0))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(4, 0)),
                            Tunnel(0, Segment(End(2, 0), End(3, -1)), Segment(End(3, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(8, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(8, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_5_1(self):
        knot = Knot(5, 1)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(0, 0), End(10, 0)),
                    ],
                    [
                    ],
                ),
            ]
        )

    def test_1_6(self):
        knot = Knot(1, 6)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(12, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(8, 0), End(9, -1)), Segment(End(9, -1), End(9, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(6, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(7, 0))),
                        ),
                        Bridge(
                            Segment(End(7, 0), End(8, 1)),
                            Segment(End(8, 1), End(8, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(9, 0), End(10, 1)),
                            Segment(End(10, 1), End(10, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_2_5(self):
        knot = Knot(2, 5)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(8, 0), End(9, -1)), Segment(End(9, -1), End(10, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(7, 0))),
                        ),
                        Bridge(
                            Segment(End(5, 0), End(6, 1)),
                            Segment(End(6, 1), End(6, 0)),
                            Tunnel(0, Segment(End(13, 0), End(14, -1)), Segment(End(14, -1), End(15, 0))),
                        ),
                        Bridge(
                            Segment(End(7, 0), End(8, 1)),
                            Segment(End(8, 1), End(8, 0)),
                            Tunnel(0, Segment(End(11, 0), End(12, -1)), Segment(End(12, -1), End(12, 0))),
                        ),
                        Bridge(
                            Segment(End(10, 0), End(11, 1)),
                            Segment(End(11, 1), End(11, 0)),
                            Tunnel(0, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(20, 0))),
                        ),
                        Bridge(
                            Segment(End(12, 0), End(13, 1)),
                            Segment(End(13, 1), End(13, 0)),
                            Tunnel(0, Segment(End(16, 0), End(17, -1)), Segment(End(17, -1), End(17, 0))),
                        ),
                        Bridge(
                            Segment(End(15, 0), End(16, 1)),
                            Segment(End(16, 1), End(16, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(5, 0))),
                        ),
                        Bridge(
                            Segment(End(17, 0), End(18, 1)),
                            Segment(End(18, 1), End(18, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_3_4(self):
        knot = Knot(3, 4)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(0, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_4_3(self):
        knot = Knot(4, 3)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_5_2(self):
        knot = Knot(5, 2)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(2, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(12, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(6, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(16, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(10, 0)),
                            Tunnel(0, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(20, 0))),
                        ),
                        Bridge(
                            Segment(End(12, 0), End(13, 1)),
                            Segment(End(13, 1), End(14, 0)),
                            Tunnel(0, Segment(End(2, 0), End(3, -1)), Segment(End(3, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(16, 0), End(17, 1)),
                            Segment(End(17, 1), End(18, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(8, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_6_1(self):
        knot = Knot(6, 1)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(0, 0), End(12, 0)),
                    ],
                    [
                    ],
                ),
            ]
        )

    def test_1_7(self):
        knot = Knot(1, 7)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(14, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(11, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(0, Segment(End(8, 0), End(9, -1)), Segment(End(9, -1), End(9, 0))),
                        ),
                        Bridge(
                            Segment(End(7, 0), End(8, 1)),
                            Segment(End(8, 1), End(8, 0)),
                            Tunnel(0, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(7, 0))),
                        ),
                        Bridge(
                            Segment(End(9, 0), End(10, 1)),
                            Segment(End(10, 1), End(10, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(11, 0), End(12, 1)),
                            Segment(End(12, 1), End(12, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_2_6(self):
        knot = Knot(2, 6)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(1, Segment(End(0, 0), End(1, -1)), Segment(End(1, -1), End(2, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(1, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(11, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(6, 0)),
                            Tunnel(1, Segment(End(8, 0), End(9, -1)), Segment(End(9, -1), End(9, 0))),
                        ),
                        Bridge(
                            Segment(End(7, 0), End(8, 1)),
                            Segment(End(8, 1), End(8, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(9, 0), End(10, 1)),
                            Segment(End(10, 1), End(10, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, 0), End(0, 1)),
                            Segment(End(0, 1), End(0, 0)),
                            Tunnel(1, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(12, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(0, Segment(End(8, 0), End(9, -1)), Segment(End(9, -1), End(9, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(8, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(7, 0))),
                        ),
                        Bridge(
                            Segment(End(9, 0), End(10, 1)),
                            Segment(End(10, 1), End(10, 0)),
                            Tunnel(1, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_3_5(self):
        knot = Knot(3, 5)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_4_4(self):
        knot = Knot(4, 4)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(3, Segment(End(1, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(3, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(0, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(1, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(3, -1)),
                            Tunnel(2, Segment(End(-1, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_5_3(self):
        knot = Knot(5, 3)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_6_2(self):
        knot = Knot(6, 2)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(2, 0)),
                            Tunnel(1, Segment(End(0, 0), End(1, -1)), Segment(End(1, -1), End(2, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(6, 0)),
                            Tunnel(1, Segment(End(4, 0), End(5, -1)), Segment(End(5, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(10, 0)),
                            Tunnel(1, Segment(End(8, 0), End(9, -1)), Segment(End(9, -1), End(10, 0))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(4, 0)),
                            Tunnel(0, Segment(End(2, 0), End(3, -1)), Segment(End(3, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(8, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(10, 0), End(11, 1)),
                            Segment(End(11, 1), End(12, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(12, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_7_1(self):
        knot = Knot(7, 1)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(0, 0), End(14, 0)),
                    ],
                    [
                    ],
                ),
            ]
        )

    def test_1_8(self):
        knot = Knot(1, 8)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(16, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(13, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(11, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(8, 0)),
                            Tunnel(0, Segment(End(8, 0), End(9, -1)), Segment(End(9, -1), End(9, 0))),
                        ),
                        Bridge(
                            Segment(End(9, 0), End(10, 1)),
                            Segment(End(10, 1), End(10, 0)),
                            Tunnel(0, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(11, 0), End(12, 1)),
                            Segment(End(12, 1), End(12, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(13, 0), End(14, 1)),
                            Segment(End(14, 1), End(14, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_2_7(self):
        knot = Knot(2, 7)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(14, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(11, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(0, Segment(End(8, 0), End(9, -1)), Segment(End(9, -1), End(9, 0))),
                        ),
                        Bridge(
                            Segment(End(7, 0), End(8, 1)),
                            Segment(End(8, 1), End(8, 0)),
                            Tunnel(0, Segment(End(19, 0), End(20, -1)), Segment(End(20, -1), End(21, 0))),
                        ),
                        Bridge(
                            Segment(End(9, 0), End(10, 1)),
                            Segment(End(10, 1), End(10, 0)),
                            Tunnel(0, Segment(End(17, 0), End(18, -1)), Segment(End(18, -1), End(18, 0))),
                        ),
                        Bridge(
                            Segment(End(11, 0), End(12, 1)),
                            Segment(End(12, 1), End(12, 0)),
                            Tunnel(0, Segment(End(15, 0), End(16, -1)), Segment(End(16, -1), End(16, 0))),
                        ),
                        Bridge(
                            Segment(End(14, 0), End(15, 1)),
                            Segment(End(15, 1), End(15, 0)),
                            Tunnel(0, Segment(End(26, 0), End(27, -1)), Segment(End(27, -1), End(28, 0))),
                        ),
                        Bridge(
                            Segment(End(16, 0), End(17, 1)),
                            Segment(End(17, 1), End(17, 0)),
                            Tunnel(0, Segment(End(24, 0), End(25, -1)), Segment(End(25, -1), End(25, 0))),
                        ),
                        Bridge(
                            Segment(End(18, 0), End(19, 1)),
                            Segment(End(19, 1), End(19, 0)),
                            Tunnel(0, Segment(End(22, 0), End(23, -1)), Segment(End(23, -1), End(23, 0))),
                        ),
                        Bridge(
                            Segment(End(21, 0), End(22, 1)),
                            Segment(End(22, 1), End(22, 0)),
                            Tunnel(0, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(7, 0))),
                        ),
                        Bridge(
                            Segment(End(23, 0), End(24, 1)),
                            Segment(End(24, 1), End(24, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(25, 0), End(26, 1)),
                            Segment(End(26, 1), End(26, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_3_6(self):
        knot = Knot(3, 6)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(2, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(0, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(1, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_4_5(self):
        knot = Knot(4, 5)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_5_4(self):
        knot = Knot(5, 4)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(0, Segment(End(27, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(29, -1)),
                            Tunnel(0, Segment(End(35, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(37, -1)),
                            Tunnel(0, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_6_3(self):
        knot = Knot(6, 3)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-2, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_7_2(self):
        knot = Knot(7, 2)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(2, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(16, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(6, 0)),
                            Tunnel(0, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(20, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(10, 0)),
                            Tunnel(0, Segment(End(22, 0), End(23, -1)), Segment(End(23, -1), End(24, 0))),
                        ),
                        Bridge(
                            Segment(End(12, 0), End(13, 1)),
                            Segment(End(13, 1), End(14, 0)),
                            Tunnel(0, Segment(End(26, 0), End(27, -1)), Segment(End(27, -1), End(28, 0))),
                        ),
                        Bridge(
                            Segment(End(16, 0), End(17, 1)),
                            Segment(End(17, 1), End(18, 0)),
                            Tunnel(0, Segment(End(2, 0), End(3, -1)), Segment(End(3, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(20, 0), End(21, 1)),
                            Segment(End(21, 1), End(22, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(24, 0), End(25, 1)),
                            Segment(End(25, 1), End(26, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(12, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_8_1(self):
        knot = Knot(8, 1)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(0, 0), End(16, 0)),
                    ],
                    [
                    ],
                ),
            ]
        )

    def test_1_9(self):
        knot = Knot(1, 9)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(16, 0), End(17, -1)), Segment(End(17, -1), End(18, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(15, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(0, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(13, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(7, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(11, 0))),
                        ),
                        Bridge(
                            Segment(End(9, 0), End(10, 1)),
                            Segment(End(10, 1), End(10, 0)),
                            Tunnel(0, Segment(End(7, 0), End(8, -1)), Segment(End(8, -1), End(9, 0))),
                        ),
                        Bridge(
                            Segment(End(11, 0), End(12, 1)),
                            Segment(End(12, 1), End(12, 0)),
                            Tunnel(0, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(13, 0), End(14, 1)),
                            Segment(End(14, 1), End(14, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(15, 0), End(16, 1)),
                            Segment(End(16, 1), End(16, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_2_8(self):
        knot = Knot(2, 8)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(1, Segment(End(0, 0), End(1, -1)), Segment(End(1, -1), End(2, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(1, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(15, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(1, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(13, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(8, 0)),
                            Tunnel(1, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(11, 0))),
                        ),
                        Bridge(
                            Segment(End(9, 0), End(10, 1)),
                            Segment(End(10, 1), End(10, 0)),
                            Tunnel(0, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(11, 0), End(12, 1)),
                            Segment(End(12, 1), End(12, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(13, 0), End(14, 1)),
                            Segment(End(14, 1), End(14, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, 0), End(0, 1)),
                            Segment(End(0, 1), End(0, 0)),
                            Tunnel(1, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(16, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(0, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(13, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(7, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(11, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(10, 0)),
                            Tunnel(0, Segment(End(8, 0), End(9, -1)), Segment(End(9, -1), End(9, 0))),
                        ),
                        Bridge(
                            Segment(End(11, 0), End(12, 1)),
                            Segment(End(12, 1), End(12, 0)),
                            Tunnel(1, Segment(End(7, 0), End(8, -1)), Segment(End(8, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(13, 0), End(14, 1)),
                            Segment(End(14, 1), End(14, 0)),
                            Tunnel(1, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_3_7(self):
        knot = Knot(3, 7)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_4_6(self):
        knot = Knot(4, 6)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(1, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(1, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(1, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(1, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(0, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(0, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_5_5(self):
        knot = Knot(5, 5)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(4, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(4, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-2, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(3, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(4, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(4, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                    ],
                ),
                String(
                    4,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_6_4(self):
        knot = Knot(6, 4)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(1, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(1, Segment(End(21, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(1, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(0, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(1, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(23, -1)),
                            Tunnel(0, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_7_3(self):
        knot = Knot(7, 3)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(28, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_8_2(self):
        knot = Knot(8, 2)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(2, 0)),
                            Tunnel(1, Segment(End(0, 0), End(1, -1)), Segment(End(1, -1), End(2, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(6, 0)),
                            Tunnel(1, Segment(End(4, 0), End(5, -1)), Segment(End(5, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(10, 0)),
                            Tunnel(1, Segment(End(8, 0), End(9, -1)), Segment(End(9, -1), End(10, 0))),
                        ),
                        Bridge(
                            Segment(End(12, 0), End(13, 1)),
                            Segment(End(13, 1), End(14, 0)),
                            Tunnel(1, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(14, 0))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(4, 0)),
                            Tunnel(0, Segment(End(2, 0), End(3, -1)), Segment(End(3, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(8, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(10, 0), End(11, 1)),
                            Segment(End(11, 1), End(12, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(12, 0))),
                        ),
                        Bridge(
                            Segment(End(14, 0), End(15, 1)),
                            Segment(End(15, 1), End(16, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(16, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_9_1(self):
        knot = Knot(9, 1)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(0, 0), End(18, 0)),
                    ],
                    [
                    ],
                ),
            ]
        )

    def test_1_10(self):
        knot = Knot(1, 10)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(20, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(16, 0), End(17, -1)), Segment(End(17, -1), End(17, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(15, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(7, 0)),
                            Tunnel(0, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(13, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(10, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(11, 0))),
                        ),
                        Bridge(
                            Segment(End(11, 0), End(12, 1)),
                            Segment(End(12, 1), End(12, 0)),
                            Tunnel(0, Segment(End(7, 0), End(8, -1)), Segment(End(8, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(13, 0), End(14, 1)),
                            Segment(End(14, 1), End(14, 0)),
                            Tunnel(0, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(15, 0), End(16, 1)),
                            Segment(End(16, 1), End(16, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(17, 0), End(18, 1)),
                            Segment(End(18, 1), End(18, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_2_9(self):
        knot = Knot(2, 9)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(16, 0), End(17, -1)), Segment(End(17, -1), End(18, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(15, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(0, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(13, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(7, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(11, 0))),
                        ),
                        Bridge(
                            Segment(End(9, 0), End(10, 1)),
                            Segment(End(10, 1), End(10, 0)),
                            Tunnel(0, Segment(End(25, 0), End(26, -1)), Segment(End(26, -1), End(27, 0))),
                        ),
                        Bridge(
                            Segment(End(11, 0), End(12, 1)),
                            Segment(End(12, 1), End(12, 0)),
                            Tunnel(0, Segment(End(23, 0), End(24, -1)), Segment(End(24, -1), End(24, 0))),
                        ),
                        Bridge(
                            Segment(End(13, 0), End(14, 1)),
                            Segment(End(14, 1), End(14, 0)),
                            Tunnel(0, Segment(End(21, 0), End(22, -1)), Segment(End(22, -1), End(22, 0))),
                        ),
                        Bridge(
                            Segment(End(15, 0), End(16, 1)),
                            Segment(End(16, 1), End(16, 0)),
                            Tunnel(0, Segment(End(19, 0), End(20, -1)), Segment(End(20, -1), End(20, 0))),
                        ),
                        Bridge(
                            Segment(End(18, 0), End(19, 1)),
                            Segment(End(19, 1), End(19, 0)),
                            Tunnel(0, Segment(End(34, 0), End(35, -1)), Segment(End(35, -1), End(36, 0))),
                        ),
                        Bridge(
                            Segment(End(20, 0), End(21, 1)),
                            Segment(End(21, 1), End(21, 0)),
                            Tunnel(0, Segment(End(32, 0), End(33, -1)), Segment(End(33, -1), End(33, 0))),
                        ),
                        Bridge(
                            Segment(End(22, 0), End(23, 1)),
                            Segment(End(23, 1), End(23, 0)),
                            Tunnel(0, Segment(End(30, 0), End(31, -1)), Segment(End(31, -1), End(31, 0))),
                        ),
                        Bridge(
                            Segment(End(24, 0), End(25, 1)),
                            Segment(End(25, 1), End(25, 0)),
                            Tunnel(0, Segment(End(28, 0), End(29, -1)), Segment(End(29, -1), End(29, 0))),
                        ),
                        Bridge(
                            Segment(End(27, 0), End(28, 1)),
                            Segment(End(28, 1), End(28, 0)),
                            Tunnel(0, Segment(End(7, 0), End(8, -1)), Segment(End(8, -1), End(9, 0))),
                        ),
                        Bridge(
                            Segment(End(29, 0), End(30, 1)),
                            Segment(End(30, 1), End(30, 0)),
                            Tunnel(0, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(31, 0), End(32, 1)),
                            Segment(End(32, 1), End(32, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(33, 0), End(34, 1)),
                            Segment(End(34, 1), End(34, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_3_8(self):
        knot = Knot(3, 8)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(0, Segment(End(23, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(25, -1)),
                            Tunnel(0, Segment(End(39, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(41, -1)),
                            Tunnel(0, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_4_7(self):
        knot = Knot(4, 7)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(57, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_5_6(self):
        knot = Knot(5, 6)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(0, Segment(End(53, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(0, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(0, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(43, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(55, -1)),
                            Tunnel(0, Segment(End(41, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_6_5(self):
        knot = Knot(6, 5)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(0, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_7_4(self):
        knot = Knot(7, 4)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(57, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(0, Segment(End(43, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(51, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(0, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(29, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(37, -1)),
                            Tunnel(0, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(45, -1)),
                            Tunnel(0, Segment(End(27, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(53, -1)),
                            Tunnel(0, Segment(End(35, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_8_3(self):
        knot = Knot(8, 3)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_9_2(self):
        knot = Knot(9, 2)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(2, 0)),
                            Tunnel(0, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(20, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(6, 0)),
                            Tunnel(0, Segment(End(22, 0), End(23, -1)), Segment(End(23, -1), End(24, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(10, 0)),
                            Tunnel(0, Segment(End(26, 0), End(27, -1)), Segment(End(27, -1), End(28, 0))),
                        ),
                        Bridge(
                            Segment(End(12, 0), End(13, 1)),
                            Segment(End(13, 1), End(14, 0)),
                            Tunnel(0, Segment(End(30, 0), End(31, -1)), Segment(End(31, -1), End(32, 0))),
                        ),
                        Bridge(
                            Segment(End(16, 0), End(17, 1)),
                            Segment(End(17, 1), End(18, 0)),
                            Tunnel(0, Segment(End(34, 0), End(35, -1)), Segment(End(35, -1), End(36, 0))),
                        ),
                        Bridge(
                            Segment(End(20, 0), End(21, 1)),
                            Segment(End(21, 1), End(22, 0)),
                            Tunnel(0, Segment(End(2, 0), End(3, -1)), Segment(End(3, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(24, 0), End(25, 1)),
                            Segment(End(25, 1), End(26, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(28, 0), End(29, 1)),
                            Segment(End(29, 1), End(30, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(12, 0))),
                        ),
                        Bridge(
                            Segment(End(32, 0), End(33, 1)),
                            Segment(End(33, 1), End(34, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(16, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_10_1(self):
        knot = Knot(10, 1)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(0, 0), End(20, 0)),
                    ],
                    [
                    ],
                ),
            ]
        )

    def test_1_11(self):
        knot = Knot(1, 11)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(20, 0), End(21, -1)), Segment(End(21, -1), End(22, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(19, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(0, Segment(End(16, 0), End(17, -1)), Segment(End(17, -1), End(17, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(7, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(15, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(9, 0)),
                            Tunnel(0, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(13, 0))),
                        ),
                        Bridge(
                            Segment(End(11, 0), End(12, 1)),
                            Segment(End(12, 1), End(12, 0)),
                            Tunnel(0, Segment(End(9, 0), End(10, -1)), Segment(End(10, -1), End(11, 0))),
                        ),
                        Bridge(
                            Segment(End(13, 0), End(14, 1)),
                            Segment(End(14, 1), End(14, 0)),
                            Tunnel(0, Segment(End(7, 0), End(8, -1)), Segment(End(8, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(15, 0), End(16, 1)),
                            Segment(End(16, 1), End(16, 0)),
                            Tunnel(0, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(17, 0), End(18, 1)),
                            Segment(End(18, 1), End(18, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(19, 0), End(20, 1)),
                            Segment(End(20, 1), End(20, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_2_10(self):
        knot = Knot(2, 10)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(1, Segment(End(0, 0), End(1, -1)), Segment(End(1, -1), End(2, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(1, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(19, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(1, Segment(End(16, 0), End(17, -1)), Segment(End(17, -1), End(17, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(7, 0)),
                            Tunnel(1, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(15, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(10, 0)),
                            Tunnel(1, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(13, 0))),
                        ),
                        Bridge(
                            Segment(End(11, 0), End(12, 1)),
                            Segment(End(12, 1), End(12, 0)),
                            Tunnel(0, Segment(End(7, 0), End(8, -1)), Segment(End(8, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(13, 0), End(14, 1)),
                            Segment(End(14, 1), End(14, 0)),
                            Tunnel(0, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(15, 0), End(16, 1)),
                            Segment(End(16, 1), End(16, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(17, 0), End(18, 1)),
                            Segment(End(18, 1), End(18, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, 0), End(0, 1)),
                            Segment(End(0, 1), End(0, 0)),
                            Tunnel(1, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(20, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(0, Segment(End(16, 0), End(17, -1)), Segment(End(17, -1), End(17, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(7, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(15, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(9, 0)),
                            Tunnel(0, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(13, 0))),
                        ),
                        Bridge(
                            Segment(End(10, 0), End(11, 1)),
                            Segment(End(11, 1), End(12, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(11, 0))),
                        ),
                        Bridge(
                            Segment(End(13, 0), End(14, 1)),
                            Segment(End(14, 1), End(14, 0)),
                            Tunnel(1, Segment(End(9, 0), End(10, -1)), Segment(End(10, -1), End(10, 0))),
                        ),
                        Bridge(
                            Segment(End(15, 0), End(16, 1)),
                            Segment(End(16, 1), End(16, 0)),
                            Tunnel(1, Segment(End(7, 0), End(8, -1)), Segment(End(8, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(17, 0), End(18, 1)),
                            Segment(End(18, 1), End(18, 0)),
                            Tunnel(1, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_3_9(self):
        knot = Knot(3, 9)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(2, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_4_8(self):
        knot = Knot(4, 8)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(3, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(2, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(0, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(3, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(3, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(1, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(2, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_5_7(self):
        knot = Knot(5, 7)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(71, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_6_6(self):
        knot = Knot(6, 6)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(5, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(4, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(5, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(4, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(0, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(4, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(5, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(1, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(5, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(4, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(2, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                    ],
                ),
                String(
                    4,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(3, -1)),
                            Tunnel(3, Segment(End(-1, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(5, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(1, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                    ],
                ),
                String(
                    5,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(4, Segment(End(1, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_7_5(self):
        knot = Knot(7, 5)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(71, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_8_4(self):
        knot = Knot(8, 4)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(3, Segment(End(1, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(3, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(3, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(0, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(2, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(1, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(3, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(17, -1)),
                            Tunnel(1, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(3, -1)),
                            Tunnel(2, Segment(End(-1, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(2, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(1, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_9_3(self):
        knot = Knot(9, 3)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(18, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(18, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-2, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(2, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_10_2(self):
        knot = Knot(10, 2)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(2, 0)),
                            Tunnel(1, Segment(End(0, 0), End(1, -1)), Segment(End(1, -1), End(2, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(6, 0)),
                            Tunnel(1, Segment(End(4, 0), End(5, -1)), Segment(End(5, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(10, 0)),
                            Tunnel(1, Segment(End(8, 0), End(9, -1)), Segment(End(9, -1), End(10, 0))),
                        ),
                        Bridge(
                            Segment(End(12, 0), End(13, 1)),
                            Segment(End(13, 1), End(14, 0)),
                            Tunnel(1, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(14, 0))),
                        ),
                        Bridge(
                            Segment(End(16, 0), End(17, 1)),
                            Segment(End(17, 1), End(18, 0)),
                            Tunnel(1, Segment(End(16, 0), End(17, -1)), Segment(End(17, -1), End(18, 0))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(4, 0)),
                            Tunnel(0, Segment(End(2, 0), End(3, -1)), Segment(End(3, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(8, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(10, 0), End(11, 1)),
                            Segment(End(11, 1), End(12, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(12, 0))),
                        ),
                        Bridge(
                            Segment(End(14, 0), End(15, 1)),
                            Segment(End(15, 1), End(16, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(16, 0))),
                        ),
                        Bridge(
                            Segment(End(18, 0), End(19, 1)),
                            Segment(End(19, 1), End(20, 0)),
                            Tunnel(0, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(20, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_11_1(self):
        knot = Knot(11, 1)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(0, 0), End(22, 0)),
                    ],
                    [
                    ],
                ),
            ]
        )

    def test_1_12(self):
        knot = Knot(1, 12)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(22, 0), End(23, -1)), Segment(End(23, -1), End(24, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(20, 0), End(21, -1)), Segment(End(21, -1), End(21, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(0, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(19, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(7, 0)),
                            Tunnel(0, Segment(End(16, 0), End(17, -1)), Segment(End(17, -1), End(17, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(9, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(15, 0))),
                        ),
                        Bridge(
                            Segment(End(10, 0), End(11, 1)),
                            Segment(End(11, 1), End(12, 0)),
                            Tunnel(0, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(13, 0))),
                        ),
                        Bridge(
                            Segment(End(13, 0), End(14, 1)),
                            Segment(End(14, 1), End(14, 0)),
                            Tunnel(0, Segment(End(9, 0), End(10, -1)), Segment(End(10, -1), End(10, 0))),
                        ),
                        Bridge(
                            Segment(End(15, 0), End(16, 1)),
                            Segment(End(16, 1), End(16, 0)),
                            Tunnel(0, Segment(End(7, 0), End(8, -1)), Segment(End(8, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(17, 0), End(18, 1)),
                            Segment(End(18, 1), End(18, 0)),
                            Tunnel(0, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(19, 0), End(20, 1)),
                            Segment(End(20, 1), End(20, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(21, 0), End(22, 1)),
                            Segment(End(22, 1), End(22, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_2_11(self):
        knot = Knot(2, 11)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(45, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(0, Segment(End(20, 0), End(21, -1)), Segment(End(21, -1), End(22, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(19, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(0, Segment(End(16, 0), End(17, -1)), Segment(End(17, -1), End(17, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(7, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(15, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(9, 0)),
                            Tunnel(0, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(13, 0))),
                        ),
                        Bridge(
                            Segment(End(11, 0), End(12, 1)),
                            Segment(End(12, 1), End(12, 0)),
                            Tunnel(0, Segment(End(31, 0), End(32, -1)), Segment(End(32, -1), End(33, 0))),
                        ),
                        Bridge(
                            Segment(End(13, 0), End(14, 1)),
                            Segment(End(14, 1), End(14, 0)),
                            Tunnel(0, Segment(End(29, 0), End(30, -1)), Segment(End(30, -1), End(30, 0))),
                        ),
                        Bridge(
                            Segment(End(15, 0), End(16, 1)),
                            Segment(End(16, 1), End(16, 0)),
                            Tunnel(0, Segment(End(27, 0), End(28, -1)), Segment(End(28, -1), End(28, 0))),
                        ),
                        Bridge(
                            Segment(End(17, 0), End(18, 1)),
                            Segment(End(18, 1), End(18, 0)),
                            Tunnel(0, Segment(End(25, 0), End(26, -1)), Segment(End(26, -1), End(26, 0))),
                        ),
                        Bridge(
                            Segment(End(19, 0), End(20, 1)),
                            Segment(End(20, 1), End(20, 0)),
                            Tunnel(0, Segment(End(23, 0), End(24, -1)), Segment(End(24, -1), End(24, 0))),
                        ),
                        Bridge(
                            Segment(End(22, 0), End(23, 1)),
                            Segment(End(23, 1), End(23, 0)),
                            Tunnel(0, Segment(End(42, 0), End(43, -1)), Segment(End(43, -1), End(44, 0))),
                        ),
                        Bridge(
                            Segment(End(24, 0), End(25, 1)),
                            Segment(End(25, 1), End(25, 0)),
                            Tunnel(0, Segment(End(40, 0), End(41, -1)), Segment(End(41, -1), End(41, 0))),
                        ),
                        Bridge(
                            Segment(End(26, 0), End(27, 1)),
                            Segment(End(27, 1), End(27, 0)),
                            Tunnel(0, Segment(End(38, 0), End(39, -1)), Segment(End(39, -1), End(39, 0))),
                        ),
                        Bridge(
                            Segment(End(28, 0), End(29, 1)),
                            Segment(End(29, 1), End(29, 0)),
                            Tunnel(0, Segment(End(36, 0), End(37, -1)), Segment(End(37, -1), End(37, 0))),
                        ),
                        Bridge(
                            Segment(End(30, 0), End(31, 1)),
                            Segment(End(31, 1), End(31, 0)),
                            Tunnel(0, Segment(End(34, 0), End(35, -1)), Segment(End(35, -1), End(35, 0))),
                        ),
                        Bridge(
                            Segment(End(33, 0), End(34, 1)),
                            Segment(End(34, 1), End(34, 0)),
                            Tunnel(0, Segment(End(9, 0), End(10, -1)), Segment(End(10, -1), End(11, 0))),
                        ),
                        Bridge(
                            Segment(End(35, 0), End(36, 1)),
                            Segment(End(36, 1), End(36, 0)),
                            Tunnel(0, Segment(End(7, 0), End(8, -1)), Segment(End(8, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(37, 0), End(38, 1)),
                            Segment(End(38, 1), End(38, 0)),
                            Tunnel(0, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(39, 0), End(40, 1)),
                            Segment(End(40, 1), End(40, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(41, 0), End(42, 1)),
                            Segment(End(42, 1), End(42, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_3_10(self):
        knot = Knot(3, 10)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(0, Segment(End(49, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(0, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(51, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_4_9(self):
        knot = Knot(4, 9)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_5_8(self):
        knot = Knot(5, 8)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(73, -1)),
                        Segment(End(73, -1), End(74, 1)),
                        Segment(End(74, 1), End(75, -1)),
                        Segment(End(75, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(74, 1), End(75, -1)), Segment(End(75, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(0, Segment(End(55, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(25, -1)),
                            Tunnel(0, Segment(End(71, 1), End(73, -1)), Segment(End(73, -1), End(74, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(41, -1)),
                            Tunnel(0, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(57, -1)),
                            Tunnel(0, Segment(End(23, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(73, -1)),
                            Tunnel(0, Segment(End(39, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(73, -1), End(74, 1)),
                            Segment(End(74, 1), End(75, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(75, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_6_7(self):
        knot = Knot(6, 7)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(85, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_7_6(self):
        knot = Knot(7, 6)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(65, 1)),
                        Segment(End(65, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(77, 1)),
                        Segment(End(77, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(85, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(0, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(77, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(0, Segment(End(41, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(43, -1)),
                            Tunnel(0, Segment(End(53, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(55, -1)),
                            Tunnel(0, Segment(End(65, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(65, 1)),
                            Segment(End(65, 1), End(67, -1)),
                            Tunnel(0, Segment(End(77, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(77, 1)),
                            Segment(End(77, 1), End(79, -1)),
                            Tunnel(0, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_8_5(self):
        knot = Knot(8, 5)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_9_4(self):
        knot = Knot(9, 4)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(0, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(27, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(0, Segment(End(35, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(29, -1)),
                            Tunnel(0, Segment(End(43, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(37, -1)),
                            Tunnel(0, Segment(End(51, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(45, -1)),
                            Tunnel(0, Segment(End(59, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(53, -1)),
                            Tunnel(0, Segment(End(67, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(61, -1)),
                            Tunnel(0, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(69, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_10_3(self):
        knot = Knot(10, 3)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(40, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_11_2(self):
        knot = Knot(11, 2)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(43, -1)),
                        Segment(End(43, -1), End(45, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(2, 0)),
                            Tunnel(0, Segment(End(22, 0), End(23, -1)), Segment(End(23, -1), End(24, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(6, 0)),
                            Tunnel(0, Segment(End(26, 0), End(27, -1)), Segment(End(27, -1), End(28, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(10, 0)),
                            Tunnel(0, Segment(End(30, 0), End(31, -1)), Segment(End(31, -1), End(32, 0))),
                        ),
                        Bridge(
                            Segment(End(12, 0), End(13, 1)),
                            Segment(End(13, 1), End(14, 0)),
                            Tunnel(0, Segment(End(34, 0), End(35, -1)), Segment(End(35, -1), End(36, 0))),
                        ),
                        Bridge(
                            Segment(End(16, 0), End(17, 1)),
                            Segment(End(17, 1), End(18, 0)),
                            Tunnel(0, Segment(End(38, 0), End(39, -1)), Segment(End(39, -1), End(40, 0))),
                        ),
                        Bridge(
                            Segment(End(20, 0), End(21, 1)),
                            Segment(End(21, 1), End(22, 0)),
                            Tunnel(0, Segment(End(42, 0), End(43, -1)), Segment(End(43, -1), End(44, 0))),
                        ),
                        Bridge(
                            Segment(End(24, 0), End(25, 1)),
                            Segment(End(25, 1), End(26, 0)),
                            Tunnel(0, Segment(End(2, 0), End(3, -1)), Segment(End(3, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(28, 0), End(29, 1)),
                            Segment(End(29, 1), End(30, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(32, 0), End(33, 1)),
                            Segment(End(33, 1), End(34, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(12, 0))),
                        ),
                        Bridge(
                            Segment(End(36, 0), End(37, 1)),
                            Segment(End(37, 1), End(38, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(16, 0))),
                        ),
                        Bridge(
                            Segment(End(40, 0), End(41, 1)),
                            Segment(End(41, 1), End(42, 0)),
                            Tunnel(0, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(20, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_12_1(self):
        knot = Knot(12, 1)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(0, 0), End(24, 0)),
                    ],
                    [
                    ],
                ),
            ]
        )

    def test_2_12(self):
        knot = Knot(2, 12)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(1, 0)),
                            Tunnel(1, Segment(End(0, 0), End(1, -1)), Segment(End(1, -1), End(2, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(1, Segment(End(22, 0), End(23, -1)), Segment(End(23, -1), End(23, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(1, Segment(End(20, 0), End(21, -1)), Segment(End(21, -1), End(21, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(7, 0)),
                            Tunnel(1, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(19, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(9, 0)),
                            Tunnel(1, Segment(End(16, 0), End(17, -1)), Segment(End(17, -1), End(17, 0))),
                        ),
                        Bridge(
                            Segment(End(10, 0), End(11, 1)),
                            Segment(End(11, 1), End(12, 0)),
                            Tunnel(1, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(15, 0))),
                        ),
                        Bridge(
                            Segment(End(13, 0), End(14, 1)),
                            Segment(End(14, 1), End(14, 0)),
                            Tunnel(0, Segment(End(9, 0), End(10, -1)), Segment(End(10, -1), End(10, 0))),
                        ),
                        Bridge(
                            Segment(End(15, 0), End(16, 1)),
                            Segment(End(16, 1), End(16, 0)),
                            Tunnel(0, Segment(End(7, 0), End(8, -1)), Segment(End(8, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(17, 0), End(18, 1)),
                            Segment(End(18, 1), End(18, 0)),
                            Tunnel(0, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(19, 0), End(20, 1)),
                            Segment(End(20, 1), End(20, 0)),
                            Tunnel(0, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(21, 0), End(22, 1)),
                            Segment(End(22, 1), End(22, 0)),
                            Tunnel(0, Segment(End(1, 0), End(2, -1)), Segment(End(2, -1), End(2, 0))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, 0), End(0, 1)),
                            Segment(End(0, 1), End(0, 0)),
                            Tunnel(1, Segment(End(3, 0), End(4, -1)), Segment(End(4, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(3, 0)),
                            Tunnel(0, Segment(End(22, 0), End(23, -1)), Segment(End(23, -1), End(24, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(5, 0)),
                            Tunnel(0, Segment(End(20, 0), End(21, -1)), Segment(End(21, -1), End(21, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(7, 0)),
                            Tunnel(0, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(19, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(9, 0)),
                            Tunnel(0, Segment(End(16, 0), End(17, -1)), Segment(End(17, -1), End(17, 0))),
                        ),
                        Bridge(
                            Segment(End(10, 0), End(11, 1)),
                            Segment(End(11, 1), End(11, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(15, 0))),
                        ),
                        Bridge(
                            Segment(End(12, 0), End(13, 1)),
                            Segment(End(13, 1), End(14, 0)),
                            Tunnel(0, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(13, 0))),
                        ),
                        Bridge(
                            Segment(End(15, 0), End(16, 1)),
                            Segment(End(16, 1), End(16, 0)),
                            Tunnel(1, Segment(End(11, 0), End(12, -1)), Segment(End(12, -1), End(12, 0))),
                        ),
                        Bridge(
                            Segment(End(17, 0), End(18, 1)),
                            Segment(End(18, 1), End(18, 0)),
                            Tunnel(1, Segment(End(9, 0), End(10, -1)), Segment(End(10, -1), End(10, 0))),
                        ),
                        Bridge(
                            Segment(End(19, 0), End(20, 1)),
                            Segment(End(20, 1), End(20, 0)),
                            Tunnel(1, Segment(End(7, 0), End(8, -1)), Segment(End(8, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(21, 0), End(22, 1)),
                            Segment(End(22, 1), End(22, 0)),
                            Tunnel(1, Segment(End(5, 0), End(6, -1)), Segment(End(6, -1), End(6, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_3_11(self):
        knot = Knot(3, 11)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(67, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_4_10(self):
        knot = Knot(4, 10)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(1, Segment(End(31, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(1, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(1, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(1, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(1, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(1, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(1, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(33, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(1, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_5_9(self):
        knot = Knot(5, 9)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(77, 1)),
                        Segment(End(77, 1), End(78, -1)),
                        Segment(End(78, -1), End(79, 1)),
                        Segment(End(79, 1), End(80, -1)),
                        Segment(End(80, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(84, 1)),
                        Segment(End(84, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(88, 1)),
                        Segment(End(88, 1), End(89, -1)),
                        Segment(End(89, -1), End(91, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(84, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(79, 1), End(80, -1)), Segment(End(80, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(88, 1), End(89, -1)), Segment(End(89, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(84, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(0, Segment(End(77, 1), End(78, -1)), Segment(End(78, -1), End(79, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(88, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(77, 1)),
                            Segment(End(77, 1), End(78, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(78, -1), End(79, 1)),
                            Segment(End(79, 1), End(80, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(80, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(84, 1)),
                            Segment(End(84, 1), End(85, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(77, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(88, 1)),
                            Segment(End(88, 1), End(89, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_6_8(self):
        knot = Knot(6, 8)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(1, Segment(End(41, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(1, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(1, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(25, -1)),
                            Tunnel(1, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(1, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(1, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(41, -1)),
                            Tunnel(1, Segment(End(25, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(0, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(1, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(1, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(27, -1)),
                            Tunnel(0, Segment(End(23, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(1, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(1, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(43, -1)),
                            Tunnel(0, Segment(End(39, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(1, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_7_7(self):
        knot = Knot(7, 7)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(5, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(5, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(6, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(4, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(6, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(4, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(5, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(5, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(3, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(14, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(14, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-2, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(4, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(6, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(4, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(6, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(1, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                    ],
                ),
                String(
                    4,
                    [
                        Segment(End(0, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(5, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(5, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(2, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                    ],
                ),
                String(
                    5,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(3, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(6, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(6, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                    ],
                ),
                String(
                    6,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(4, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(4, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_8_6(self):
        knot = Knot(8, 6)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(1, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(1, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(1, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(1, Segment(End(31, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(1, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(1, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(1, Segment(End(43, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(1, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(1, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(43, -1)),
                            Tunnel(1, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(0, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(1, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(0, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(1, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(33, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(1, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(45, -1)),
                            Tunnel(0, Segment(End(41, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(1, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_9_5(self):
        knot = Knot(9, 5)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(84, -1)),
                        Segment(End(84, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(88, 1)),
                        Segment(End(88, 1), End(89, -1)),
                        Segment(End(89, -1), End(91, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(88, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(0, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(83, 1), End(84, -1)), Segment(End(84, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(88, 1), End(89, -1)), Segment(End(89, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(84, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(84, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(88, 1)),
                            Segment(End(88, 1), End(89, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_10_4(self):
        knot = Knot(10, 4)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(1, Segment(End(21, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(1, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(1, Segment(End(37, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(1, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(29, -1)),
                            Tunnel(1, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(1, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(37, -1)),
                            Tunnel(1, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(0, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(1, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(23, -1)),
                            Tunnel(0, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(0, Segment(End(27, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(1, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(39, -1)),
                            Tunnel(0, Segment(End(35, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_11_3(self):
        knot = Knot(11, 3)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(67, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_12_2(self):
        knot = Knot(12, 2)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, 0), End(1, 1)),
                            Segment(End(1, 1), End(2, 0)),
                            Tunnel(1, Segment(End(0, 0), End(1, -1)), Segment(End(1, -1), End(2, 0))),
                        ),
                        Bridge(
                            Segment(End(4, 0), End(5, 1)),
                            Segment(End(5, 1), End(6, 0)),
                            Tunnel(1, Segment(End(4, 0), End(5, -1)), Segment(End(5, -1), End(6, 0))),
                        ),
                        Bridge(
                            Segment(End(8, 0), End(9, 1)),
                            Segment(End(9, 1), End(10, 0)),
                            Tunnel(1, Segment(End(8, 0), End(9, -1)), Segment(End(9, -1), End(10, 0))),
                        ),
                        Bridge(
                            Segment(End(12, 0), End(13, 1)),
                            Segment(End(13, 1), End(14, 0)),
                            Tunnel(1, Segment(End(12, 0), End(13, -1)), Segment(End(13, -1), End(14, 0))),
                        ),
                        Bridge(
                            Segment(End(16, 0), End(17, 1)),
                            Segment(End(17, 1), End(18, 0)),
                            Tunnel(1, Segment(End(16, 0), End(17, -1)), Segment(End(17, -1), End(18, 0))),
                        ),
                        Bridge(
                            Segment(End(20, 0), End(21, 1)),
                            Segment(End(21, 1), End(22, 0)),
                            Tunnel(1, Segment(End(20, 0), End(21, -1)), Segment(End(21, -1), End(22, 0))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(25, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(2, 0), End(3, 1)),
                            Segment(End(3, 1), End(4, 0)),
                            Tunnel(0, Segment(End(2, 0), End(3, -1)), Segment(End(3, -1), End(4, 0))),
                        ),
                        Bridge(
                            Segment(End(6, 0), End(7, 1)),
                            Segment(End(7, 1), End(8, 0)),
                            Tunnel(0, Segment(End(6, 0), End(7, -1)), Segment(End(7, -1), End(8, 0))),
                        ),
                        Bridge(
                            Segment(End(10, 0), End(11, 1)),
                            Segment(End(11, 1), End(12, 0)),
                            Tunnel(0, Segment(End(10, 0), End(11, -1)), Segment(End(11, -1), End(12, 0))),
                        ),
                        Bridge(
                            Segment(End(14, 0), End(15, 1)),
                            Segment(End(15, 1), End(16, 0)),
                            Tunnel(0, Segment(End(14, 0), End(15, -1)), Segment(End(15, -1), End(16, 0))),
                        ),
                        Bridge(
                            Segment(End(18, 0), End(19, 1)),
                            Segment(End(19, 1), End(20, 0)),
                            Tunnel(0, Segment(End(18, 0), End(19, -1)), Segment(End(19, -1), End(20, 0))),
                        ),
                        Bridge(
                            Segment(End(22, 0), End(23, 1)),
                            Segment(End(23, 1), End(24, 0)),
                            Tunnel(0, Segment(End(22, 0), End(23, -1)), Segment(End(23, -1), End(24, 0))),
                        ),
                    ],
                ),
            ]
        )

    def test_3_12(self):
        knot = Knot(3, 12)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(2, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(2, Segment(End(15, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(2, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(17, -1)),
                            Tunnel(1, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(1, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_4_11(self):
        knot = Knot(4, 11)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(84, 1)),
                        Segment(End(84, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(89, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(84, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(84, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(84, 1)),
                            Segment(End(84, 1), End(85, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_5_10(self):
        knot = Knot(5, 10)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(2, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(4, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(4, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(4, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(4, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(3, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(3, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(4, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(1, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(4, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(4, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(4, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(3, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(17, -1)),
                            Tunnel(2, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                    ],
                ),
                String(
                    4,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(3, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(4, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(3, Segment(End(15, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_6_9(self):
        knot = Knot(6, 9)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(2, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(2, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(1, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(1, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(1, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(2, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(1, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(2, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(2, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(2, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(2, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_7_8(self):
        knot = Knot(7, 8)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(73, -1)),
                        Segment(End(73, -1), End(74, 1)),
                        Segment(End(74, 1), End(75, -1)),
                        Segment(End(75, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(84, -1)),
                        Segment(End(84, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(89, -1)),
                        Segment(End(89, -1), End(90, 1)),
                        Segment(End(90, 1), End(91, -1)),
                        Segment(End(91, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(110, 1)),
                        Segment(End(110, 1), End(111, -1)),
                        Segment(End(111, -1), End(113, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(74, 1), End(75, -1)), Segment(End(75, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(0, Segment(End(103, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(99, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(90, 1), End(91, -1)), Segment(End(91, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(25, -1)),
                            Tunnel(0, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(83, 1), End(84, -1)), Segment(End(84, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(41, -1)),
                            Tunnel(0, Segment(End(23, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(57, -1)),
                            Tunnel(0, Segment(End(39, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(110, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(73, -1)),
                            Tunnel(0, Segment(End(55, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(73, -1), End(74, 1)),
                            Segment(End(74, 1), End(75, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(75, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(84, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(84, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(89, -1)),
                            Tunnel(0, Segment(End(71, 1), End(73, -1)), Segment(End(73, -1), End(74, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(90, 1)),
                            Segment(End(90, 1), End(91, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(91, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(110, 1), End(111, -1)), Segment(End(111, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(105, -1)),
                            Tunnel(0, Segment(End(87, 1), End(89, -1)), Segment(End(89, -1), End(90, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(110, 1)),
                            Segment(End(110, 1), End(111, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_8_7(self):
        knot = Knot(8, 7)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(88, -1)),
                        Segment(End(88, -1), End(89, 1)),
                        Segment(End(89, 1), End(90, -1)),
                        Segment(End(90, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(96, 1)),
                        Segment(End(96, 1), End(97, -1)),
                        Segment(End(97, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(110, 1)),
                        Segment(End(110, 1), End(111, -1)),
                        Segment(End(111, -1), End(113, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(96, 1), End(97, -1)), Segment(End(97, -1), End(99, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(110, 1), End(111, -1)), Segment(End(111, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(0, Segment(End(87, 1), End(88, -1)), Segment(End(88, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(96, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(110, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(88, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(88, -1), End(89, 1)),
                            Segment(End(89, 1), End(90, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(90, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(96, 1)),
                            Segment(End(96, 1), End(97, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(97, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(89, 1), End(90, -1)), Segment(End(90, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(110, 1)),
                            Segment(End(110, 1), End(111, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_9_6(self):
        knot = Knot(9, 6)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(2, Segment(End(21, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(2, Segment(End(33, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(2, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(1, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(2, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(2, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(1, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(0, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(2, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(2, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(1, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(0, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(2, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(1, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(33, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(2, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(1, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(1, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(23, -1)),
                            Tunnel(1, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(2, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(35, -1)),
                            Tunnel(1, Segment(End(31, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_10_5(self):
        knot = Knot(10, 5)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(1, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(3, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(3, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(4, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(4, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(4, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(2, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(4, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(2, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(20, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(20, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-2, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(3, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(3, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(4, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(4, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(4, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(1, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(4, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                    ],
                ),
                String(
                    4,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(2, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(2, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_11_4(self):
        knot = Knot(11, 4)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(89, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(0, Segment(End(67, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(75, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(0, Segment(End(83, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(29, -1)),
                            Tunnel(0, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(37, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(45, -1)),
                            Tunnel(0, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(53, -1)),
                            Tunnel(0, Segment(End(27, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(61, -1)),
                            Tunnel(0, Segment(End(35, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(69, -1)),
                            Tunnel(0, Segment(End(43, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(77, -1)),
                            Tunnel(0, Segment(End(51, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(85, -1)),
                            Tunnel(0, Segment(End(59, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_12_3(self):
        knot = Knot(12, 3)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(1, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(24, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-2, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(2, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(2, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(2, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_4_12(self):
        knot = Knot(4, 12)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(3, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(3, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(2, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(2, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(3, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(3, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(3, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(3, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(17, -1)),
                            Tunnel(1, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(2, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(2, Segment(End(15, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(1, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(3, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_5_11(self):
        knot = Knot(5, 11)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(84, 1)),
                        Segment(End(84, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(89, 1)),
                        Segment(End(89, 1), End(90, -1)),
                        Segment(End(90, -1), End(91, 1)),
                        Segment(End(91, 1), End(92, -1)),
                        Segment(End(92, -1), End(93, 1)),
                        Segment(End(93, 1), End(94, -1)),
                        Segment(End(94, -1), End(95, 1)),
                        Segment(End(95, 1), End(96, -1)),
                        Segment(End(96, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(100, 1)),
                        Segment(End(100, 1), End(101, -1)),
                        Segment(End(101, -1), End(102, 1)),
                        Segment(End(102, 1), End(103, -1)),
                        Segment(End(103, -1), End(104, 1)),
                        Segment(End(104, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(111, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(104, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(89, 1), End(90, -1)), Segment(End(90, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(84, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(100, 1), End(101, -1)), Segment(End(101, -1), End(102, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(95, 1), End(96, -1)), Segment(End(96, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(91, 1), End(92, -1)), Segment(End(92, -1), End(93, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(102, 1), End(103, -1)), Segment(End(103, -1), End(104, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(100, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(84, 1)),
                            Segment(End(84, 1), End(85, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(89, 1)),
                            Segment(End(89, 1), End(90, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(111, 1))),
                        ),
                        Bridge(
                            Segment(End(90, -1), End(91, 1)),
                            Segment(End(91, 1), End(92, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(92, -1), End(93, 1)),
                            Segment(End(93, 1), End(94, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(84, 1))),
                        ),
                        Bridge(
                            Segment(End(94, -1), End(95, 1)),
                            Segment(End(95, 1), End(96, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(96, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(100, 1)),
                            Segment(End(100, 1), End(101, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(101, -1), End(102, 1)),
                            Segment(End(102, 1), End(103, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(103, -1), End(104, 1)),
                            Segment(End(104, 1), End(105, -1)),
                            Tunnel(0, Segment(End(93, 1), End(94, -1)), Segment(End(94, -1), End(95, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_6_10(self):
        knot = Knot(6, 10)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(1, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(1, Segment(End(31, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(1, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(1, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(1, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(1, Segment(End(51, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(1, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(1, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(1, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(51, -1)),
                            Tunnel(1, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(1, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(1, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(33, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(1, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(1, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(53, -1)),
                            Tunnel(0, Segment(End(49, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(1, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(1, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_7_9(self):
        knot = Knot(7, 9)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(77, 1)),
                        Segment(End(77, 1), End(78, -1)),
                        Segment(End(78, -1), End(79, 1)),
                        Segment(End(79, 1), End(80, -1)),
                        Segment(End(80, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(84, 1)),
                        Segment(End(84, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(88, 1)),
                        Segment(End(88, 1), End(89, -1)),
                        Segment(End(89, -1), End(91, 1)),
                        Segment(End(91, 1), End(92, -1)),
                        Segment(End(92, -1), End(93, 1)),
                        Segment(End(93, 1), End(94, -1)),
                        Segment(End(94, -1), End(95, 1)),
                        Segment(End(95, 1), End(96, -1)),
                        Segment(End(96, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(100, 1)),
                        Segment(End(100, 1), End(101, -1)),
                        Segment(End(101, -1), End(102, 1)),
                        Segment(End(102, 1), End(103, -1)),
                        Segment(End(103, -1), End(104, 1)),
                        Segment(End(104, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(109, 1)),
                        Segment(End(109, 1), End(110, -1)),
                        Segment(End(110, -1), End(111, 1)),
                        Segment(End(111, 1), End(112, -1)),
                        Segment(End(112, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(120, 1)),
                        Segment(End(120, 1), End(121, -1)),
                        Segment(End(121, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(127, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(88, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(102, 1), End(103, -1)), Segment(End(103, -1), End(104, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(120, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(79, 1), End(80, -1)), Segment(End(80, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(95, 1), End(96, -1)), Segment(End(96, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(111, 1), End(112, -1)), Segment(End(112, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(88, 1), End(89, -1)), Segment(End(89, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(104, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(120, 1), End(121, -1)), Segment(End(121, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(100, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(109, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(127, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(77, 1)),
                            Segment(End(77, 1), End(78, -1)),
                            Tunnel(0, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(78, -1), End(79, 1)),
                            Segment(End(79, 1), End(80, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(80, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(84, 1)),
                            Segment(End(84, 1), End(85, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(88, 1)),
                            Segment(End(88, 1), End(89, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(91, 1)),
                            Segment(End(91, 1), End(92, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(92, -1), End(93, 1)),
                            Segment(End(93, 1), End(94, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(94, -1), End(95, 1)),
                            Segment(End(95, 1), End(96, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(96, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(84, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(100, 1)),
                            Segment(End(100, 1), End(101, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(101, -1), End(102, 1)),
                            Segment(End(102, 1), End(103, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(103, -1), End(104, 1)),
                            Segment(End(104, 1), End(105, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(77, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(91, 1), End(92, -1)), Segment(End(92, -1), End(93, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(109, 1)),
                            Segment(End(109, 1), End(110, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(110, -1), End(111, 1)),
                            Segment(End(111, 1), End(112, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(112, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(84, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(100, 1), End(101, -1)), Segment(End(101, -1), End(102, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(119, -1), End(120, 1)),
                            Segment(End(120, 1), End(121, -1)),
                            Tunnel(0, Segment(End(77, 1), End(78, -1)), Segment(End(78, -1), End(79, 1))),
                        ),
                        Bridge(
                            Segment(End(121, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(93, 1), End(94, -1)), Segment(End(94, -1), End(95, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(109, 1), End(110, -1)), Segment(End(110, -1), End(111, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_8_8(self):
        knot = Knot(8, 8)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(5, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(7, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(4, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(6, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(7, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(4, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(6, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(0, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(3, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(5, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(6, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(5, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(7, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(1, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(4, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(5, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(7, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(4, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(6, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(2, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                    ],
                ),
                String(
                    4,
                    [
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(6, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(5, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(7, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(17, -1)),
                            Tunnel(3, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                    ],
                ),
                String(
                    5,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(3, -1)),
                            Tunnel(4, Segment(End(-1, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(7, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(6, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(2, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                    ],
                ),
                String(
                    6,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(3, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(5, Segment(End(1, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(4, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(7, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                    ],
                ),
                String(
                    7,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(4, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(6, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(5, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_9_7(self):
        knot = Knot(9, 7)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(88, -1)),
                        Segment(End(88, -1), End(89, 1)),
                        Segment(End(89, 1), End(90, -1)),
                        Segment(End(90, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(96, 1)),
                        Segment(End(96, 1), End(97, -1)),
                        Segment(End(97, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(110, 1)),
                        Segment(End(110, 1), End(111, -1)),
                        Segment(End(111, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(117, 1)),
                        Segment(End(117, 1), End(118, -1)),
                        Segment(End(118, -1), End(120, 1)),
                        Segment(End(120, 1), End(121, -1)),
                        Segment(End(121, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(127, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(89, 1), End(90, -1)), Segment(End(90, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(96, 1), End(97, -1)), Segment(End(97, -1), End(99, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(87, 1), End(88, -1)), Segment(End(88, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(110, 1), End(111, -1)), Segment(End(111, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(96, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(117, 1), End(118, -1)), Segment(End(118, -1), End(120, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(127, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(110, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(117, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(88, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(88, -1), End(89, 1)),
                            Segment(End(89, 1), End(90, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(90, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(96, 1)),
                            Segment(End(96, 1), End(97, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(97, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(120, 1), End(121, -1)), Segment(End(121, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(110, 1)),
                            Segment(End(110, 1), End(111, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(117, 1)),
                            Segment(End(117, 1), End(118, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(118, -1), End(120, 1)),
                            Segment(End(120, 1), End(121, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(121, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_10_6(self):
        knot = Knot(10, 6)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(1, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(1, Segment(End(43, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(1, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(1, Segment(End(55, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(1, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(1, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(1, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(1, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(1, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(43, -1)),
                            Tunnel(1, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(1, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(1, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(55, -1)),
                            Tunnel(1, Segment(End(31, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(0, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(1, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(0, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(1, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(33, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(1, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(1, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(45, -1)),
                            Tunnel(0, Segment(End(41, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(1, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(57, -1)),
                            Tunnel(0, Segment(End(53, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(1, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_11_5(self):
        knot = Knot(11, 5)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(84, -1)),
                        Segment(End(84, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(88, 1)),
                        Segment(End(88, 1), End(89, -1)),
                        Segment(End(89, -1), End(91, 1)),
                        Segment(End(91, 1), End(92, -1)),
                        Segment(End(92, -1), End(93, 1)),
                        Segment(End(93, 1), End(94, -1)),
                        Segment(End(94, -1), End(96, 1)),
                        Segment(End(96, 1), End(97, -1)),
                        Segment(End(97, -1), End(98, 1)),
                        Segment(End(98, 1), End(99, -1)),
                        Segment(End(99, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(111, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(88, 1), End(89, -1)), Segment(End(89, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(93, 1), End(94, -1)), Segment(End(94, -1), End(96, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(98, 1), End(99, -1)), Segment(End(99, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(111, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(88, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(0, Segment(End(91, 1), End(92, -1)), Segment(End(92, -1), End(93, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(96, 1), End(97, -1)), Segment(End(97, -1), End(98, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(84, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(84, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(88, 1)),
                            Segment(End(88, 1), End(89, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(91, 1)),
                            Segment(End(91, 1), End(92, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(92, -1), End(93, 1)),
                            Segment(End(93, 1), End(94, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(94, -1), End(96, 1)),
                            Segment(End(96, 1), End(97, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(97, -1), End(98, 1)),
                            Segment(End(98, 1), End(99, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(99, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(83, 1), End(84, -1)), Segment(End(84, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_12_4(self):
        knot = Knot(12, 4)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(3, Segment(End(1, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(3, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(3, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(2, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(3, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(0, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(2, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(3, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(2, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(23, -1)),
                            Tunnel(0, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(25, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(1, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(3, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(17, -1)),
                            Tunnel(1, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(3, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(25, -1)),
                            Tunnel(1, Segment(End(21, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(3, -1)),
                            Tunnel(2, Segment(End(-1, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(2, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(1, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(2, Segment(End(15, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(1, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_5_12(self):
        knot = Knot(5, 12)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(77, 1)),
                        Segment(End(77, 1), End(78, -1)),
                        Segment(End(78, -1), End(79, 1)),
                        Segment(End(79, 1), End(80, -1)),
                        Segment(End(80, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(88, 1)),
                        Segment(End(88, 1), End(89, -1)),
                        Segment(End(89, -1), End(90, 1)),
                        Segment(End(90, 1), End(91, -1)),
                        Segment(End(91, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(105, 1)),
                        Segment(End(105, 1), End(106, -1)),
                        Segment(End(106, -1), End(107, 1)),
                        Segment(End(107, 1), End(109, -1)),
                        Segment(End(109, -1), End(110, 1)),
                        Segment(End(110, 1), End(111, -1)),
                        Segment(End(111, -1), End(112, 1)),
                        Segment(End(112, 1), End(113, -1)),
                        Segment(End(113, -1), End(114, 1)),
                        Segment(End(114, 1), End(115, -1)),
                        Segment(End(115, -1), End(116, 1)),
                        Segment(End(116, 1), End(117, -1)),
                        Segment(End(117, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(121, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(114, 1), End(115, -1)), Segment(End(115, -1), End(116, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(59, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(77, 1), End(78, -1)), Segment(End(78, -1), End(79, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(116, 1), End(117, -1)), Segment(End(117, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(37, -1)),
                            Tunnel(0, Segment(End(83, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(79, 1), End(80, -1)), Segment(End(80, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(121, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(88, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(61, -1)),
                            Tunnel(0, Segment(End(107, 1), End(109, -1)), Segment(End(109, -1), End(110, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(105, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(77, 1)),
                            Segment(End(77, 1), End(78, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(78, -1), End(79, 1)),
                            Segment(End(79, 1), End(80, -1)),
                            Tunnel(0, Segment(End(88, 1), End(89, -1)), Segment(End(89, -1), End(90, 1))),
                        ),
                        Bridge(
                            Segment(End(80, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(110, 1), End(111, -1)), Segment(End(111, -1), End(112, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(85, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(105, 1), End(106, -1)), Segment(End(106, -1), End(107, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(88, 1)),
                            Segment(End(88, 1), End(89, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(90, 1)),
                            Segment(End(90, 1), End(91, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(91, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(90, 1), End(91, -1)), Segment(End(91, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(112, 1), End(113, -1)), Segment(End(113, -1), End(114, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(105, 1)),
                            Segment(End(105, 1), End(106, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(106, -1), End(107, 1)),
                            Segment(End(107, 1), End(109, -1)),
                            Tunnel(0, Segment(End(35, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(110, 1)),
                            Segment(End(110, 1), End(111, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(112, 1)),
                            Segment(End(112, 1), End(113, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(113, -1), End(114, 1)),
                            Segment(End(114, 1), End(115, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(115, -1), End(116, 1)),
                            Segment(End(116, 1), End(117, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(77, 1))),
                        ),
                        Bridge(
                            Segment(End(117, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(99, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_6_11(self):
        knot = Knot(6, 11)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(84, 1)),
                        Segment(End(84, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(89, 1)),
                        Segment(End(89, 1), End(90, -1)),
                        Segment(End(90, -1), End(91, 1)),
                        Segment(End(91, 1), End(92, -1)),
                        Segment(End(92, -1), End(93, 1)),
                        Segment(End(93, 1), End(94, -1)),
                        Segment(End(94, -1), End(95, 1)),
                        Segment(End(95, 1), End(96, -1)),
                        Segment(End(96, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(100, 1)),
                        Segment(End(100, 1), End(101, -1)),
                        Segment(End(101, -1), End(102, 1)),
                        Segment(End(102, 1), End(103, -1)),
                        Segment(End(103, -1), End(104, 1)),
                        Segment(End(104, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(111, 1)),
                        Segment(End(111, 1), End(112, -1)),
                        Segment(End(112, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(117, 1)),
                        Segment(End(117, 1), End(118, -1)),
                        Segment(End(118, -1), End(119, 1)),
                        Segment(End(119, 1), End(120, -1)),
                        Segment(End(120, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(126, 1)),
                        Segment(End(126, 1), End(127, -1)),
                        Segment(End(127, -1), End(128, 1)),
                        Segment(End(128, 1), End(129, -1)),
                        Segment(End(129, -1), End(130, 1)),
                        Segment(End(130, 1), End(131, -1)),
                        Segment(End(131, -1), End(133, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(111, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(102, 1), End(103, -1)), Segment(End(103, -1), End(104, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(119, 1), End(120, -1)), Segment(End(120, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(130, 1), End(131, -1)), Segment(End(131, -1), End(133, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(84, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(126, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(95, 1), End(96, -1)), Segment(End(96, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(89, 1), End(90, -1)), Segment(End(90, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(100, 1), End(101, -1)), Segment(End(101, -1), End(102, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(117, 1), End(118, -1)), Segment(End(118, -1), End(119, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(111, 1), End(112, -1)), Segment(End(112, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(128, 1), End(129, -1)), Segment(End(129, -1), End(130, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(84, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(93, 1), End(94, -1)), Segment(End(94, -1), End(95, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(84, 1)),
                            Segment(End(84, 1), End(85, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(89, 1)),
                            Segment(End(89, 1), End(90, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(90, -1), End(91, 1)),
                            Segment(End(91, 1), End(92, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(92, -1), End(93, 1)),
                            Segment(End(93, 1), End(94, -1)),
                            Tunnel(0, Segment(End(104, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(94, -1), End(95, 1)),
                            Segment(End(95, 1), End(96, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(96, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(100, 1)),
                            Segment(End(100, 1), End(101, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(101, -1), End(102, 1)),
                            Segment(End(102, 1), End(103, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(103, -1), End(104, 1)),
                            Segment(End(104, 1), End(105, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(117, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(111, 1)),
                            Segment(End(111, 1), End(112, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(112, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(126, 1), End(127, -1)), Segment(End(127, -1), End(128, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(117, 1)),
                            Segment(End(117, 1), End(118, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(118, -1), End(119, 1)),
                            Segment(End(119, 1), End(120, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(120, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(100, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(126, 1)),
                            Segment(End(126, 1), End(127, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(127, -1), End(128, 1)),
                            Segment(End(128, 1), End(129, -1)),
                            Tunnel(0, Segment(End(91, 1), End(92, -1)), Segment(End(92, -1), End(93, 1))),
                        ),
                        Bridge(
                            Segment(End(129, -1), End(130, 1)),
                            Segment(End(130, 1), End(131, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_7_10(self):
        knot = Knot(7, 10)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(71, -1)),
                        Segment(End(71, -1), End(72, 1)),
                        Segment(End(72, 1), End(73, -1)),
                        Segment(End(73, -1), End(74, 1)),
                        Segment(End(74, 1), End(75, -1)),
                        Segment(End(75, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(84, -1)),
                        Segment(End(84, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(88, -1)),
                        Segment(End(88, -1), End(89, 1)),
                        Segment(End(89, 1), End(91, -1)),
                        Segment(End(91, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(96, 1)),
                        Segment(End(96, 1), End(97, -1)),
                        Segment(End(97, -1), End(98, 1)),
                        Segment(End(98, 1), End(99, -1)),
                        Segment(End(99, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(105, 1)),
                        Segment(End(105, 1), End(106, -1)),
                        Segment(End(106, -1), End(107, 1)),
                        Segment(End(107, 1), End(108, -1)),
                        Segment(End(108, -1), End(109, 1)),
                        Segment(End(109, 1), End(111, -1)),
                        Segment(End(111, -1), End(112, 1)),
                        Segment(End(112, 1), End(113, -1)),
                        Segment(End(113, -1), End(114, 1)),
                        Segment(End(114, 1), End(115, -1)),
                        Segment(End(115, -1), End(116, 1)),
                        Segment(End(116, 1), End(117, -1)),
                        Segment(End(117, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(121, 1)),
                        Segment(End(121, 1), End(122, -1)),
                        Segment(End(122, -1), End(123, 1)),
                        Segment(End(123, 1), End(124, -1)),
                        Segment(End(124, -1), End(125, 1)),
                        Segment(End(125, 1), End(126, -1)),
                        Segment(End(126, -1), End(127, 1)),
                        Segment(End(127, 1), End(128, -1)),
                        Segment(End(128, -1), End(129, 1)),
                        Segment(End(129, 1), End(131, -1)),
                        Segment(End(131, -1), End(132, 1)),
                        Segment(End(132, 1), End(133, -1)),
                        Segment(End(133, -1), End(134, 1)),
                        Segment(End(134, 1), End(135, -1)),
                        Segment(End(135, -1), End(136, 1)),
                        Segment(End(136, 1), End(137, -1)),
                        Segment(End(137, -1), End(138, 1)),
                        Segment(End(138, 1), End(139, -1)),
                        Segment(End(139, -1), End(141, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(98, 1), End(99, -1)), Segment(End(99, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(74, 1), End(75, -1)), Segment(End(75, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(132, 1), End(133, -1)), Segment(End(133, -1), End(134, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(0, Segment(End(49, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(125, 1), End(126, -1)), Segment(End(126, -1), End(127, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(121, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(96, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(0, Segment(End(69, 1), End(71, -1)), Segment(End(71, -1), End(72, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(87, 1), End(88, -1)), Segment(End(88, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(121, 1), End(122, -1)), Segment(End(122, -1), End(123, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(138, 1), End(139, -1)), Segment(End(139, -1), End(141, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(114, 1), End(115, -1)), Segment(End(115, -1), End(116, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(51, -1)),
                            Tunnel(0, Segment(End(89, 1), End(91, -1)), Segment(End(91, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(107, 1), End(108, -1)), Segment(End(108, -1), End(109, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(83, 1), End(84, -1)), Segment(End(84, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(0, Segment(End(134, 1), End(135, -1)), Segment(End(135, -1), End(136, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(71, -1)),
                            Tunnel(0, Segment(End(109, 1), End(111, -1)), Segment(End(111, -1), End(112, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(72, 1)),
                            Segment(End(72, 1), End(73, -1)),
                            Tunnel(0, Segment(End(127, 1), End(128, -1)), Segment(End(128, -1), End(129, 1))),
                        ),
                        Bridge(
                            Segment(End(73, -1), End(74, 1)),
                            Segment(End(74, 1), End(75, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(75, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(105, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(84, -1)),
                            Tunnel(0, Segment(End(96, 1), End(97, -1)), Segment(End(97, -1), End(98, 1))),
                        ),
                        Bridge(
                            Segment(End(84, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(88, -1)),
                            Tunnel(0, Segment(End(72, 1), End(73, -1)), Segment(End(73, -1), End(74, 1))),
                        ),
                        Bridge(
                            Segment(End(88, -1), End(89, 1)),
                            Segment(End(89, 1), End(91, -1)),
                            Tunnel(0, Segment(End(129, 1), End(131, -1)), Segment(End(131, -1), End(132, 1))),
                        ),
                        Bridge(
                            Segment(End(91, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(96, 1)),
                            Segment(End(96, 1), End(97, -1)),
                            Tunnel(0, Segment(End(123, 1), End(124, -1)), Segment(End(124, -1), End(125, 1))),
                        ),
                        Bridge(
                            Segment(End(97, -1), End(98, 1)),
                            Segment(End(98, 1), End(99, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(99, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(116, 1), End(117, -1)), Segment(End(117, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(105, 1)),
                            Segment(End(105, 1), End(106, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(106, -1), End(107, 1)),
                            Segment(End(107, 1), End(108, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(108, -1), End(109, 1)),
                            Segment(End(109, 1), End(111, -1)),
                            Tunnel(0, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(112, 1)),
                            Segment(End(112, 1), End(113, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(113, -1), End(114, 1)),
                            Segment(End(114, 1), End(115, -1)),
                            Tunnel(0, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                        Bridge(
                            Segment(End(115, -1), End(116, 1)),
                            Segment(End(116, 1), End(117, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(117, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(119, -1), End(121, 1)),
                            Segment(End(121, 1), End(122, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(122, -1), End(123, 1)),
                            Segment(End(123, 1), End(124, -1)),
                            Tunnel(0, Segment(End(136, 1), End(137, -1)), Segment(End(137, -1), End(138, 1))),
                        ),
                        Bridge(
                            Segment(End(124, -1), End(125, 1)),
                            Segment(End(125, 1), End(126, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(126, -1), End(127, 1)),
                            Segment(End(127, 1), End(128, -1)),
                            Tunnel(0, Segment(End(112, 1), End(113, -1)), Segment(End(113, -1), End(114, 1))),
                        ),
                        Bridge(
                            Segment(End(128, -1), End(129, 1)),
                            Segment(End(129, 1), End(131, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(131, -1), End(132, 1)),
                            Segment(End(132, 1), End(133, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(133, -1), End(134, 1)),
                            Segment(End(134, 1), End(135, -1)),
                            Tunnel(0, Segment(End(105, 1), End(106, -1)), Segment(End(106, -1), End(107, 1))),
                        ),
                        Bridge(
                            Segment(End(135, -1), End(136, 1)),
                            Segment(End(136, 1), End(137, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(137, -1), End(138, 1)),
                            Segment(End(138, 1), End(139, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_8_9(self):
        knot = Knot(8, 9)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(77, 1)),
                        Segment(End(77, 1), End(78, -1)),
                        Segment(End(78, -1), End(79, 1)),
                        Segment(End(79, 1), End(80, -1)),
                        Segment(End(80, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(84, 1)),
                        Segment(End(84, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(88, 1)),
                        Segment(End(88, 1), End(89, -1)),
                        Segment(End(89, -1), End(91, 1)),
                        Segment(End(91, 1), End(92, -1)),
                        Segment(End(92, -1), End(93, 1)),
                        Segment(End(93, 1), End(94, -1)),
                        Segment(End(94, -1), End(95, 1)),
                        Segment(End(95, 1), End(96, -1)),
                        Segment(End(96, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(100, 1)),
                        Segment(End(100, 1), End(101, -1)),
                        Segment(End(101, -1), End(102, 1)),
                        Segment(End(102, 1), End(103, -1)),
                        Segment(End(103, -1), End(104, 1)),
                        Segment(End(104, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(109, 1)),
                        Segment(End(109, 1), End(110, -1)),
                        Segment(End(110, -1), End(111, 1)),
                        Segment(End(111, 1), End(112, -1)),
                        Segment(End(112, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(120, 1)),
                        Segment(End(120, 1), End(121, -1)),
                        Segment(End(121, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(127, 1)),
                        Segment(End(127, 1), End(128, -1)),
                        Segment(End(128, -1), End(129, 1)),
                        Segment(End(129, 1), End(130, -1)),
                        Segment(End(130, -1), End(131, 1)),
                        Segment(End(131, 1), End(132, -1)),
                        Segment(End(132, -1), End(133, 1)),
                        Segment(End(133, 1), End(134, -1)),
                        Segment(End(134, -1), End(136, 1)),
                        Segment(End(136, 1), End(137, -1)),
                        Segment(End(137, -1), End(138, 1)),
                        Segment(End(138, 1), End(139, -1)),
                        Segment(End(139, -1), End(140, 1)),
                        Segment(End(140, 1), End(141, -1)),
                        Segment(End(141, -1), End(142, 1)),
                        Segment(End(142, 1), End(143, -1)),
                        Segment(End(143, -1), End(145, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(84, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(120, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(93, 1), End(94, -1)), Segment(End(94, -1), End(95, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(127, 1), End(128, -1)), Segment(End(128, -1), End(129, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(102, 1), End(103, -1)), Segment(End(103, -1), End(104, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(136, 1), End(137, -1)), Segment(End(137, -1), End(138, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(77, 1), End(78, -1)), Segment(End(78, -1), End(79, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(111, 1), End(112, -1)), Segment(End(112, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(88, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(120, 1), End(121, -1)), Segment(End(121, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(0, Segment(End(95, 1), End(96, -1)), Segment(End(96, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(129, 1), End(130, -1)), Segment(End(130, -1), End(131, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(104, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(138, 1), End(139, -1)), Segment(End(139, -1), End(140, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(79, 1), End(80, -1)), Segment(End(80, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(88, 1), End(89, -1)), Segment(End(89, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(77, 1)),
                            Segment(End(77, 1), End(78, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(78, -1), End(79, 1)),
                            Segment(End(79, 1), End(80, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(80, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(100, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(84, 1)),
                            Segment(End(84, 1), End(85, -1)),
                            Tunnel(0, Segment(End(131, 1), End(132, -1)), Segment(End(132, -1), End(133, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(88, 1)),
                            Segment(End(88, 1), End(89, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(91, 1)),
                            Segment(End(91, 1), End(92, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(109, 1))),
                        ),
                        Bridge(
                            Segment(End(92, -1), End(93, 1)),
                            Segment(End(93, 1), End(94, -1)),
                            Tunnel(0, Segment(End(140, 1), End(141, -1)), Segment(End(141, -1), End(142, 1))),
                        ),
                        Bridge(
                            Segment(End(94, -1), End(95, 1)),
                            Segment(End(95, 1), End(96, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(96, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(100, 1)),
                            Segment(End(100, 1), End(101, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(101, -1), End(102, 1)),
                            Segment(End(102, 1), End(103, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(103, -1), End(104, 1)),
                            Segment(End(104, 1), End(105, -1)),
                            Tunnel(0, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(109, 1)),
                            Segment(End(109, 1), End(110, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(127, 1))),
                        ),
                        Bridge(
                            Segment(End(110, -1), End(111, 1)),
                            Segment(End(111, 1), End(112, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(112, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(84, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(133, 1), End(134, -1)), Segment(End(134, -1), End(136, 1))),
                        ),
                        Bridge(
                            Segment(End(119, -1), End(120, 1)),
                            Segment(End(120, 1), End(121, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(121, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(91, 1), End(92, -1)), Segment(End(92, -1), End(93, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(127, 1)),
                            Segment(End(127, 1), End(128, -1)),
                            Tunnel(0, Segment(End(142, 1), End(143, -1)), Segment(End(143, -1), End(145, 1))),
                        ),
                        Bridge(
                            Segment(End(128, -1), End(129, 1)),
                            Segment(End(129, 1), End(130, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(130, -1), End(131, 1)),
                            Segment(End(131, 1), End(132, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(132, -1), End(133, 1)),
                            Segment(End(133, 1), End(134, -1)),
                            Tunnel(0, Segment(End(100, 1), End(101, -1)), Segment(End(101, -1), End(102, 1))),
                        ),
                        Bridge(
                            Segment(End(134, -1), End(136, 1)),
                            Segment(End(136, 1), End(137, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(137, -1), End(138, 1)),
                            Segment(End(138, 1), End(139, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(139, -1), End(140, 1)),
                            Segment(End(140, 1), End(141, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(77, 1))),
                        ),
                        Bridge(
                            Segment(End(141, -1), End(142, 1)),
                            Segment(End(142, 1), End(143, -1)),
                            Tunnel(0, Segment(End(109, 1), End(110, -1)), Segment(End(110, -1), End(111, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_9_8(self):
        knot = Knot(9, 8)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(73, -1)),
                        Segment(End(73, -1), End(74, 1)),
                        Segment(End(74, 1), End(75, -1)),
                        Segment(End(75, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(84, -1)),
                        Segment(End(84, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(89, -1)),
                        Segment(End(89, -1), End(90, 1)),
                        Segment(End(90, 1), End(91, -1)),
                        Segment(End(91, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(110, 1)),
                        Segment(End(110, 1), End(111, -1)),
                        Segment(End(111, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(117, 1)),
                        Segment(End(117, 1), End(118, -1)),
                        Segment(End(118, -1), End(119, 1)),
                        Segment(End(119, 1), End(121, -1)),
                        Segment(End(121, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(126, 1)),
                        Segment(End(126, 1), End(127, -1)),
                        Segment(End(127, -1), End(129, 1)),
                        Segment(End(129, 1), End(130, -1)),
                        Segment(End(130, -1), End(131, 1)),
                        Segment(End(131, 1), End(132, -1)),
                        Segment(End(132, -1), End(133, 1)),
                        Segment(End(133, 1), End(134, -1)),
                        Segment(End(134, -1), End(135, 1)),
                        Segment(End(135, 1), End(137, -1)),
                        Segment(End(137, -1), End(138, 1)),
                        Segment(End(138, 1), End(139, -1)),
                        Segment(End(139, -1), End(140, 1)),
                        Segment(End(140, 1), End(141, -1)),
                        Segment(End(141, -1), End(142, 1)),
                        Segment(End(142, 1), End(143, -1)),
                        Segment(End(143, -1), End(145, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(126, 1), End(127, -1)), Segment(End(127, -1), End(129, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(0, Segment(End(23, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(117, 1), End(118, -1)), Segment(End(118, -1), End(119, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(83, 1), End(84, -1)), Segment(End(84, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(142, 1), End(143, -1)), Segment(End(143, -1), End(145, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(110, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(74, 1), End(75, -1)), Segment(End(75, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(25, -1)),
                            Tunnel(0, Segment(End(39, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(133, 1), End(134, -1)), Segment(End(134, -1), End(135, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(126, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(90, 1), End(91, -1)), Segment(End(91, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(41, -1)),
                            Tunnel(0, Segment(End(55, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(117, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(140, 1), End(141, -1)), Segment(End(141, -1), End(142, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(57, -1)),
                            Tunnel(0, Segment(End(71, 1), End(73, -1)), Segment(End(73, -1), End(74, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(131, 1), End(132, -1)), Segment(End(132, -1), End(133, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(99, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(73, -1)),
                            Tunnel(0, Segment(End(87, 1), End(89, -1)), Segment(End(89, -1), End(90, 1))),
                        ),
                        Bridge(
                            Segment(End(73, -1), End(74, 1)),
                            Segment(End(74, 1), End(75, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(75, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(84, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(84, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(0, Segment(End(138, 1), End(139, -1)), Segment(End(139, -1), End(140, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(89, -1)),
                            Tunnel(0, Segment(End(103, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(90, 1)),
                            Segment(End(90, 1), End(91, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(91, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(129, 1), End(130, -1)), Segment(End(130, -1), End(131, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(105, -1)),
                            Tunnel(0, Segment(End(119, 1), End(121, -1)), Segment(End(121, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(110, 1)),
                            Segment(End(110, 1), End(111, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(117, 1)),
                            Segment(End(117, 1), End(118, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(118, -1), End(119, 1)),
                            Segment(End(119, 1), End(121, -1)),
                            Tunnel(0, Segment(End(135, 1), End(137, -1)), Segment(End(137, -1), End(138, 1))),
                        ),
                        Bridge(
                            Segment(End(121, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(126, 1)),
                            Segment(End(126, 1), End(127, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(127, -1), End(129, 1)),
                            Segment(End(129, 1), End(130, -1)),
                            Tunnel(0, Segment(End(110, 1), End(111, -1)), Segment(End(111, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(130, -1), End(131, 1)),
                            Segment(End(131, 1), End(132, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(132, -1), End(133, 1)),
                            Segment(End(133, 1), End(134, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(134, -1), End(135, 1)),
                            Segment(End(135, 1), End(137, -1)),
                            Tunnel(0, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(137, -1), End(138, 1)),
                            Segment(End(138, 1), End(139, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(139, -1), End(140, 1)),
                            Segment(End(140, 1), End(141, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(141, -1), End(142, 1)),
                            Segment(End(142, 1), End(143, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_10_7(self):
        knot = Knot(10, 7)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(88, -1)),
                        Segment(End(88, -1), End(89, 1)),
                        Segment(End(89, 1), End(90, -1)),
                        Segment(End(90, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(96, 1)),
                        Segment(End(96, 1), End(97, -1)),
                        Segment(End(97, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(110, 1)),
                        Segment(End(110, 1), End(111, -1)),
                        Segment(End(111, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(117, 1)),
                        Segment(End(117, 1), End(118, -1)),
                        Segment(End(118, -1), End(120, 1)),
                        Segment(End(120, 1), End(121, -1)),
                        Segment(End(121, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(127, 1)),
                        Segment(End(127, 1), End(128, -1)),
                        Segment(End(128, -1), End(129, 1)),
                        Segment(End(129, 1), End(130, -1)),
                        Segment(End(130, -1), End(131, 1)),
                        Segment(End(131, 1), End(132, -1)),
                        Segment(End(132, -1), End(134, 1)),
                        Segment(End(134, 1), End(135, -1)),
                        Segment(End(135, -1), End(136, 1)),
                        Segment(End(136, 1), End(137, -1)),
                        Segment(End(137, -1), End(138, 1)),
                        Segment(End(138, 1), End(139, -1)),
                        Segment(End(139, -1), End(141, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(129, 1), End(130, -1)), Segment(End(130, -1), End(131, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(136, 1), End(137, -1)), Segment(End(137, -1), End(138, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(89, 1), End(90, -1)), Segment(End(90, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(96, 1), End(97, -1)), Segment(End(97, -1), End(99, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(120, 1), End(121, -1)), Segment(End(121, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(127, 1), End(128, -1)), Segment(End(128, -1), End(129, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(110, 1), End(111, -1)), Segment(End(111, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(134, 1), End(135, -1)), Segment(End(135, -1), End(136, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(117, 1), End(118, -1)), Segment(End(118, -1), End(120, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(127, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(88, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(88, -1), End(89, 1)),
                            Segment(End(89, 1), End(90, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(90, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(131, 1), End(132, -1)), Segment(End(132, -1), End(134, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(96, 1)),
                            Segment(End(96, 1), End(97, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(97, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(0, Segment(End(138, 1), End(139, -1)), Segment(End(139, -1), End(141, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(87, 1), End(88, -1)), Segment(End(88, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(110, 1)),
                            Segment(End(110, 1), End(111, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(96, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(117, 1)),
                            Segment(End(117, 1), End(118, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(118, -1), End(120, 1)),
                            Segment(End(120, 1), End(121, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(121, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(127, 1)),
                            Segment(End(127, 1), End(128, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(128, -1), End(129, 1)),
                            Segment(End(129, 1), End(130, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(110, 1))),
                        ),
                        Bridge(
                            Segment(End(130, -1), End(131, 1)),
                            Segment(End(131, 1), End(132, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(132, -1), End(134, 1)),
                            Segment(End(134, 1), End(135, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(135, -1), End(136, 1)),
                            Segment(End(136, 1), End(137, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(117, 1))),
                        ),
                        Bridge(
                            Segment(End(137, -1), End(138, 1)),
                            Segment(End(138, 1), End(139, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_11_6(self):
        knot = Knot(11, 6)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(65, 1)),
                        Segment(End(65, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(77, 1)),
                        Segment(End(77, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(88, -1)),
                        Segment(End(88, -1), End(89, 1)),
                        Segment(End(89, 1), End(91, -1)),
                        Segment(End(91, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(103, -1)),
                        Segment(End(103, -1), End(104, 1)),
                        Segment(End(104, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(109, 1)),
                        Segment(End(109, 1), End(110, -1)),
                        Segment(End(110, -1), End(111, 1)),
                        Segment(End(111, 1), End(112, -1)),
                        Segment(End(112, -1), End(113, 1)),
                        Segment(End(113, 1), End(115, -1)),
                        Segment(End(115, -1), End(116, 1)),
                        Segment(End(116, 1), End(117, -1)),
                        Segment(End(117, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(121, 1)),
                        Segment(End(121, 1), End(122, -1)),
                        Segment(End(122, -1), End(123, 1)),
                        Segment(End(123, 1), End(124, -1)),
                        Segment(End(124, -1), End(125, 1)),
                        Segment(End(125, 1), End(127, -1)),
                        Segment(End(127, -1), End(128, 1)),
                        Segment(End(128, 1), End(129, -1)),
                        Segment(End(129, -1), End(130, 1)),
                        Segment(End(130, 1), End(131, -1)),
                        Segment(End(131, -1), End(133, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(0, Segment(End(113, 1), End(115, -1)), Segment(End(115, -1), End(116, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(99, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(0, Segment(End(125, 1), End(127, -1)), Segment(End(127, -1), End(128, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(109, 1), End(110, -1)), Segment(End(110, -1), End(111, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(0, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(77, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(121, 1), End(122, -1)), Segment(End(122, -1), End(123, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(104, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(43, -1)),
                            Tunnel(0, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(87, 1), End(88, -1)), Segment(End(88, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(116, 1), End(117, -1)), Segment(End(117, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(55, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(0, Segment(End(128, 1), End(129, -1)), Segment(End(129, -1), End(130, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(65, 1)),
                            Segment(End(65, 1), End(67, -1)),
                            Tunnel(0, Segment(End(41, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(111, 1), End(112, -1)), Segment(End(112, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(77, 1)),
                            Segment(End(77, 1), End(79, -1)),
                            Tunnel(0, Segment(End(53, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(123, 1), End(124, -1)), Segment(End(124, -1), End(125, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(109, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(88, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(88, -1), End(89, 1)),
                            Segment(End(89, 1), End(91, -1)),
                            Tunnel(0, Segment(End(65, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(91, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(121, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(103, -1)),
                            Tunnel(0, Segment(End(77, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(103, -1), End(104, 1)),
                            Segment(End(104, 1), End(105, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(109, 1)),
                            Segment(End(109, 1), End(110, -1)),
                            Tunnel(0, Segment(End(130, 1), End(131, -1)), Segment(End(131, -1), End(133, 1))),
                        ),
                        Bridge(
                            Segment(End(110, -1), End(111, 1)),
                            Segment(End(111, 1), End(112, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(112, -1), End(113, 1)),
                            Segment(End(113, 1), End(115, -1)),
                            Tunnel(0, Segment(End(89, 1), End(91, -1)), Segment(End(91, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(115, -1), End(116, 1)),
                            Segment(End(116, 1), End(117, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(117, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(119, -1), End(121, 1)),
                            Segment(End(121, 1), End(122, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(122, -1), End(123, 1)),
                            Segment(End(123, 1), End(124, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(124, -1), End(125, 1)),
                            Segment(End(125, 1), End(127, -1)),
                            Tunnel(0, Segment(End(101, 1), End(103, -1)), Segment(End(103, -1), End(104, 1))),
                        ),
                        Bridge(
                            Segment(End(127, -1), End(128, 1)),
                            Segment(End(128, 1), End(129, -1)),
                            Tunnel(0, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(129, -1), End(130, 1)),
                            Segment(End(130, 1), End(131, -1)),
                            Tunnel(0, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_12_5(self):
        knot = Knot(12, 5)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(84, -1)),
                        Segment(End(84, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(88, 1)),
                        Segment(End(88, 1), End(89, -1)),
                        Segment(End(89, -1), End(91, 1)),
                        Segment(End(91, 1), End(92, -1)),
                        Segment(End(92, -1), End(93, 1)),
                        Segment(End(93, 1), End(94, -1)),
                        Segment(End(94, -1), End(96, 1)),
                        Segment(End(96, 1), End(97, -1)),
                        Segment(End(97, -1), End(98, 1)),
                        Segment(End(98, 1), End(99, -1)),
                        Segment(End(99, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(111, 1)),
                        Segment(End(111, 1), End(112, -1)),
                        Segment(End(112, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(116, 1)),
                        Segment(End(116, 1), End(117, -1)),
                        Segment(End(117, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(121, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(83, 1), End(84, -1)), Segment(End(84, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(88, 1), End(89, -1)), Segment(End(89, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(93, 1), End(94, -1)), Segment(End(94, -1), End(96, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(98, 1), End(99, -1)), Segment(End(99, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(111, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(88, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(116, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(91, 1), End(92, -1)), Segment(End(92, -1), End(93, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(121, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(96, 1), End(97, -1)), Segment(End(97, -1), End(98, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(84, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(84, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(88, 1)),
                            Segment(End(88, 1), End(89, -1)),
                            Tunnel(0, Segment(End(111, 1), End(112, -1)), Segment(End(112, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(91, 1)),
                            Segment(End(91, 1), End(92, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(92, -1), End(93, 1)),
                            Segment(End(93, 1), End(94, -1)),
                            Tunnel(0, Segment(End(116, 1), End(117, -1)), Segment(End(117, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(94, -1), End(96, 1)),
                            Segment(End(96, 1), End(97, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(97, -1), End(98, 1)),
                            Segment(End(98, 1), End(99, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(99, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(111, 1)),
                            Segment(End(111, 1), End(112, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(112, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(116, 1)),
                            Segment(End(116, 1), End(117, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(117, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_6_12(self):
        knot = Knot(6, 12)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(5, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(3, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(5, Segment(End(21, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(4, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(4, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(5, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(4, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(4, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(3, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(5, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(4, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(5, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(3, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(5, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(17, -1)),
                            Tunnel(1, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(4, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(3, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(5, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(4, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(2, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(4, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(2, Segment(End(15, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(5, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                    ],
                ),
                String(
                    4,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(4, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(5, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(1, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(3, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(5, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(3, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                    ],
                ),
                String(
                    5,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(3, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(5, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(4, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(2, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(23, -1)),
                            Tunnel(4, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_7_11(self):
        knot = Knot(7, 11)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(84, 1)),
                        Segment(End(84, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(89, 1)),
                        Segment(End(89, 1), End(90, -1)),
                        Segment(End(90, -1), End(91, 1)),
                        Segment(End(91, 1), End(92, -1)),
                        Segment(End(92, -1), End(93, 1)),
                        Segment(End(93, 1), End(94, -1)),
                        Segment(End(94, -1), End(95, 1)),
                        Segment(End(95, 1), End(96, -1)),
                        Segment(End(96, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(100, 1)),
                        Segment(End(100, 1), End(101, -1)),
                        Segment(End(101, -1), End(102, 1)),
                        Segment(End(102, 1), End(103, -1)),
                        Segment(End(103, -1), End(104, 1)),
                        Segment(End(104, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(111, 1)),
                        Segment(End(111, 1), End(112, -1)),
                        Segment(End(112, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(117, 1)),
                        Segment(End(117, 1), End(118, -1)),
                        Segment(End(118, -1), End(119, 1)),
                        Segment(End(119, 1), End(120, -1)),
                        Segment(End(120, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(126, 1)),
                        Segment(End(126, 1), End(127, -1)),
                        Segment(End(127, -1), End(128, 1)),
                        Segment(End(128, 1), End(129, -1)),
                        Segment(End(129, -1), End(130, 1)),
                        Segment(End(130, 1), End(131, -1)),
                        Segment(End(131, -1), End(133, 1)),
                        Segment(End(133, 1), End(134, -1)),
                        Segment(End(134, -1), End(135, 1)),
                        Segment(End(135, 1), End(136, -1)),
                        Segment(End(136, -1), End(137, 1)),
                        Segment(End(137, 1), End(138, -1)),
                        Segment(End(138, -1), End(139, 1)),
                        Segment(End(139, 1), End(140, -1)),
                        Segment(End(140, -1), End(141, 1)),
                        Segment(End(141, 1), End(142, -1)),
                        Segment(End(142, -1), End(144, 1)),
                        Segment(End(144, 1), End(145, -1)),
                        Segment(End(145, -1), End(146, 1)),
                        Segment(End(146, 1), End(147, -1)),
                        Segment(End(147, -1), End(148, 1)),
                        Segment(End(148, 1), End(149, -1)),
                        Segment(End(149, -1), End(150, 1)),
                        Segment(End(150, 1), End(151, -1)),
                        Segment(End(151, -1), End(152, 1)),
                        Segment(End(152, 1), End(153, -1)),
                        Segment(End(153, -1), End(155, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(128, 1), End(129, -1)), Segment(End(129, -1), End(130, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(146, 1), End(147, -1)), Segment(End(147, -1), End(148, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(139, 1), End(140, -1)), Segment(End(140, -1), End(141, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(89, 1), End(90, -1)), Segment(End(90, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(150, 1), End(151, -1)), Segment(End(151, -1), End(152, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(84, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(100, 1), End(101, -1)), Segment(End(101, -1), End(102, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(93, 1), End(94, -1)), Segment(End(94, -1), End(95, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(111, 1), End(112, -1)), Segment(End(112, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(104, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(100, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(117, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(133, 1), End(134, -1)), Segment(End(134, -1), End(135, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(111, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(126, 1), End(127, -1)), Segment(End(127, -1), End(128, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(144, 1), End(145, -1)), Segment(End(145, -1), End(146, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(119, 1), End(120, -1)), Segment(End(120, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(137, 1), End(138, -1)), Segment(End(138, -1), End(139, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(84, 1)),
                            Segment(End(84, 1), End(85, -1)),
                            Tunnel(0, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(89, 1)),
                            Segment(End(89, 1), End(90, -1)),
                            Tunnel(0, Segment(End(130, 1), End(131, -1)), Segment(End(131, -1), End(133, 1))),
                        ),
                        Bridge(
                            Segment(End(90, -1), End(91, 1)),
                            Segment(End(91, 1), End(92, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(92, -1), End(93, 1)),
                            Segment(End(93, 1), End(94, -1)),
                            Tunnel(0, Segment(End(148, 1), End(149, -1)), Segment(End(149, -1), End(150, 1))),
                        ),
                        Bridge(
                            Segment(End(94, -1), End(95, 1)),
                            Segment(End(95, 1), End(96, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(96, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(100, 1)),
                            Segment(End(100, 1), End(101, -1)),
                            Tunnel(0, Segment(End(141, 1), End(142, -1)), Segment(End(142, -1), End(144, 1))),
                        ),
                        Bridge(
                            Segment(End(101, -1), End(102, 1)),
                            Segment(End(102, 1), End(103, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(103, -1), End(104, 1)),
                            Segment(End(104, 1), End(105, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(91, 1), End(92, -1)), Segment(End(92, -1), End(93, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(111, 1)),
                            Segment(End(111, 1), End(112, -1)),
                            Tunnel(0, Segment(End(152, 1), End(153, -1)), Segment(End(153, -1), End(155, 1))),
                        ),
                        Bridge(
                            Segment(End(112, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(84, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(117, 1)),
                            Segment(End(117, 1), End(118, -1)),
                            Tunnel(0, Segment(End(102, 1), End(103, -1)), Segment(End(103, -1), End(104, 1))),
                        ),
                        Bridge(
                            Segment(End(118, -1), End(119, 1)),
                            Segment(End(119, 1), End(120, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(120, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(95, 1), End(96, -1)), Segment(End(96, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(126, 1)),
                            Segment(End(126, 1), End(127, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(127, -1), End(128, 1)),
                            Segment(End(128, 1), End(129, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(129, -1), End(130, 1)),
                            Segment(End(130, 1), End(131, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(131, -1), End(133, 1)),
                            Segment(End(133, 1), End(134, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(134, -1), End(135, 1)),
                            Segment(End(135, 1), End(136, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(136, -1), End(137, 1)),
                            Segment(End(137, 1), End(138, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(138, -1), End(139, 1)),
                            Segment(End(139, 1), End(140, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(126, 1))),
                        ),
                        Bridge(
                            Segment(End(140, -1), End(141, 1)),
                            Segment(End(141, 1), End(142, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(142, -1), End(144, 1)),
                            Segment(End(144, 1), End(145, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(145, -1), End(146, 1)),
                            Segment(End(146, 1), End(147, -1)),
                            Tunnel(0, Segment(End(117, 1), End(118, -1)), Segment(End(118, -1), End(119, 1))),
                        ),
                        Bridge(
                            Segment(End(147, -1), End(148, 1)),
                            Segment(End(148, 1), End(149, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(149, -1), End(150, 1)),
                            Segment(End(150, 1), End(151, -1)),
                            Tunnel(0, Segment(End(135, 1), End(136, -1)), Segment(End(136, -1), End(137, 1))),
                        ),
                        Bridge(
                            Segment(End(151, -1), End(152, 1)),
                            Segment(End(152, 1), End(153, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_8_10(self):
        knot = Knot(8, 10)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(71, -1)),
                        Segment(End(71, -1), End(72, 1)),
                        Segment(End(72, 1), End(73, -1)),
                        Segment(End(73, -1), End(74, 1)),
                        Segment(End(74, 1), End(75, -1)),
                        Segment(End(75, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(1, Segment(End(71, 1), End(73, -1)), Segment(End(73, -1), End(74, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(1, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(1, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(1, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(1, Segment(End(74, 1), End(75, -1)), Segment(End(75, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(1, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(1, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(1, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(1, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(51, -1)),
                            Tunnel(1, Segment(End(31, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(1, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(1, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(1, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(71, -1)),
                            Tunnel(1, Segment(End(51, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(72, 1)),
                            Segment(End(72, 1), End(73, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(73, -1), End(74, 1)),
                            Segment(End(74, 1), End(75, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(75, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(73, -1)),
                        Segment(End(73, -1), End(74, 1)),
                        Segment(End(74, 1), End(75, -1)),
                        Segment(End(75, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(72, 1), End(73, -1)), Segment(End(73, -1), End(74, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(1, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(1, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(74, 1), End(75, -1)), Segment(End(75, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(33, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(1, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(1, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(1, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(53, -1)),
                            Tunnel(0, Segment(End(49, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(1, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(1, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(1, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(73, -1)),
                            Tunnel(0, Segment(End(69, 1), End(71, -1)), Segment(End(71, -1), End(72, 1))),
                        ),
                        Bridge(
                            Segment(End(73, -1), End(74, 1)),
                            Segment(End(74, 1), End(75, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(75, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(1, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(1, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_9_9(self):
        knot = Knot(9, 9)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(5, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(7, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(3, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(5, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(7, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(8, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(4, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(6, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(8, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(4, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(6, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(7, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(5, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(7, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(3, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(5, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(6, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(8, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(4, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(6, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(8, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(4, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(18, 1))),
                        ),
                    ],
                ),
                String(
                    4,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(18, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-2, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(5, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(7, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(5, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(7, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(2, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                    ],
                ),
                String(
                    5,
                    [
                        Segment(End(0, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(6, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(8, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(6, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(8, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(3, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                    ],
                ),
                String(
                    6,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(4, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(7, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(4, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(7, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(2, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                    ],
                ),
                String(
                    7,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(3, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(5, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(8, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(5, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(8, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                    ],
                ),
                String(
                    8,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(4, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(6, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(4, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(6, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_10_8(self):
        knot = Knot(10, 8)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(73, -1)),
                        Segment(End(73, -1), End(74, 1)),
                        Segment(End(74, 1), End(75, -1)),
                        Segment(End(75, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(1, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(1, Segment(End(25, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(1, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(1, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(25, -1)),
                            Tunnel(1, Segment(End(41, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(1, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(1, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(41, -1)),
                            Tunnel(1, Segment(End(57, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(1, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(1, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(57, -1)),
                            Tunnel(1, Segment(End(73, 1), End(75, -1)), Segment(End(75, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(1, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(1, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(1, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(73, -1)),
                            Tunnel(1, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(73, -1), End(74, 1)),
                            Segment(End(74, 1), End(75, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(75, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(75, -1)),
                        Segment(End(75, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(0, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(1, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(1, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(27, -1)),
                            Tunnel(0, Segment(End(23, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(1, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(1, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(43, -1)),
                            Tunnel(0, Segment(End(39, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(1, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(1, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(74, 1), End(75, -1)), Segment(End(75, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(59, -1)),
                            Tunnel(0, Segment(End(55, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(1, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(1, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(75, -1)),
                            Tunnel(0, Segment(End(71, 1), End(73, -1)), Segment(End(73, -1), End(74, 1))),
                        ),
                        Bridge(
                            Segment(End(75, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(1, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(1, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_11_7(self):
        knot = Knot(11, 7)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(88, -1)),
                        Segment(End(88, -1), End(89, 1)),
                        Segment(End(89, 1), End(90, -1)),
                        Segment(End(90, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(96, 1)),
                        Segment(End(96, 1), End(97, -1)),
                        Segment(End(97, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(110, 1)),
                        Segment(End(110, 1), End(111, -1)),
                        Segment(End(111, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(117, 1)),
                        Segment(End(117, 1), End(118, -1)),
                        Segment(End(118, -1), End(120, 1)),
                        Segment(End(120, 1), End(121, -1)),
                        Segment(End(121, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(127, 1)),
                        Segment(End(127, 1), End(128, -1)),
                        Segment(End(128, -1), End(129, 1)),
                        Segment(End(129, 1), End(130, -1)),
                        Segment(End(130, -1), End(131, 1)),
                        Segment(End(131, 1), End(132, -1)),
                        Segment(End(132, -1), End(134, 1)),
                        Segment(End(134, 1), End(135, -1)),
                        Segment(End(135, -1), End(136, 1)),
                        Segment(End(136, 1), End(137, -1)),
                        Segment(End(137, -1), End(138, 1)),
                        Segment(End(138, 1), End(139, -1)),
                        Segment(End(139, -1), End(141, 1)),
                        Segment(End(141, 1), End(142, -1)),
                        Segment(End(142, -1), End(143, 1)),
                        Segment(End(143, 1), End(144, -1)),
                        Segment(End(144, -1), End(145, 1)),
                        Segment(End(145, 1), End(146, -1)),
                        Segment(End(146, -1), End(148, 1)),
                        Segment(End(148, 1), End(149, -1)),
                        Segment(End(149, -1), End(150, 1)),
                        Segment(End(150, 1), End(151, -1)),
                        Segment(End(151, -1), End(152, 1)),
                        Segment(End(152, 1), End(153, -1)),
                        Segment(End(153, -1), End(155, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(110, 1), End(111, -1)), Segment(End(111, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(117, 1), End(118, -1)), Segment(End(118, -1), End(120, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(127, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(131, 1), End(132, -1)), Segment(End(132, -1), End(134, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(138, 1), End(139, -1)), Segment(End(139, -1), End(141, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(120, 1), End(121, -1)), Segment(End(121, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(145, 1), End(146, -1)), Segment(End(146, -1), End(148, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(127, 1), End(128, -1)), Segment(End(128, -1), End(129, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(152, 1), End(153, -1)), Segment(End(153, -1), End(155, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(134, 1), End(135, -1)), Segment(End(135, -1), End(136, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(141, 1), End(142, -1)), Segment(End(142, -1), End(143, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(148, 1), End(149, -1)), Segment(End(149, -1), End(150, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(87, 1), End(88, -1)), Segment(End(88, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(96, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(88, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(110, 1))),
                        ),
                        Bridge(
                            Segment(End(88, -1), End(89, 1)),
                            Segment(End(89, 1), End(90, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(90, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(117, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(96, 1)),
                            Segment(End(96, 1), End(97, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(97, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(129, 1), End(130, -1)), Segment(End(130, -1), End(131, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(110, 1)),
                            Segment(End(110, 1), End(111, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(136, 1), End(137, -1)), Segment(End(137, -1), End(138, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(117, 1)),
                            Segment(End(117, 1), End(118, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(118, -1), End(120, 1)),
                            Segment(End(120, 1), End(121, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(121, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(143, 1), End(144, -1)), Segment(End(144, -1), End(145, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(127, 1)),
                            Segment(End(127, 1), End(128, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(128, -1), End(129, 1)),
                            Segment(End(129, 1), End(130, -1)),
                            Tunnel(0, Segment(End(150, 1), End(151, -1)), Segment(End(151, -1), End(152, 1))),
                        ),
                        Bridge(
                            Segment(End(130, -1), End(131, 1)),
                            Segment(End(131, 1), End(132, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(132, -1), End(134, 1)),
                            Segment(End(134, 1), End(135, -1)),
                            Tunnel(0, Segment(End(89, 1), End(90, -1)), Segment(End(90, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(135, -1), End(136, 1)),
                            Segment(End(136, 1), End(137, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(137, -1), End(138, 1)),
                            Segment(End(138, 1), End(139, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(139, -1), End(141, 1)),
                            Segment(End(141, 1), End(142, -1)),
                            Tunnel(0, Segment(End(96, 1), End(97, -1)), Segment(End(97, -1), End(99, 1))),
                        ),
                        Bridge(
                            Segment(End(142, -1), End(143, 1)),
                            Segment(End(143, 1), End(144, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(144, -1), End(145, 1)),
                            Segment(End(145, 1), End(146, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(146, -1), End(148, 1)),
                            Segment(End(148, 1), End(149, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(149, -1), End(150, 1)),
                            Segment(End(150, 1), End(151, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(151, -1), End(152, 1)),
                            Segment(End(152, 1), End(153, -1)),
                            Tunnel(0, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_12_6(self):
        knot = Knot(12, 6)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(5, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(4, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(3, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(5, Segment(End(15, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(2, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(4, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(5, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(4, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(0, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(5, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(2, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(4, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(0, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(3, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(4, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(5, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(1, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(4, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(3, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(5, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(23, -1)),
                            Tunnel(1, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(25, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(5, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(4, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(2, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(5, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(4, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(25, -1)),
                            Tunnel(2, Segment(End(21, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                    ],
                ),
                String(
                    4,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(3, -1)),
                            Tunnel(3, Segment(End(-1, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(5, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(1, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(3, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(2, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(5, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(1, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                    ],
                ),
                String(
                    5,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(4, Segment(End(1, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(2, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(17, -1)),
                            Tunnel(4, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(1, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(3, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_7_12(self):
        knot = Knot(7, 12)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(77, 1)),
                        Segment(End(77, 1), End(78, -1)),
                        Segment(End(78, -1), End(79, 1)),
                        Segment(End(79, 1), End(80, -1)),
                        Segment(End(80, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(88, 1)),
                        Segment(End(88, 1), End(89, -1)),
                        Segment(End(89, -1), End(90, 1)),
                        Segment(End(90, 1), End(91, -1)),
                        Segment(End(91, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(105, 1)),
                        Segment(End(105, 1), End(106, -1)),
                        Segment(End(106, -1), End(107, 1)),
                        Segment(End(107, 1), End(109, -1)),
                        Segment(End(109, -1), End(110, 1)),
                        Segment(End(110, 1), End(111, -1)),
                        Segment(End(111, -1), End(112, 1)),
                        Segment(End(112, 1), End(113, -1)),
                        Segment(End(113, -1), End(114, 1)),
                        Segment(End(114, 1), End(115, -1)),
                        Segment(End(115, -1), End(116, 1)),
                        Segment(End(116, 1), End(117, -1)),
                        Segment(End(117, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(121, 1)),
                        Segment(End(121, 1), End(122, -1)),
                        Segment(End(122, -1), End(123, 1)),
                        Segment(End(123, 1), End(124, -1)),
                        Segment(End(124, -1), End(125, 1)),
                        Segment(End(125, 1), End(126, -1)),
                        Segment(End(126, -1), End(127, 1)),
                        Segment(End(127, 1), End(128, -1)),
                        Segment(End(128, -1), End(129, 1)),
                        Segment(End(129, 1), End(130, -1)),
                        Segment(End(130, -1), End(131, 1)),
                        Segment(End(131, 1), End(133, -1)),
                        Segment(End(133, -1), End(134, 1)),
                        Segment(End(134, 1), End(135, -1)),
                        Segment(End(135, -1), End(136, 1)),
                        Segment(End(136, 1), End(137, -1)),
                        Segment(End(137, -1), End(138, 1)),
                        Segment(End(138, 1), End(139, -1)),
                        Segment(End(139, -1), End(140, 1)),
                        Segment(End(140, 1), End(141, -1)),
                        Segment(End(141, -1), End(142, 1)),
                        Segment(End(142, 1), End(143, -1)),
                        Segment(End(143, -1), End(145, 1)),
                        Segment(End(145, 1), End(146, -1)),
                        Segment(End(146, -1), End(147, 1)),
                        Segment(End(147, 1), End(148, -1)),
                        Segment(End(148, -1), End(149, 1)),
                        Segment(End(149, 1), End(150, -1)),
                        Segment(End(150, -1), End(151, 1)),
                        Segment(End(151, 1), End(152, -1)),
                        Segment(End(152, -1), End(153, 1)),
                        Segment(End(153, 1), End(154, -1)),
                        Segment(End(154, -1), End(155, 1)),
                        Segment(End(155, 1), End(157, -1)),
                        Segment(End(157, -1), End(158, 1)),
                        Segment(End(158, 1), End(159, -1)),
                        Segment(End(159, -1), End(160, 1)),
                        Segment(End(160, 1), End(161, -1)),
                        Segment(End(161, -1), End(162, 1)),
                        Segment(End(162, 1), End(163, -1)),
                        Segment(End(163, -1), End(164, 1)),
                        Segment(End(164, 1), End(165, -1)),
                        Segment(End(165, -1), End(166, 1)),
                        Segment(End(166, 1), End(167, -1)),
                        Segment(End(167, -1), End(169, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(160, 1), End(161, -1)), Segment(End(161, -1), End(162, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(134, 1), End(135, -1)), Segment(End(135, -1), End(136, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(107, 1), End(109, -1)), Segment(End(109, -1), End(110, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(153, 1), End(154, -1)), Segment(End(154, -1), End(155, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(127, 1), End(128, -1)), Segment(End(128, -1), End(129, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(77, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(158, 1), End(159, -1)), Segment(End(159, -1), End(160, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(37, -1)),
                            Tunnel(0, Segment(End(131, 1), End(133, -1)), Segment(End(133, -1), End(134, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(151, 1), End(152, -1)), Segment(End(152, -1), End(153, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(125, 1), End(126, -1)), Segment(End(126, -1), End(127, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(121, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(61, -1)),
                            Tunnel(0, Segment(End(155, 1), End(157, -1)), Segment(End(157, -1), End(158, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(149, 1), End(150, -1)), Segment(End(150, -1), End(151, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(123, 1), End(124, -1)), Segment(End(124, -1), End(125, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(99, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(142, 1), End(143, -1)), Segment(End(143, -1), End(145, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(116, 1), End(117, -1)), Segment(End(117, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(77, 1)),
                            Segment(End(77, 1), End(78, -1)),
                            Tunnel(0, Segment(End(90, 1), End(91, -1)), Segment(End(91, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(78, -1), End(79, 1)),
                            Segment(End(79, 1), End(80, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(80, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(85, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(88, 1)),
                            Segment(End(88, 1), End(89, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(90, 1)),
                            Segment(End(90, 1), End(91, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(91, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(147, 1), End(148, -1)), Segment(End(148, -1), End(149, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(121, 1), End(122, -1)), Segment(End(122, -1), End(123, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(166, 1), End(167, -1)), Segment(End(167, -1), End(169, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(0, Segment(End(140, 1), End(141, -1)), Segment(End(141, -1), End(142, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(114, 1), End(115, -1)), Segment(End(115, -1), End(116, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(88, 1), End(89, -1)), Segment(End(89, -1), End(90, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(105, 1)),
                            Segment(End(105, 1), End(106, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(106, -1), End(107, 1)),
                            Segment(End(107, 1), End(109, -1)),
                            Tunnel(0, Segment(End(35, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(110, 1)),
                            Segment(End(110, 1), End(111, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(112, 1)),
                            Segment(End(112, 1), End(113, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(113, -1), End(114, 1)),
                            Segment(End(114, 1), End(115, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(115, -1), End(116, 1)),
                            Segment(End(116, 1), End(117, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(117, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(145, 1), End(146, -1)), Segment(End(146, -1), End(147, 1))),
                        ),
                        Bridge(
                            Segment(End(119, -1), End(121, 1)),
                            Segment(End(121, 1), End(122, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(122, -1), End(123, 1)),
                            Segment(End(123, 1), End(124, -1)),
                            Tunnel(0, Segment(End(164, 1), End(165, -1)), Segment(End(165, -1), End(166, 1))),
                        ),
                        Bridge(
                            Segment(End(124, -1), End(125, 1)),
                            Segment(End(125, 1), End(126, -1)),
                            Tunnel(0, Segment(End(138, 1), End(139, -1)), Segment(End(139, -1), End(140, 1))),
                        ),
                        Bridge(
                            Segment(End(126, -1), End(127, 1)),
                            Segment(End(127, 1), End(128, -1)),
                            Tunnel(0, Segment(End(112, 1), End(113, -1)), Segment(End(113, -1), End(114, 1))),
                        ),
                        Bridge(
                            Segment(End(128, -1), End(129, 1)),
                            Segment(End(129, 1), End(130, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(88, 1))),
                        ),
                        Bridge(
                            Segment(End(130, -1), End(131, 1)),
                            Segment(End(131, 1), End(133, -1)),
                            Tunnel(0, Segment(End(59, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(133, -1), End(134, 1)),
                            Segment(End(134, 1), End(135, -1)),
                            Tunnel(0, Segment(End(105, 1), End(106, -1)), Segment(End(106, -1), End(107, 1))),
                        ),
                        Bridge(
                            Segment(End(135, -1), End(136, 1)),
                            Segment(End(136, 1), End(137, -1)),
                            Tunnel(0, Segment(End(79, 1), End(80, -1)), Segment(End(80, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(137, -1), End(138, 1)),
                            Segment(End(138, 1), End(139, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(139, -1), End(140, 1)),
                            Segment(End(140, 1), End(141, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(141, -1), End(142, 1)),
                            Segment(End(142, 1), End(143, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(143, -1), End(145, 1)),
                            Segment(End(145, 1), End(146, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(146, -1), End(147, 1)),
                            Segment(End(147, 1), End(148, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(148, -1), End(149, 1)),
                            Segment(End(149, 1), End(150, -1)),
                            Tunnel(0, Segment(End(162, 1), End(163, -1)), Segment(End(163, -1), End(164, 1))),
                        ),
                        Bridge(
                            Segment(End(150, -1), End(151, 1)),
                            Segment(End(151, 1), End(152, -1)),
                            Tunnel(0, Segment(End(136, 1), End(137, -1)), Segment(End(137, -1), End(138, 1))),
                        ),
                        Bridge(
                            Segment(End(152, -1), End(153, 1)),
                            Segment(End(153, 1), End(154, -1)),
                            Tunnel(0, Segment(End(110, 1), End(111, -1)), Segment(End(111, -1), End(112, 1))),
                        ),
                        Bridge(
                            Segment(End(154, -1), End(155, 1)),
                            Segment(End(155, 1), End(157, -1)),
                            Tunnel(0, Segment(End(83, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(157, -1), End(158, 1)),
                            Segment(End(158, 1), End(159, -1)),
                            Tunnel(0, Segment(End(129, 1), End(130, -1)), Segment(End(130, -1), End(131, 1))),
                        ),
                        Bridge(
                            Segment(End(159, -1), End(160, 1)),
                            Segment(End(160, 1), End(161, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(105, 1))),
                        ),
                        Bridge(
                            Segment(End(161, -1), End(162, 1)),
                            Segment(End(162, 1), End(163, -1)),
                            Tunnel(0, Segment(End(77, 1), End(78, -1)), Segment(End(78, -1), End(79, 1))),
                        ),
                        Bridge(
                            Segment(End(163, -1), End(164, 1)),
                            Segment(End(164, 1), End(165, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(165, -1), End(166, 1)),
                            Segment(End(166, 1), End(167, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_8_11(self):
        knot = Knot(8, 11)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(84, 1)),
                        Segment(End(84, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(89, 1)),
                        Segment(End(89, 1), End(90, -1)),
                        Segment(End(90, -1), End(91, 1)),
                        Segment(End(91, 1), End(92, -1)),
                        Segment(End(92, -1), End(93, 1)),
                        Segment(End(93, 1), End(94, -1)),
                        Segment(End(94, -1), End(95, 1)),
                        Segment(End(95, 1), End(96, -1)),
                        Segment(End(96, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(100, 1)),
                        Segment(End(100, 1), End(101, -1)),
                        Segment(End(101, -1), End(102, 1)),
                        Segment(End(102, 1), End(103, -1)),
                        Segment(End(103, -1), End(104, 1)),
                        Segment(End(104, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(111, 1)),
                        Segment(End(111, 1), End(112, -1)),
                        Segment(End(112, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(117, 1)),
                        Segment(End(117, 1), End(118, -1)),
                        Segment(End(118, -1), End(119, 1)),
                        Segment(End(119, 1), End(120, -1)),
                        Segment(End(120, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(126, 1)),
                        Segment(End(126, 1), End(127, -1)),
                        Segment(End(127, -1), End(128, 1)),
                        Segment(End(128, 1), End(129, -1)),
                        Segment(End(129, -1), End(130, 1)),
                        Segment(End(130, 1), End(131, -1)),
                        Segment(End(131, -1), End(133, 1)),
                        Segment(End(133, 1), End(134, -1)),
                        Segment(End(134, -1), End(135, 1)),
                        Segment(End(135, 1), End(136, -1)),
                        Segment(End(136, -1), End(137, 1)),
                        Segment(End(137, 1), End(138, -1)),
                        Segment(End(138, -1), End(139, 1)),
                        Segment(End(139, 1), End(140, -1)),
                        Segment(End(140, -1), End(141, 1)),
                        Segment(End(141, 1), End(142, -1)),
                        Segment(End(142, -1), End(144, 1)),
                        Segment(End(144, 1), End(145, -1)),
                        Segment(End(145, -1), End(146, 1)),
                        Segment(End(146, 1), End(147, -1)),
                        Segment(End(147, -1), End(148, 1)),
                        Segment(End(148, 1), End(149, -1)),
                        Segment(End(149, -1), End(150, 1)),
                        Segment(End(150, 1), End(151, -1)),
                        Segment(End(151, -1), End(152, 1)),
                        Segment(End(152, 1), End(153, -1)),
                        Segment(End(153, -1), End(155, 1)),
                        Segment(End(155, 1), End(156, -1)),
                        Segment(End(156, -1), End(157, 1)),
                        Segment(End(157, 1), End(158, -1)),
                        Segment(End(158, -1), End(159, 1)),
                        Segment(End(159, 1), End(160, -1)),
                        Segment(End(160, -1), End(161, 1)),
                        Segment(End(161, 1), End(162, -1)),
                        Segment(End(162, -1), End(163, 1)),
                        Segment(End(163, 1), End(164, -1)),
                        Segment(End(164, -1), End(166, 1)),
                        Segment(End(166, 1), End(167, -1)),
                        Segment(End(167, -1), End(168, 1)),
                        Segment(End(168, 1), End(169, -1)),
                        Segment(End(169, -1), End(170, 1)),
                        Segment(End(170, 1), End(171, -1)),
                        Segment(End(171, -1), End(172, 1)),
                        Segment(End(172, 1), End(173, -1)),
                        Segment(End(173, -1), End(174, 1)),
                        Segment(End(174, 1), End(175, -1)),
                        Segment(End(175, -1), End(177, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(148, 1), End(149, -1)), Segment(End(149, -1), End(150, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(102, 1), End(103, -1)), Segment(End(103, -1), End(104, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(159, 1), End(160, -1)), Segment(End(160, -1), End(161, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(170, 1), End(171, -1)), Segment(End(171, -1), End(172, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(126, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(100, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(135, 1), End(136, -1)), Segment(End(136, -1), End(137, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(89, 1), End(90, -1)), Segment(End(90, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(111, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(146, 1), End(147, -1)), Segment(End(147, -1), End(148, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(100, 1), End(101, -1)), Segment(End(101, -1), End(102, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(119, 1), End(120, -1)), Segment(End(120, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(157, 1), End(158, -1)), Segment(End(158, -1), End(159, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(111, 1), End(112, -1)), Segment(End(112, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(130, 1), End(131, -1)), Segment(End(131, -1), End(133, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(84, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(168, 1), End(169, -1)), Segment(End(169, -1), End(170, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(141, 1), End(142, -1)), Segment(End(142, -1), End(144, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(95, 1), End(96, -1)), Segment(End(96, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(84, 1)),
                            Segment(End(84, 1), End(85, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(133, 1), End(134, -1)), Segment(End(134, -1), End(135, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(89, 1)),
                            Segment(End(89, 1), End(90, -1)),
                            Tunnel(0, Segment(End(152, 1), End(153, -1)), Segment(End(153, -1), End(155, 1))),
                        ),
                        Bridge(
                            Segment(End(90, -1), End(91, 1)),
                            Segment(End(91, 1), End(92, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(92, -1), End(93, 1)),
                            Segment(End(93, 1), End(94, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(94, -1), End(95, 1)),
                            Segment(End(95, 1), End(96, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(96, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(144, 1), End(145, -1)), Segment(End(145, -1), End(146, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(100, 1)),
                            Segment(End(100, 1), End(101, -1)),
                            Tunnel(0, Segment(End(163, 1), End(164, -1)), Segment(End(164, -1), End(166, 1))),
                        ),
                        Bridge(
                            Segment(End(101, -1), End(102, 1)),
                            Segment(End(102, 1), End(103, -1)),
                            Tunnel(0, Segment(End(117, 1), End(118, -1)), Segment(End(118, -1), End(119, 1))),
                        ),
                        Bridge(
                            Segment(End(103, -1), End(104, 1)),
                            Segment(End(104, 1), End(105, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(155, 1), End(156, -1)), Segment(End(156, -1), End(157, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(111, 1)),
                            Segment(End(111, 1), End(112, -1)),
                            Tunnel(0, Segment(End(174, 1), End(175, -1)), Segment(End(175, -1), End(177, 1))),
                        ),
                        Bridge(
                            Segment(End(112, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(128, 1), End(129, -1)), Segment(End(129, -1), End(130, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(84, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(117, 1)),
                            Segment(End(117, 1), End(118, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(118, -1), End(119, 1)),
                            Segment(End(119, 1), End(120, -1)),
                            Tunnel(0, Segment(End(166, 1), End(167, -1)), Segment(End(167, -1), End(168, 1))),
                        ),
                        Bridge(
                            Segment(End(120, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(139, 1), End(140, -1)), Segment(End(140, -1), End(141, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(126, 1)),
                            Segment(End(126, 1), End(127, -1)),
                            Tunnel(0, Segment(End(93, 1), End(94, -1)), Segment(End(94, -1), End(95, 1))),
                        ),
                        Bridge(
                            Segment(End(127, -1), End(128, 1)),
                            Segment(End(128, 1), End(129, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(129, -1), End(130, 1)),
                            Segment(End(130, 1), End(131, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(131, -1), End(133, 1)),
                            Segment(End(133, 1), End(134, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(134, -1), End(135, 1)),
                            Segment(End(135, 1), End(136, -1)),
                            Tunnel(0, Segment(End(150, 1), End(151, -1)), Segment(End(151, -1), End(152, 1))),
                        ),
                        Bridge(
                            Segment(End(136, -1), End(137, 1)),
                            Segment(End(137, 1), End(138, -1)),
                            Tunnel(0, Segment(End(104, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(138, -1), End(139, 1)),
                            Segment(End(139, 1), End(140, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(140, -1), End(141, 1)),
                            Segment(End(141, 1), End(142, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(142, -1), End(144, 1)),
                            Segment(End(144, 1), End(145, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(145, -1), End(146, 1)),
                            Segment(End(146, 1), End(147, -1)),
                            Tunnel(0, Segment(End(161, 1), End(162, -1)), Segment(End(162, -1), End(163, 1))),
                        ),
                        Bridge(
                            Segment(End(147, -1), End(148, 1)),
                            Segment(End(148, 1), End(149, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(117, 1))),
                        ),
                        Bridge(
                            Segment(End(149, -1), End(150, 1)),
                            Segment(End(150, 1), End(151, -1)),
                            Tunnel(0, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(151, -1), End(152, 1)),
                            Segment(End(152, 1), End(153, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(153, -1), End(155, 1)),
                            Segment(End(155, 1), End(156, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(156, -1), End(157, 1)),
                            Segment(End(157, 1), End(158, -1)),
                            Tunnel(0, Segment(End(172, 1), End(173, -1)), Segment(End(173, -1), End(174, 1))),
                        ),
                        Bridge(
                            Segment(End(158, -1), End(159, 1)),
                            Segment(End(159, 1), End(160, -1)),
                            Tunnel(0, Segment(End(126, 1), End(127, -1)), Segment(End(127, -1), End(128, 1))),
                        ),
                        Bridge(
                            Segment(End(160, -1), End(161, 1)),
                            Segment(End(161, 1), End(162, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(162, -1), End(163, 1)),
                            Segment(End(163, 1), End(164, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(164, -1), End(166, 1)),
                            Segment(End(166, 1), End(167, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(167, -1), End(168, 1)),
                            Segment(End(168, 1), End(169, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(169, -1), End(170, 1)),
                            Segment(End(170, 1), End(171, -1)),
                            Tunnel(0, Segment(End(137, 1), End(138, -1)), Segment(End(138, -1), End(139, 1))),
                        ),
                        Bridge(
                            Segment(End(171, -1), End(172, 1)),
                            Segment(End(172, 1), End(173, -1)),
                            Tunnel(0, Segment(End(91, 1), End(92, -1)), Segment(End(92, -1), End(93, 1))),
                        ),
                        Bridge(
                            Segment(End(173, -1), End(174, 1)),
                            Segment(End(174, 1), End(175, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_9_10(self):
        knot = Knot(9, 10)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(71, -1)),
                        Segment(End(71, -1), End(72, 1)),
                        Segment(End(72, 1), End(73, -1)),
                        Segment(End(73, -1), End(74, 1)),
                        Segment(End(74, 1), End(75, -1)),
                        Segment(End(75, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(84, -1)),
                        Segment(End(84, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(88, -1)),
                        Segment(End(88, -1), End(89, 1)),
                        Segment(End(89, 1), End(91, -1)),
                        Segment(End(91, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(96, 1)),
                        Segment(End(96, 1), End(97, -1)),
                        Segment(End(97, -1), End(98, 1)),
                        Segment(End(98, 1), End(99, -1)),
                        Segment(End(99, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(105, 1)),
                        Segment(End(105, 1), End(106, -1)),
                        Segment(End(106, -1), End(107, 1)),
                        Segment(End(107, 1), End(108, -1)),
                        Segment(End(108, -1), End(109, 1)),
                        Segment(End(109, 1), End(111, -1)),
                        Segment(End(111, -1), End(112, 1)),
                        Segment(End(112, 1), End(113, -1)),
                        Segment(End(113, -1), End(114, 1)),
                        Segment(End(114, 1), End(115, -1)),
                        Segment(End(115, -1), End(116, 1)),
                        Segment(End(116, 1), End(117, -1)),
                        Segment(End(117, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(121, 1)),
                        Segment(End(121, 1), End(122, -1)),
                        Segment(End(122, -1), End(123, 1)),
                        Segment(End(123, 1), End(124, -1)),
                        Segment(End(124, -1), End(125, 1)),
                        Segment(End(125, 1), End(126, -1)),
                        Segment(End(126, -1), End(127, 1)),
                        Segment(End(127, 1), End(128, -1)),
                        Segment(End(128, -1), End(129, 1)),
                        Segment(End(129, 1), End(131, -1)),
                        Segment(End(131, -1), End(132, 1)),
                        Segment(End(132, 1), End(133, -1)),
                        Segment(End(133, -1), End(134, 1)),
                        Segment(End(134, 1), End(135, -1)),
                        Segment(End(135, -1), End(136, 1)),
                        Segment(End(136, 1), End(137, -1)),
                        Segment(End(137, -1), End(138, 1)),
                        Segment(End(138, 1), End(139, -1)),
                        Segment(End(139, -1), End(141, 1)),
                        Segment(End(141, 1), End(142, -1)),
                        Segment(End(142, -1), End(143, 1)),
                        Segment(End(143, 1), End(144, -1)),
                        Segment(End(144, -1), End(145, 1)),
                        Segment(End(145, 1), End(146, -1)),
                        Segment(End(146, -1), End(147, 1)),
                        Segment(End(147, 1), End(148, -1)),
                        Segment(End(148, -1), End(149, 1)),
                        Segment(End(149, 1), End(151, -1)),
                        Segment(End(151, -1), End(152, 1)),
                        Segment(End(152, 1), End(153, -1)),
                        Segment(End(153, -1), End(154, 1)),
                        Segment(End(154, 1), End(155, -1)),
                        Segment(End(155, -1), End(156, 1)),
                        Segment(End(156, 1), End(157, -1)),
                        Segment(End(157, -1), End(158, 1)),
                        Segment(End(158, 1), End(159, -1)),
                        Segment(End(159, -1), End(161, 1)),
                        Segment(End(161, 1), End(162, -1)),
                        Segment(End(162, -1), End(163, 1)),
                        Segment(End(163, 1), End(164, -1)),
                        Segment(End(164, -1), End(165, 1)),
                        Segment(End(165, 1), End(166, -1)),
                        Segment(End(166, -1), End(167, 1)),
                        Segment(End(167, 1), End(168, -1)),
                        Segment(End(168, -1), End(169, 1)),
                        Segment(End(169, 1), End(171, -1)),
                        Segment(End(171, -1), End(172, 1)),
                        Segment(End(172, 1), End(173, -1)),
                        Segment(End(173, -1), End(174, 1)),
                        Segment(End(174, 1), End(175, -1)),
                        Segment(End(175, -1), End(176, 1)),
                        Segment(End(176, 1), End(177, -1)),
                        Segment(End(177, -1), End(178, 1)),
                        Segment(End(178, 1), End(179, -1)),
                        Segment(End(179, -1), End(181, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(96, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(132, 1), End(133, -1)), Segment(End(133, -1), End(134, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(0, Segment(End(169, 1), End(171, -1)), Segment(End(171, -1), End(172, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(123, 1), End(124, -1)), Segment(End(124, -1), End(125, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(161, 1), End(162, -1)), Segment(End(162, -1), End(163, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(114, 1), End(115, -1)), Segment(End(115, -1), End(116, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(152, 1), End(153, -1)), Segment(End(153, -1), End(154, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(0, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(105, 1), End(106, -1)), Segment(End(106, -1), End(107, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(143, 1), End(144, -1)), Segment(End(144, -1), End(145, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(96, 1), End(97, -1)), Segment(End(97, -1), End(98, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(134, 1), End(135, -1)), Segment(End(135, -1), End(136, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(172, 1), End(173, -1)), Segment(End(173, -1), End(174, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(51, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(87, 1), End(88, -1)), Segment(End(88, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(125, 1), End(126, -1)), Segment(End(126, -1), End(127, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(163, 1), End(164, -1)), Segment(End(164, -1), End(165, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(0, Segment(End(116, 1), End(117, -1)), Segment(End(117, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(0, Segment(End(154, 1), End(155, -1)), Segment(End(155, -1), End(156, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(71, -1)),
                            Tunnel(0, Segment(End(49, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(72, 1)),
                            Segment(End(72, 1), End(73, -1)),
                            Tunnel(0, Segment(End(107, 1), End(108, -1)), Segment(End(108, -1), End(109, 1))),
                        ),
                        Bridge(
                            Segment(End(73, -1), End(74, 1)),
                            Segment(End(74, 1), End(75, -1)),
                            Tunnel(0, Segment(End(145, 1), End(146, -1)), Segment(End(146, -1), End(147, 1))),
                        ),
                        Bridge(
                            Segment(End(75, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(98, 1), End(99, -1)), Segment(End(99, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(84, -1)),
                            Tunnel(0, Segment(End(136, 1), End(137, -1)), Segment(End(137, -1), End(138, 1))),
                        ),
                        Bridge(
                            Segment(End(84, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(0, Segment(End(174, 1), End(175, -1)), Segment(End(175, -1), End(176, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(88, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(88, -1), End(89, 1)),
                            Segment(End(89, 1), End(91, -1)),
                            Tunnel(0, Segment(End(69, 1), End(71, -1)), Segment(End(71, -1), End(72, 1))),
                        ),
                        Bridge(
                            Segment(End(91, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(127, 1), End(128, -1)), Segment(End(128, -1), End(129, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(165, 1), End(166, -1)), Segment(End(166, -1), End(167, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(96, 1)),
                            Segment(End(96, 1), End(97, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(97, -1), End(98, 1)),
                            Segment(End(98, 1), End(99, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(99, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(121, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(156, 1), End(157, -1)), Segment(End(157, -1), End(158, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(105, 1)),
                            Segment(End(105, 1), End(106, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(106, -1), End(107, 1)),
                            Segment(End(107, 1), End(108, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(108, -1), End(109, 1)),
                            Segment(End(109, 1), End(111, -1)),
                            Tunnel(0, Segment(End(89, 1), End(91, -1)), Segment(End(91, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(112, 1)),
                            Segment(End(112, 1), End(113, -1)),
                            Tunnel(0, Segment(End(147, 1), End(148, -1)), Segment(End(148, -1), End(149, 1))),
                        ),
                        Bridge(
                            Segment(End(113, -1), End(114, 1)),
                            Segment(End(114, 1), End(115, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(115, -1), End(116, 1)),
                            Segment(End(116, 1), End(117, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(117, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(119, -1), End(121, 1)),
                            Segment(End(121, 1), End(122, -1)),
                            Tunnel(0, Segment(End(138, 1), End(139, -1)), Segment(End(139, -1), End(141, 1))),
                        ),
                        Bridge(
                            Segment(End(122, -1), End(123, 1)),
                            Segment(End(123, 1), End(124, -1)),
                            Tunnel(0, Segment(End(176, 1), End(177, -1)), Segment(End(177, -1), End(178, 1))),
                        ),
                        Bridge(
                            Segment(End(124, -1), End(125, 1)),
                            Segment(End(125, 1), End(126, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(126, -1), End(127, 1)),
                            Segment(End(127, 1), End(128, -1)),
                            Tunnel(0, Segment(End(72, 1), End(73, -1)), Segment(End(73, -1), End(74, 1))),
                        ),
                        Bridge(
                            Segment(End(128, -1), End(129, 1)),
                            Segment(End(129, 1), End(131, -1)),
                            Tunnel(0, Segment(End(109, 1), End(111, -1)), Segment(End(111, -1), End(112, 1))),
                        ),
                        Bridge(
                            Segment(End(131, -1), End(132, 1)),
                            Segment(End(132, 1), End(133, -1)),
                            Tunnel(0, Segment(End(167, 1), End(168, -1)), Segment(End(168, -1), End(169, 1))),
                        ),
                        Bridge(
                            Segment(End(133, -1), End(134, 1)),
                            Segment(End(134, 1), End(135, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(135, -1), End(136, 1)),
                            Segment(End(136, 1), End(137, -1)),
                            Tunnel(0, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(137, -1), End(138, 1)),
                            Segment(End(138, 1), End(139, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(139, -1), End(141, 1)),
                            Segment(End(141, 1), End(142, -1)),
                            Tunnel(0, Segment(End(158, 1), End(159, -1)), Segment(End(159, -1), End(161, 1))),
                        ),
                        Bridge(
                            Segment(End(142, -1), End(143, 1)),
                            Segment(End(143, 1), End(144, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(144, -1), End(145, 1)),
                            Segment(End(145, 1), End(146, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(146, -1), End(147, 1)),
                            Segment(End(147, 1), End(148, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(148, -1), End(149, 1)),
                            Segment(End(149, 1), End(151, -1)),
                            Tunnel(0, Segment(End(129, 1), End(131, -1)), Segment(End(131, -1), End(132, 1))),
                        ),
                        Bridge(
                            Segment(End(151, -1), End(152, 1)),
                            Segment(End(152, 1), End(153, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(153, -1), End(154, 1)),
                            Segment(End(154, 1), End(155, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(155, -1), End(156, 1)),
                            Segment(End(156, 1), End(157, -1)),
                            Tunnel(0, Segment(End(83, 1), End(84, -1)), Segment(End(84, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(157, -1), End(158, 1)),
                            Segment(End(158, 1), End(159, -1)),
                            Tunnel(0, Segment(End(121, 1), End(122, -1)), Segment(End(122, -1), End(123, 1))),
                        ),
                        Bridge(
                            Segment(End(159, -1), End(161, 1)),
                            Segment(End(161, 1), End(162, -1)),
                            Tunnel(0, Segment(End(178, 1), End(179, -1)), Segment(End(179, -1), End(181, 1))),
                        ),
                        Bridge(
                            Segment(End(162, -1), End(163, 1)),
                            Segment(End(163, 1), End(164, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(164, -1), End(165, 1)),
                            Segment(End(165, 1), End(166, -1)),
                            Tunnel(0, Segment(End(74, 1), End(75, -1)), Segment(End(75, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(166, -1), End(167, 1)),
                            Segment(End(167, 1), End(168, -1)),
                            Tunnel(0, Segment(End(112, 1), End(113, -1)), Segment(End(113, -1), End(114, 1))),
                        ),
                        Bridge(
                            Segment(End(168, -1), End(169, 1)),
                            Segment(End(169, 1), End(171, -1)),
                            Tunnel(0, Segment(End(149, 1), End(151, -1)), Segment(End(151, -1), End(152, 1))),
                        ),
                        Bridge(
                            Segment(End(171, -1), End(172, 1)),
                            Segment(End(172, 1), End(173, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(173, -1), End(174, 1)),
                            Segment(End(174, 1), End(175, -1)),
                            Tunnel(0, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(175, -1), End(176, 1)),
                            Segment(End(176, 1), End(177, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(105, 1))),
                        ),
                        Bridge(
                            Segment(End(177, -1), End(178, 1)),
                            Segment(End(178, 1), End(179, -1)),
                            Tunnel(0, Segment(End(141, 1), End(142, -1)), Segment(End(142, -1), End(143, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_10_9(self):
        knot = Knot(10, 9)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(77, 1)),
                        Segment(End(77, 1), End(78, -1)),
                        Segment(End(78, -1), End(79, 1)),
                        Segment(End(79, 1), End(80, -1)),
                        Segment(End(80, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(84, 1)),
                        Segment(End(84, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(88, 1)),
                        Segment(End(88, 1), End(89, -1)),
                        Segment(End(89, -1), End(91, 1)),
                        Segment(End(91, 1), End(92, -1)),
                        Segment(End(92, -1), End(93, 1)),
                        Segment(End(93, 1), End(94, -1)),
                        Segment(End(94, -1), End(95, 1)),
                        Segment(End(95, 1), End(96, -1)),
                        Segment(End(96, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(100, 1)),
                        Segment(End(100, 1), End(101, -1)),
                        Segment(End(101, -1), End(102, 1)),
                        Segment(End(102, 1), End(103, -1)),
                        Segment(End(103, -1), End(104, 1)),
                        Segment(End(104, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(109, 1)),
                        Segment(End(109, 1), End(110, -1)),
                        Segment(End(110, -1), End(111, 1)),
                        Segment(End(111, 1), End(112, -1)),
                        Segment(End(112, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(120, 1)),
                        Segment(End(120, 1), End(121, -1)),
                        Segment(End(121, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(127, 1)),
                        Segment(End(127, 1), End(128, -1)),
                        Segment(End(128, -1), End(129, 1)),
                        Segment(End(129, 1), End(130, -1)),
                        Segment(End(130, -1), End(131, 1)),
                        Segment(End(131, 1), End(132, -1)),
                        Segment(End(132, -1), End(133, 1)),
                        Segment(End(133, 1), End(134, -1)),
                        Segment(End(134, -1), End(136, 1)),
                        Segment(End(136, 1), End(137, -1)),
                        Segment(End(137, -1), End(138, 1)),
                        Segment(End(138, 1), End(139, -1)),
                        Segment(End(139, -1), End(140, 1)),
                        Segment(End(140, 1), End(141, -1)),
                        Segment(End(141, -1), End(142, 1)),
                        Segment(End(142, 1), End(143, -1)),
                        Segment(End(143, -1), End(145, 1)),
                        Segment(End(145, 1), End(146, -1)),
                        Segment(End(146, -1), End(147, 1)),
                        Segment(End(147, 1), End(148, -1)),
                        Segment(End(148, -1), End(149, 1)),
                        Segment(End(149, 1), End(150, -1)),
                        Segment(End(150, -1), End(151, 1)),
                        Segment(End(151, 1), End(152, -1)),
                        Segment(End(152, -1), End(154, 1)),
                        Segment(End(154, 1), End(155, -1)),
                        Segment(End(155, -1), End(156, 1)),
                        Segment(End(156, 1), End(157, -1)),
                        Segment(End(157, -1), End(158, 1)),
                        Segment(End(158, 1), End(159, -1)),
                        Segment(End(159, -1), End(160, 1)),
                        Segment(End(160, 1), End(161, -1)),
                        Segment(End(161, -1), End(163, 1)),
                        Segment(End(163, 1), End(164, -1)),
                        Segment(End(164, -1), End(165, 1)),
                        Segment(End(165, 1), End(166, -1)),
                        Segment(End(166, -1), End(167, 1)),
                        Segment(End(167, 1), End(168, -1)),
                        Segment(End(168, -1), End(169, 1)),
                        Segment(End(169, 1), End(170, -1)),
                        Segment(End(170, -1), End(172, 1)),
                        Segment(End(172, 1), End(173, -1)),
                        Segment(End(173, -1), End(174, 1)),
                        Segment(End(174, 1), End(175, -1)),
                        Segment(End(175, -1), End(176, 1)),
                        Segment(End(176, 1), End(177, -1)),
                        Segment(End(177, -1), End(178, 1)),
                        Segment(End(178, 1), End(179, -1)),
                        Segment(End(179, -1), End(181, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(160, 1), End(161, -1)), Segment(End(161, -1), End(163, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(84, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(169, 1), End(170, -1)), Segment(End(170, -1), End(172, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(131, 1), End(132, -1)), Segment(End(132, -1), End(133, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(93, 1), End(94, -1)), Segment(End(94, -1), End(95, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(178, 1), End(179, -1)), Segment(End(179, -1), End(181, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(140, 1), End(141, -1)), Segment(End(141, -1), End(142, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(102, 1), End(103, -1)), Segment(End(103, -1), End(104, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(149, 1), End(150, -1)), Segment(End(150, -1), End(151, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(111, 1), End(112, -1)), Segment(End(112, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(158, 1), End(159, -1)), Segment(End(159, -1), End(160, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(120, 1), End(121, -1)), Segment(End(121, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(84, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(0, Segment(End(167, 1), End(168, -1)), Segment(End(168, -1), End(169, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(129, 1), End(130, -1)), Segment(End(130, -1), End(131, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(91, 1), End(92, -1)), Segment(End(92, -1), End(93, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(176, 1), End(177, -1)), Segment(End(177, -1), End(178, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(138, 1), End(139, -1)), Segment(End(139, -1), End(140, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(100, 1), End(101, -1)), Segment(End(101, -1), End(102, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(147, 1), End(148, -1)), Segment(End(148, -1), End(149, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(109, 1), End(110, -1)), Segment(End(110, -1), End(111, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(77, 1)),
                            Segment(End(77, 1), End(78, -1)),
                            Tunnel(0, Segment(End(156, 1), End(157, -1)), Segment(End(157, -1), End(158, 1))),
                        ),
                        Bridge(
                            Segment(End(78, -1), End(79, 1)),
                            Segment(End(79, 1), End(80, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(120, 1))),
                        ),
                        Bridge(
                            Segment(End(80, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(84, 1)),
                            Segment(End(84, 1), End(85, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(165, 1), End(166, -1)), Segment(End(166, -1), End(167, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(88, 1)),
                            Segment(End(88, 1), End(89, -1)),
                            Tunnel(0, Segment(End(127, 1), End(128, -1)), Segment(End(128, -1), End(129, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(91, 1)),
                            Segment(End(91, 1), End(92, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(92, -1), End(93, 1)),
                            Segment(End(93, 1), End(94, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(94, -1), End(95, 1)),
                            Segment(End(95, 1), End(96, -1)),
                            Tunnel(0, Segment(End(174, 1), End(175, -1)), Segment(End(175, -1), End(176, 1))),
                        ),
                        Bridge(
                            Segment(End(96, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(136, 1), End(137, -1)), Segment(End(137, -1), End(138, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(100, 1)),
                            Segment(End(100, 1), End(101, -1)),
                            Tunnel(0, Segment(End(79, 1), End(80, -1)), Segment(End(80, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(101, -1), End(102, 1)),
                            Segment(End(102, 1), End(103, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(103, -1), End(104, 1)),
                            Segment(End(104, 1), End(105, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(145, 1), End(146, -1)), Segment(End(146, -1), End(147, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(109, 1)),
                            Segment(End(109, 1), End(110, -1)),
                            Tunnel(0, Segment(End(88, 1), End(89, -1)), Segment(End(89, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(110, -1), End(111, 1)),
                            Segment(End(111, 1), End(112, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(112, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(154, 1), End(155, -1)), Segment(End(155, -1), End(156, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(100, 1))),
                        ),
                        Bridge(
                            Segment(End(119, -1), End(120, 1)),
                            Segment(End(120, 1), End(121, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(121, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(163, 1), End(164, -1)), Segment(End(164, -1), End(165, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(127, 1)),
                            Segment(End(127, 1), End(128, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(109, 1))),
                        ),
                        Bridge(
                            Segment(End(128, -1), End(129, 1)),
                            Segment(End(129, 1), End(130, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(130, -1), End(131, 1)),
                            Segment(End(131, 1), End(132, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(132, -1), End(133, 1)),
                            Segment(End(133, 1), End(134, -1)),
                            Tunnel(0, Segment(End(172, 1), End(173, -1)), Segment(End(173, -1), End(174, 1))),
                        ),
                        Bridge(
                            Segment(End(134, -1), End(136, 1)),
                            Segment(End(136, 1), End(137, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(137, -1), End(138, 1)),
                            Segment(End(138, 1), End(139, -1)),
                            Tunnel(0, Segment(End(77, 1), End(78, -1)), Segment(End(78, -1), End(79, 1))),
                        ),
                        Bridge(
                            Segment(End(139, -1), End(140, 1)),
                            Segment(End(140, 1), End(141, -1)),
                            Tunnel(0, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(141, -1), End(142, 1)),
                            Segment(End(142, 1), End(143, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(143, -1), End(145, 1)),
                            Segment(End(145, 1), End(146, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(127, 1))),
                        ),
                        Bridge(
                            Segment(End(146, -1), End(147, 1)),
                            Segment(End(147, 1), End(148, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(88, 1))),
                        ),
                        Bridge(
                            Segment(End(148, -1), End(149, 1)),
                            Segment(End(149, 1), End(150, -1)),
                            Tunnel(0, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(150, -1), End(151, 1)),
                            Segment(End(151, 1), End(152, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(152, -1), End(154, 1)),
                            Segment(End(154, 1), End(155, -1)),
                            Tunnel(0, Segment(End(133, 1), End(134, -1)), Segment(End(134, -1), End(136, 1))),
                        ),
                        Bridge(
                            Segment(End(155, -1), End(156, 1)),
                            Segment(End(156, 1), End(157, -1)),
                            Tunnel(0, Segment(End(95, 1), End(96, -1)), Segment(End(96, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(157, -1), End(158, 1)),
                            Segment(End(158, 1), End(159, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(159, -1), End(160, 1)),
                            Segment(End(160, 1), End(161, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(161, -1), End(163, 1)),
                            Segment(End(163, 1), End(164, -1)),
                            Tunnel(0, Segment(End(142, 1), End(143, -1)), Segment(End(143, -1), End(145, 1))),
                        ),
                        Bridge(
                            Segment(End(164, -1), End(165, 1)),
                            Segment(End(165, 1), End(166, -1)),
                            Tunnel(0, Segment(End(104, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(166, -1), End(167, 1)),
                            Segment(End(167, 1), End(168, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(168, -1), End(169, 1)),
                            Segment(End(169, 1), End(170, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(170, -1), End(172, 1)),
                            Segment(End(172, 1), End(173, -1)),
                            Tunnel(0, Segment(End(151, 1), End(152, -1)), Segment(End(152, -1), End(154, 1))),
                        ),
                        Bridge(
                            Segment(End(173, -1), End(174, 1)),
                            Segment(End(174, 1), End(175, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(175, -1), End(176, 1)),
                            Segment(End(176, 1), End(177, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(77, 1))),
                        ),
                        Bridge(
                            Segment(End(177, -1), End(178, 1)),
                            Segment(End(178, 1), End(179, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_11_8(self):
        knot = Knot(11, 8)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(73, -1)),
                        Segment(End(73, -1), End(74, 1)),
                        Segment(End(74, 1), End(75, -1)),
                        Segment(End(75, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(84, -1)),
                        Segment(End(84, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(89, -1)),
                        Segment(End(89, -1), End(90, 1)),
                        Segment(End(90, 1), End(91, -1)),
                        Segment(End(91, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(110, 1)),
                        Segment(End(110, 1), End(111, -1)),
                        Segment(End(111, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(117, 1)),
                        Segment(End(117, 1), End(118, -1)),
                        Segment(End(118, -1), End(119, 1)),
                        Segment(End(119, 1), End(121, -1)),
                        Segment(End(121, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(126, 1)),
                        Segment(End(126, 1), End(127, -1)),
                        Segment(End(127, -1), End(129, 1)),
                        Segment(End(129, 1), End(130, -1)),
                        Segment(End(130, -1), End(131, 1)),
                        Segment(End(131, 1), End(132, -1)),
                        Segment(End(132, -1), End(133, 1)),
                        Segment(End(133, 1), End(134, -1)),
                        Segment(End(134, -1), End(135, 1)),
                        Segment(End(135, 1), End(137, -1)),
                        Segment(End(137, -1), End(138, 1)),
                        Segment(End(138, 1), End(139, -1)),
                        Segment(End(139, -1), End(140, 1)),
                        Segment(End(140, 1), End(141, -1)),
                        Segment(End(141, -1), End(142, 1)),
                        Segment(End(142, 1), End(143, -1)),
                        Segment(End(143, -1), End(145, 1)),
                        Segment(End(145, 1), End(146, -1)),
                        Segment(End(146, -1), End(147, 1)),
                        Segment(End(147, 1), End(148, -1)),
                        Segment(End(148, -1), End(149, 1)),
                        Segment(End(149, 1), End(150, -1)),
                        Segment(End(150, -1), End(151, 1)),
                        Segment(End(151, 1), End(153, -1)),
                        Segment(End(153, -1), End(154, 1)),
                        Segment(End(154, 1), End(155, -1)),
                        Segment(End(155, -1), End(156, 1)),
                        Segment(End(156, 1), End(157, -1)),
                        Segment(End(157, -1), End(158, 1)),
                        Segment(End(158, 1), End(159, -1)),
                        Segment(End(159, -1), End(161, 1)),
                        Segment(End(161, 1), End(162, -1)),
                        Segment(End(162, -1), End(163, 1)),
                        Segment(End(163, 1), End(164, -1)),
                        Segment(End(164, -1), End(165, 1)),
                        Segment(End(165, 1), End(166, -1)),
                        Segment(End(166, -1), End(167, 1)),
                        Segment(End(167, 1), End(169, -1)),
                        Segment(End(169, -1), End(170, 1)),
                        Segment(End(170, 1), End(171, -1)),
                        Segment(End(171, -1), End(172, 1)),
                        Segment(End(172, 1), End(173, -1)),
                        Segment(End(173, -1), End(174, 1)),
                        Segment(End(174, 1), End(175, -1)),
                        Segment(End(175, -1), End(177, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(110, 1), End(111, -1)), Segment(End(111, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(156, 1), End(157, -1)), Segment(End(157, -1), End(158, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(0, Segment(End(71, 1), End(73, -1)), Segment(End(73, -1), End(74, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(145, 1), End(146, -1)), Segment(End(146, -1), End(147, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(126, 1), End(127, -1)), Segment(End(127, -1), End(129, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(172, 1), End(173, -1)), Segment(End(173, -1), End(174, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(25, -1)),
                            Tunnel(0, Segment(End(87, 1), End(89, -1)), Segment(End(89, -1), End(90, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(117, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(161, 1), End(162, -1)), Segment(End(162, -1), End(163, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(142, 1), End(143, -1)), Segment(End(143, -1), End(145, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(41, -1)),
                            Tunnel(0, Segment(End(103, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(131, 1), End(132, -1)), Segment(End(132, -1), End(133, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(158, 1), End(159, -1)), Segment(End(159, -1), End(161, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(74, 1), End(75, -1)), Segment(End(75, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(57, -1)),
                            Tunnel(0, Segment(End(119, 1), End(121, -1)), Segment(End(121, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(147, 1), End(148, -1)), Segment(End(148, -1), End(149, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(0, Segment(End(174, 1), End(175, -1)), Segment(End(175, -1), End(177, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(90, 1), End(91, -1)), Segment(End(91, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(73, -1)),
                            Tunnel(0, Segment(End(135, 1), End(137, -1)), Segment(End(137, -1), End(138, 1))),
                        ),
                        Bridge(
                            Segment(End(73, -1), End(74, 1)),
                            Segment(End(74, 1), End(75, -1)),
                            Tunnel(0, Segment(End(117, 1), End(118, -1)), Segment(End(118, -1), End(119, 1))),
                        ),
                        Bridge(
                            Segment(End(75, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(163, 1), End(164, -1)), Segment(End(164, -1), End(165, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(84, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(84, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(89, -1)),
                            Tunnel(0, Segment(End(151, 1), End(153, -1)), Segment(End(153, -1), End(154, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(90, 1)),
                            Segment(End(90, 1), End(91, -1)),
                            Tunnel(0, Segment(End(133, 1), End(134, -1)), Segment(End(134, -1), End(135, 1))),
                        ),
                        Bridge(
                            Segment(End(91, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(105, -1)),
                            Tunnel(0, Segment(End(167, 1), End(169, -1)), Segment(End(169, -1), End(170, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(149, 1), End(150, -1)), Segment(End(150, -1), End(151, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(110, 1)),
                            Segment(End(110, 1), End(111, -1)),
                            Tunnel(0, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(117, 1)),
                            Segment(End(117, 1), End(118, -1)),
                            Tunnel(0, Segment(End(138, 1), End(139, -1)), Segment(End(139, -1), End(140, 1))),
                        ),
                        Bridge(
                            Segment(End(118, -1), End(119, 1)),
                            Segment(End(119, 1), End(121, -1)),
                            Tunnel(0, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(121, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(165, 1), End(166, -1)), Segment(End(166, -1), End(167, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(126, 1)),
                            Segment(End(126, 1), End(127, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(127, -1), End(129, 1)),
                            Segment(End(129, 1), End(130, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(130, -1), End(131, 1)),
                            Segment(End(131, 1), End(132, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(110, 1))),
                        ),
                        Bridge(
                            Segment(End(132, -1), End(133, 1)),
                            Segment(End(133, 1), End(134, -1)),
                            Tunnel(0, Segment(End(154, 1), End(155, -1)), Segment(End(155, -1), End(156, 1))),
                        ),
                        Bridge(
                            Segment(End(134, -1), End(135, 1)),
                            Segment(End(135, 1), End(137, -1)),
                            Tunnel(0, Segment(End(23, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(137, -1), End(138, 1)),
                            Segment(End(138, 1), End(139, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(139, -1), End(140, 1)),
                            Segment(End(140, 1), End(141, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(141, -1), End(142, 1)),
                            Segment(End(142, 1), End(143, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(99, 1))),
                        ),
                        Bridge(
                            Segment(End(143, -1), End(145, 1)),
                            Segment(End(145, 1), End(146, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(146, -1), End(147, 1)),
                            Segment(End(147, 1), End(148, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(126, 1))),
                        ),
                        Bridge(
                            Segment(End(148, -1), End(149, 1)),
                            Segment(End(149, 1), End(150, -1)),
                            Tunnel(0, Segment(End(170, 1), End(171, -1)), Segment(End(171, -1), End(172, 1))),
                        ),
                        Bridge(
                            Segment(End(150, -1), End(151, 1)),
                            Segment(End(151, 1), End(153, -1)),
                            Tunnel(0, Segment(End(39, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(153, -1), End(154, 1)),
                            Segment(End(154, 1), End(155, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(155, -1), End(156, 1)),
                            Segment(End(156, 1), End(157, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(157, -1), End(158, 1)),
                            Segment(End(158, 1), End(159, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(159, -1), End(161, 1)),
                            Segment(End(161, 1), End(162, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(162, -1), End(163, 1)),
                            Segment(End(163, 1), End(164, -1)),
                            Tunnel(0, Segment(End(140, 1), End(141, -1)), Segment(End(141, -1), End(142, 1))),
                        ),
                        Bridge(
                            Segment(End(164, -1), End(165, 1)),
                            Segment(End(165, 1), End(166, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(166, -1), End(167, 1)),
                            Segment(End(167, 1), End(169, -1)),
                            Tunnel(0, Segment(End(55, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(169, -1), End(170, 1)),
                            Segment(End(170, 1), End(171, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(171, -1), End(172, 1)),
                            Segment(End(172, 1), End(173, -1)),
                            Tunnel(0, Segment(End(83, 1), End(84, -1)), Segment(End(84, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(173, -1), End(174, 1)),
                            Segment(End(174, 1), End(175, -1)),
                            Tunnel(0, Segment(End(129, 1), End(130, -1)), Segment(End(130, -1), End(131, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_12_7(self):
        knot = Knot(12, 7)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(88, -1)),
                        Segment(End(88, -1), End(89, 1)),
                        Segment(End(89, 1), End(90, -1)),
                        Segment(End(90, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(96, 1)),
                        Segment(End(96, 1), End(97, -1)),
                        Segment(End(97, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(110, 1)),
                        Segment(End(110, 1), End(111, -1)),
                        Segment(End(111, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(117, 1)),
                        Segment(End(117, 1), End(118, -1)),
                        Segment(End(118, -1), End(120, 1)),
                        Segment(End(120, 1), End(121, -1)),
                        Segment(End(121, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(127, 1)),
                        Segment(End(127, 1), End(128, -1)),
                        Segment(End(128, -1), End(129, 1)),
                        Segment(End(129, 1), End(130, -1)),
                        Segment(End(130, -1), End(131, 1)),
                        Segment(End(131, 1), End(132, -1)),
                        Segment(End(132, -1), End(134, 1)),
                        Segment(End(134, 1), End(135, -1)),
                        Segment(End(135, -1), End(136, 1)),
                        Segment(End(136, 1), End(137, -1)),
                        Segment(End(137, -1), End(138, 1)),
                        Segment(End(138, 1), End(139, -1)),
                        Segment(End(139, -1), End(141, 1)),
                        Segment(End(141, 1), End(142, -1)),
                        Segment(End(142, -1), End(143, 1)),
                        Segment(End(143, 1), End(144, -1)),
                        Segment(End(144, -1), End(145, 1)),
                        Segment(End(145, 1), End(146, -1)),
                        Segment(End(146, -1), End(148, 1)),
                        Segment(End(148, 1), End(149, -1)),
                        Segment(End(149, -1), End(150, 1)),
                        Segment(End(150, 1), End(151, -1)),
                        Segment(End(151, -1), End(152, 1)),
                        Segment(End(152, 1), End(153, -1)),
                        Segment(End(153, -1), End(155, 1)),
                        Segment(End(155, 1), End(156, -1)),
                        Segment(End(156, -1), End(157, 1)),
                        Segment(End(157, 1), End(158, -1)),
                        Segment(End(158, -1), End(159, 1)),
                        Segment(End(159, 1), End(160, -1)),
                        Segment(End(160, -1), End(162, 1)),
                        Segment(End(162, 1), End(163, -1)),
                        Segment(End(163, -1), End(164, 1)),
                        Segment(End(164, 1), End(165, -1)),
                        Segment(End(165, -1), End(166, 1)),
                        Segment(End(166, 1), End(167, -1)),
                        Segment(End(167, -1), End(169, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(96, 1), End(97, -1)), Segment(End(97, -1), End(99, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(148, 1), End(149, -1)), Segment(End(149, -1), End(150, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(129, 1), End(130, -1)), Segment(End(130, -1), End(131, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(155, 1), End(156, -1)), Segment(End(156, -1), End(157, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(110, 1), End(111, -1)), Segment(End(111, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(136, 1), End(137, -1)), Segment(End(137, -1), End(138, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(162, 1), End(163, -1)), Segment(End(163, -1), End(164, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(117, 1), End(118, -1)), Segment(End(118, -1), End(120, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(0, Segment(End(143, 1), End(144, -1)), Segment(End(144, -1), End(145, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(127, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(150, 1), End(151, -1)), Segment(End(151, -1), End(152, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(131, 1), End(132, -1)), Segment(End(132, -1), End(134, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(157, 1), End(158, -1)), Segment(End(158, -1), End(159, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(138, 1), End(139, -1)), Segment(End(139, -1), End(141, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(164, 1), End(165, -1)), Segment(End(165, -1), End(166, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(145, 1), End(146, -1)), Segment(End(146, -1), End(148, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(152, 1), End(153, -1)), Segment(End(153, -1), End(155, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(159, 1), End(160, -1)), Segment(End(160, -1), End(162, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(166, 1), End(167, -1)), Segment(End(167, -1), End(169, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(88, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(88, -1), End(89, 1)),
                            Segment(End(89, 1), End(90, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(90, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(96, 1)),
                            Segment(End(96, 1), End(97, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(97, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(110, 1)),
                            Segment(End(110, 1), End(111, -1)),
                            Tunnel(0, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(117, 1)),
                            Segment(End(117, 1), End(118, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(118, -1), End(120, 1)),
                            Segment(End(120, 1), End(121, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(121, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(127, 1)),
                            Segment(End(127, 1), End(128, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(128, -1), End(129, 1)),
                            Segment(End(129, 1), End(130, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(130, -1), End(131, 1)),
                            Segment(End(131, 1), End(132, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(132, -1), End(134, 1)),
                            Segment(End(134, 1), End(135, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(135, -1), End(136, 1)),
                            Segment(End(136, 1), End(137, -1)),
                            Tunnel(0, Segment(End(87, 1), End(88, -1)), Segment(End(88, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(137, -1), End(138, 1)),
                            Segment(End(138, 1), End(139, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(139, -1), End(141, 1)),
                            Segment(End(141, 1), End(142, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(142, -1), End(143, 1)),
                            Segment(End(143, 1), End(144, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(96, 1))),
                        ),
                        Bridge(
                            Segment(End(144, -1), End(145, 1)),
                            Segment(End(145, 1), End(146, -1)),
                            Tunnel(0, Segment(End(120, 1), End(121, -1)), Segment(End(121, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(146, -1), End(148, 1)),
                            Segment(End(148, 1), End(149, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(149, -1), End(150, 1)),
                            Segment(End(150, 1), End(151, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(151, -1), End(152, 1)),
                            Segment(End(152, 1), End(153, -1)),
                            Tunnel(0, Segment(End(127, 1), End(128, -1)), Segment(End(128, -1), End(129, 1))),
                        ),
                        Bridge(
                            Segment(End(153, -1), End(155, 1)),
                            Segment(End(155, 1), End(156, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(156, -1), End(157, 1)),
                            Segment(End(157, 1), End(158, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(110, 1))),
                        ),
                        Bridge(
                            Segment(End(158, -1), End(159, 1)),
                            Segment(End(159, 1), End(160, -1)),
                            Tunnel(0, Segment(End(134, 1), End(135, -1)), Segment(End(135, -1), End(136, 1))),
                        ),
                        Bridge(
                            Segment(End(160, -1), End(162, 1)),
                            Segment(End(162, 1), End(163, -1)),
                            Tunnel(0, Segment(End(89, 1), End(90, -1)), Segment(End(90, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(163, -1), End(164, 1)),
                            Segment(End(164, 1), End(165, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(117, 1))),
                        ),
                        Bridge(
                            Segment(End(165, -1), End(166, 1)),
                            Segment(End(166, 1), End(167, -1)),
                            Tunnel(0, Segment(End(141, 1), End(142, -1)), Segment(End(142, -1), End(143, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_8_12(self):
        knot = Knot(8, 12)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(3, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(3, Segment(End(41, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(2, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(1, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(3, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(1, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(3, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(37, -1)),
                            Tunnel(3, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(2, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(2, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(3, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(2, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(2, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(3, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(1, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(3, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(2, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(2, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(2, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(39, -1)),
                            Tunnel(0, Segment(End(35, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(3, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(3, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(1, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(3, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(1, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(3, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(17, -1)),
                            Tunnel(1, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(2, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(3, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(1, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(3, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(3, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(41, -1)),
                            Tunnel(1, Segment(End(37, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(3, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(2, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(2, Segment(End(15, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(1, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(3, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(1, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(2, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(43, -1)),
                            Tunnel(2, Segment(End(39, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(1, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(3, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_9_11(self):
        knot = Knot(9, 11)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(84, 1)),
                        Segment(End(84, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(89, 1)),
                        Segment(End(89, 1), End(90, -1)),
                        Segment(End(90, -1), End(91, 1)),
                        Segment(End(91, 1), End(92, -1)),
                        Segment(End(92, -1), End(93, 1)),
                        Segment(End(93, 1), End(94, -1)),
                        Segment(End(94, -1), End(95, 1)),
                        Segment(End(95, 1), End(96, -1)),
                        Segment(End(96, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(100, 1)),
                        Segment(End(100, 1), End(101, -1)),
                        Segment(End(101, -1), End(102, 1)),
                        Segment(End(102, 1), End(103, -1)),
                        Segment(End(103, -1), End(104, 1)),
                        Segment(End(104, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(111, 1)),
                        Segment(End(111, 1), End(112, -1)),
                        Segment(End(112, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(117, 1)),
                        Segment(End(117, 1), End(118, -1)),
                        Segment(End(118, -1), End(119, 1)),
                        Segment(End(119, 1), End(120, -1)),
                        Segment(End(120, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(126, 1)),
                        Segment(End(126, 1), End(127, -1)),
                        Segment(End(127, -1), End(128, 1)),
                        Segment(End(128, 1), End(129, -1)),
                        Segment(End(129, -1), End(130, 1)),
                        Segment(End(130, 1), End(131, -1)),
                        Segment(End(131, -1), End(133, 1)),
                        Segment(End(133, 1), End(134, -1)),
                        Segment(End(134, -1), End(135, 1)),
                        Segment(End(135, 1), End(136, -1)),
                        Segment(End(136, -1), End(137, 1)),
                        Segment(End(137, 1), End(138, -1)),
                        Segment(End(138, -1), End(139, 1)),
                        Segment(End(139, 1), End(140, -1)),
                        Segment(End(140, -1), End(141, 1)),
                        Segment(End(141, 1), End(142, -1)),
                        Segment(End(142, -1), End(144, 1)),
                        Segment(End(144, 1), End(145, -1)),
                        Segment(End(145, -1), End(146, 1)),
                        Segment(End(146, 1), End(147, -1)),
                        Segment(End(147, -1), End(148, 1)),
                        Segment(End(148, 1), End(149, -1)),
                        Segment(End(149, -1), End(150, 1)),
                        Segment(End(150, 1), End(151, -1)),
                        Segment(End(151, -1), End(152, 1)),
                        Segment(End(152, 1), End(153, -1)),
                        Segment(End(153, -1), End(155, 1)),
                        Segment(End(155, 1), End(156, -1)),
                        Segment(End(156, -1), End(157, 1)),
                        Segment(End(157, 1), End(158, -1)),
                        Segment(End(158, -1), End(159, 1)),
                        Segment(End(159, 1), End(160, -1)),
                        Segment(End(160, -1), End(161, 1)),
                        Segment(End(161, 1), End(162, -1)),
                        Segment(End(162, -1), End(163, 1)),
                        Segment(End(163, 1), End(164, -1)),
                        Segment(End(164, -1), End(166, 1)),
                        Segment(End(166, 1), End(167, -1)),
                        Segment(End(167, -1), End(168, 1)),
                        Segment(End(168, 1), End(169, -1)),
                        Segment(End(169, -1), End(170, 1)),
                        Segment(End(170, 1), End(171, -1)),
                        Segment(End(171, -1), End(172, 1)),
                        Segment(End(172, 1), End(173, -1)),
                        Segment(End(173, -1), End(174, 1)),
                        Segment(End(174, 1), End(175, -1)),
                        Segment(End(175, -1), End(177, 1)),
                        Segment(End(177, 1), End(178, -1)),
                        Segment(End(178, -1), End(179, 1)),
                        Segment(End(179, 1), End(180, -1)),
                        Segment(End(180, -1), End(181, 1)),
                        Segment(End(181, 1), End(182, -1)),
                        Segment(End(182, -1), End(183, 1)),
                        Segment(End(183, 1), End(184, -1)),
                        Segment(End(184, -1), End(185, 1)),
                        Segment(End(185, 1), End(186, -1)),
                        Segment(End(186, -1), End(188, 1)),
                        Segment(End(188, 1), End(189, -1)),
                        Segment(End(189, -1), End(190, 1)),
                        Segment(End(190, 1), End(191, -1)),
                        Segment(End(191, -1), End(192, 1)),
                        Segment(End(192, 1), End(193, -1)),
                        Segment(End(193, -1), End(194, 1)),
                        Segment(End(194, 1), End(195, -1)),
                        Segment(End(195, -1), End(196, 1)),
                        Segment(End(196, 1), End(197, -1)),
                        Segment(End(197, -1), End(199, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(111, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(128, 1), End(129, -1)), Segment(End(129, -1), End(130, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(148, 1), End(149, -1)), Segment(End(149, -1), End(150, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(168, 1), End(169, -1)), Segment(End(169, -1), End(170, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(188, 1), End(189, -1)), Segment(End(189, -1), End(190, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(119, 1), End(120, -1)), Segment(End(120, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(139, 1), End(140, -1)), Segment(End(140, -1), End(141, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(159, 1), End(160, -1)), Segment(End(160, -1), End(161, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(179, 1), End(180, -1)), Segment(End(180, -1), End(181, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(130, 1), End(131, -1)), Segment(End(131, -1), End(133, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(150, 1), End(151, -1)), Segment(End(151, -1), End(152, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(170, 1), End(171, -1)), Segment(End(171, -1), End(172, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(190, 1), End(191, -1)), Segment(End(191, -1), End(192, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(141, 1), End(142, -1)), Segment(End(142, -1), End(144, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(161, 1), End(162, -1)), Segment(End(162, -1), End(163, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(181, 1), End(182, -1)), Segment(End(182, -1), End(183, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(152, 1), End(153, -1)), Segment(End(153, -1), End(155, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(172, 1), End(173, -1)), Segment(End(173, -1), End(174, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(192, 1), End(193, -1)), Segment(End(193, -1), End(194, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(163, 1), End(164, -1)), Segment(End(164, -1), End(166, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(183, 1), End(184, -1)), Segment(End(184, -1), End(185, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(174, 1), End(175, -1)), Segment(End(175, -1), End(177, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(194, 1), End(195, -1)), Segment(End(195, -1), End(196, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(185, 1), End(186, -1)), Segment(End(186, -1), End(188, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(84, 1)),
                            Segment(End(84, 1), End(85, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(89, 1)),
                            Segment(End(89, 1), End(90, -1)),
                            Tunnel(0, Segment(End(196, 1), End(197, -1)), Segment(End(197, -1), End(199, 1))),
                        ),
                        Bridge(
                            Segment(End(90, -1), End(91, 1)),
                            Segment(End(91, 1), End(92, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(92, -1), End(93, 1)),
                            Segment(End(93, 1), End(94, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(94, -1), End(95, 1)),
                            Segment(End(95, 1), End(96, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(96, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(100, 1)),
                            Segment(End(100, 1), End(101, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(101, -1), End(102, 1)),
                            Segment(End(102, 1), End(103, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(103, -1), End(104, 1)),
                            Segment(End(104, 1), End(105, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(89, 1), End(90, -1)), Segment(End(90, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(111, 1)),
                            Segment(End(111, 1), End(112, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(112, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(117, 1)),
                            Segment(End(117, 1), End(118, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(118, -1), End(119, 1)),
                            Segment(End(119, 1), End(120, -1)),
                            Tunnel(0, Segment(End(100, 1), End(101, -1)), Segment(End(101, -1), End(102, 1))),
                        ),
                        Bridge(
                            Segment(End(120, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(126, 1)),
                            Segment(End(126, 1), End(127, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(127, -1), End(128, 1)),
                            Segment(End(128, 1), End(129, -1)),
                            Tunnel(0, Segment(End(91, 1), End(92, -1)), Segment(End(92, -1), End(93, 1))),
                        ),
                        Bridge(
                            Segment(End(129, -1), End(130, 1)),
                            Segment(End(130, 1), End(131, -1)),
                            Tunnel(0, Segment(End(111, 1), End(112, -1)), Segment(End(112, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(131, -1), End(133, 1)),
                            Segment(End(133, 1), End(134, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(134, -1), End(135, 1)),
                            Segment(End(135, 1), End(136, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(136, -1), End(137, 1)),
                            Segment(End(137, 1), End(138, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(84, 1))),
                        ),
                        Bridge(
                            Segment(End(138, -1), End(139, 1)),
                            Segment(End(139, 1), End(140, -1)),
                            Tunnel(0, Segment(End(102, 1), End(103, -1)), Segment(End(103, -1), End(104, 1))),
                        ),
                        Bridge(
                            Segment(End(140, -1), End(141, 1)),
                            Segment(End(141, 1), End(142, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(142, -1), End(144, 1)),
                            Segment(End(144, 1), End(145, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(145, -1), End(146, 1)),
                            Segment(End(146, 1), End(147, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(147, -1), End(148, 1)),
                            Segment(End(148, 1), End(149, -1)),
                            Tunnel(0, Segment(End(93, 1), End(94, -1)), Segment(End(94, -1), End(95, 1))),
                        ),
                        Bridge(
                            Segment(End(149, -1), End(150, 1)),
                            Segment(End(150, 1), End(151, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(151, -1), End(152, 1)),
                            Segment(End(152, 1), End(153, -1)),
                            Tunnel(0, Segment(End(133, 1), End(134, -1)), Segment(End(134, -1), End(135, 1))),
                        ),
                        Bridge(
                            Segment(End(153, -1), End(155, 1)),
                            Segment(End(155, 1), End(156, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(156, -1), End(157, 1)),
                            Segment(End(157, 1), End(158, -1)),
                            Tunnel(0, Segment(End(84, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(158, -1), End(159, 1)),
                            Segment(End(159, 1), End(160, -1)),
                            Tunnel(0, Segment(End(104, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(160, -1), End(161, 1)),
                            Segment(End(161, 1), End(162, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(126, 1))),
                        ),
                        Bridge(
                            Segment(End(162, -1), End(163, 1)),
                            Segment(End(163, 1), End(164, -1)),
                            Tunnel(0, Segment(End(144, 1), End(145, -1)), Segment(End(145, -1), End(146, 1))),
                        ),
                        Bridge(
                            Segment(End(164, -1), End(166, 1)),
                            Segment(End(166, 1), End(167, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(167, -1), End(168, 1)),
                            Segment(End(168, 1), End(169, -1)),
                            Tunnel(0, Segment(End(95, 1), End(96, -1)), Segment(End(96, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(169, -1), End(170, 1)),
                            Segment(End(170, 1), End(171, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(117, 1))),
                        ),
                        Bridge(
                            Segment(End(171, -1), End(172, 1)),
                            Segment(End(172, 1), End(173, -1)),
                            Tunnel(0, Segment(End(135, 1), End(136, -1)), Segment(End(136, -1), End(137, 1))),
                        ),
                        Bridge(
                            Segment(End(173, -1), End(174, 1)),
                            Segment(End(174, 1), End(175, -1)),
                            Tunnel(0, Segment(End(155, 1), End(156, -1)), Segment(End(156, -1), End(157, 1))),
                        ),
                        Bridge(
                            Segment(End(175, -1), End(177, 1)),
                            Segment(End(177, 1), End(178, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(178, -1), End(179, 1)),
                            Segment(End(179, 1), End(180, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(180, -1), End(181, 1)),
                            Segment(End(181, 1), End(182, -1)),
                            Tunnel(0, Segment(End(126, 1), End(127, -1)), Segment(End(127, -1), End(128, 1))),
                        ),
                        Bridge(
                            Segment(End(182, -1), End(183, 1)),
                            Segment(End(183, 1), End(184, -1)),
                            Tunnel(0, Segment(End(146, 1), End(147, -1)), Segment(End(147, -1), End(148, 1))),
                        ),
                        Bridge(
                            Segment(End(184, -1), End(185, 1)),
                            Segment(End(185, 1), End(186, -1)),
                            Tunnel(0, Segment(End(166, 1), End(167, -1)), Segment(End(167, -1), End(168, 1))),
                        ),
                        Bridge(
                            Segment(End(186, -1), End(188, 1)),
                            Segment(End(188, 1), End(189, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(100, 1))),
                        ),
                        Bridge(
                            Segment(End(189, -1), End(190, 1)),
                            Segment(End(190, 1), End(191, -1)),
                            Tunnel(0, Segment(End(117, 1), End(118, -1)), Segment(End(118, -1), End(119, 1))),
                        ),
                        Bridge(
                            Segment(End(191, -1), End(192, 1)),
                            Segment(End(192, 1), End(193, -1)),
                            Tunnel(0, Segment(End(137, 1), End(138, -1)), Segment(End(138, -1), End(139, 1))),
                        ),
                        Bridge(
                            Segment(End(193, -1), End(194, 1)),
                            Segment(End(194, 1), End(195, -1)),
                            Tunnel(0, Segment(End(157, 1), End(158, -1)), Segment(End(158, -1), End(159, 1))),
                        ),
                        Bridge(
                            Segment(End(195, -1), End(196, 1)),
                            Segment(End(196, 1), End(197, -1)),
                            Tunnel(0, Segment(End(177, 1), End(178, -1)), Segment(End(178, -1), End(179, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_10_10(self):
        knot = Knot(10, 10)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(5, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(7, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(9, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(4, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(6, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(8, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(9, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(4, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(6, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(8, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(3, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(5, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(7, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(8, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(5, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(7, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(9, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(1, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(4, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(6, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(7, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(9, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(4, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(6, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(8, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(17, -1)),
                            Tunnel(2, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(5, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                    ],
                ),
                String(
                    4,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(6, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(8, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(5, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(7, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(9, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(3, Segment(End(15, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                    ],
                ),
                String(
                    5,
                    [
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(7, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(9, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(6, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(8, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(2, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(4, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                    ],
                ),
                String(
                    6,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(3, -1)),
                            Tunnel(5, Segment(End(-1, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(8, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(4, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(7, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(9, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(3, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                    ],
                ),
                String(
                    7,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(4, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(6, Segment(End(1, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(9, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(5, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(8, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(2, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                    ],
                ),
                String(
                    8,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(3, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(5, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(7, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(4, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(6, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(9, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                    ],
                ),
                String(
                    9,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(4, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(6, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(8, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(3, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(5, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(7, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_11_9(self):
        knot = Knot(11, 9)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(77, 1)),
                        Segment(End(77, 1), End(78, -1)),
                        Segment(End(78, -1), End(79, 1)),
                        Segment(End(79, 1), End(80, -1)),
                        Segment(End(80, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(84, 1)),
                        Segment(End(84, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(88, 1)),
                        Segment(End(88, 1), End(89, -1)),
                        Segment(End(89, -1), End(91, 1)),
                        Segment(End(91, 1), End(92, -1)),
                        Segment(End(92, -1), End(93, 1)),
                        Segment(End(93, 1), End(94, -1)),
                        Segment(End(94, -1), End(95, 1)),
                        Segment(End(95, 1), End(96, -1)),
                        Segment(End(96, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(100, 1)),
                        Segment(End(100, 1), End(101, -1)),
                        Segment(End(101, -1), End(102, 1)),
                        Segment(End(102, 1), End(103, -1)),
                        Segment(End(103, -1), End(104, 1)),
                        Segment(End(104, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(109, 1)),
                        Segment(End(109, 1), End(110, -1)),
                        Segment(End(110, -1), End(111, 1)),
                        Segment(End(111, 1), End(112, -1)),
                        Segment(End(112, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(120, 1)),
                        Segment(End(120, 1), End(121, -1)),
                        Segment(End(121, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(127, 1)),
                        Segment(End(127, 1), End(128, -1)),
                        Segment(End(128, -1), End(129, 1)),
                        Segment(End(129, 1), End(130, -1)),
                        Segment(End(130, -1), End(131, 1)),
                        Segment(End(131, 1), End(132, -1)),
                        Segment(End(132, -1), End(133, 1)),
                        Segment(End(133, 1), End(134, -1)),
                        Segment(End(134, -1), End(136, 1)),
                        Segment(End(136, 1), End(137, -1)),
                        Segment(End(137, -1), End(138, 1)),
                        Segment(End(138, 1), End(139, -1)),
                        Segment(End(139, -1), End(140, 1)),
                        Segment(End(140, 1), End(141, -1)),
                        Segment(End(141, -1), End(142, 1)),
                        Segment(End(142, 1), End(143, -1)),
                        Segment(End(143, -1), End(145, 1)),
                        Segment(End(145, 1), End(146, -1)),
                        Segment(End(146, -1), End(147, 1)),
                        Segment(End(147, 1), End(148, -1)),
                        Segment(End(148, -1), End(149, 1)),
                        Segment(End(149, 1), End(150, -1)),
                        Segment(End(150, -1), End(151, 1)),
                        Segment(End(151, 1), End(152, -1)),
                        Segment(End(152, -1), End(154, 1)),
                        Segment(End(154, 1), End(155, -1)),
                        Segment(End(155, -1), End(156, 1)),
                        Segment(End(156, 1), End(157, -1)),
                        Segment(End(157, -1), End(158, 1)),
                        Segment(End(158, 1), End(159, -1)),
                        Segment(End(159, -1), End(160, 1)),
                        Segment(End(160, 1), End(161, -1)),
                        Segment(End(161, -1), End(163, 1)),
                        Segment(End(163, 1), End(164, -1)),
                        Segment(End(164, -1), End(165, 1)),
                        Segment(End(165, 1), End(166, -1)),
                        Segment(End(166, -1), End(167, 1)),
                        Segment(End(167, 1), End(168, -1)),
                        Segment(End(168, -1), End(169, 1)),
                        Segment(End(169, 1), End(170, -1)),
                        Segment(End(170, -1), End(172, 1)),
                        Segment(End(172, 1), End(173, -1)),
                        Segment(End(173, -1), End(174, 1)),
                        Segment(End(174, 1), End(175, -1)),
                        Segment(End(175, -1), End(176, 1)),
                        Segment(End(176, 1), End(177, -1)),
                        Segment(End(177, -1), End(178, 1)),
                        Segment(End(178, 1), End(179, -1)),
                        Segment(End(179, -1), End(181, 1)),
                        Segment(End(181, 1), End(182, -1)),
                        Segment(End(182, -1), End(183, 1)),
                        Segment(End(183, 1), End(184, -1)),
                        Segment(End(184, -1), End(185, 1)),
                        Segment(End(185, 1), End(186, -1)),
                        Segment(End(186, -1), End(187, 1)),
                        Segment(End(187, 1), End(188, -1)),
                        Segment(End(188, -1), End(190, 1)),
                        Segment(End(190, 1), End(191, -1)),
                        Segment(End(191, -1), End(192, 1)),
                        Segment(End(192, 1), End(193, -1)),
                        Segment(End(193, -1), End(194, 1)),
                        Segment(End(194, 1), End(195, -1)),
                        Segment(End(195, -1), End(196, 1)),
                        Segment(End(196, 1), End(197, -1)),
                        Segment(End(197, -1), End(199, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(88, 1), End(89, -1)), Segment(End(89, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(100, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(77, 1), End(78, -1)), Segment(End(78, -1), End(79, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(109, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(88, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(95, 1), End(96, -1)), Segment(End(96, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(77, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(127, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(104, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(84, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(133, 1), End(134, -1)), Segment(End(134, -1), End(136, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(93, 1), End(94, -1)), Segment(End(94, -1), End(95, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(142, 1), End(143, -1)), Segment(End(143, -1), End(145, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(102, 1), End(103, -1)), Segment(End(103, -1), End(104, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(84, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(151, 1), End(152, -1)), Segment(End(152, -1), End(154, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(131, 1), End(132, -1)), Segment(End(132, -1), End(133, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(111, 1), End(112, -1)), Segment(End(112, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(91, 1), End(92, -1)), Segment(End(92, -1), End(93, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(160, 1), End(161, -1)), Segment(End(161, -1), End(163, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(140, 1), End(141, -1)), Segment(End(141, -1), End(142, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(77, 1)),
                            Segment(End(77, 1), End(78, -1)),
                            Tunnel(0, Segment(End(120, 1), End(121, -1)), Segment(End(121, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(78, -1), End(79, 1)),
                            Segment(End(79, 1), End(80, -1)),
                            Tunnel(0, Segment(End(100, 1), End(101, -1)), Segment(End(101, -1), End(102, 1))),
                        ),
                        Bridge(
                            Segment(End(80, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(169, 1), End(170, -1)), Segment(End(170, -1), End(172, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(84, 1)),
                            Segment(End(84, 1), End(85, -1)),
                            Tunnel(0, Segment(End(149, 1), End(150, -1)), Segment(End(150, -1), End(151, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(129, 1), End(130, -1)), Segment(End(130, -1), End(131, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(88, 1)),
                            Segment(End(88, 1), End(89, -1)),
                            Tunnel(0, Segment(End(109, 1), End(110, -1)), Segment(End(110, -1), End(111, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(91, 1)),
                            Segment(End(91, 1), End(92, -1)),
                            Tunnel(0, Segment(End(178, 1), End(179, -1)), Segment(End(179, -1), End(181, 1))),
                        ),
                        Bridge(
                            Segment(End(92, -1), End(93, 1)),
                            Segment(End(93, 1), End(94, -1)),
                            Tunnel(0, Segment(End(158, 1), End(159, -1)), Segment(End(159, -1), End(160, 1))),
                        ),
                        Bridge(
                            Segment(End(94, -1), End(95, 1)),
                            Segment(End(95, 1), End(96, -1)),
                            Tunnel(0, Segment(End(138, 1), End(139, -1)), Segment(End(139, -1), End(140, 1))),
                        ),
                        Bridge(
                            Segment(End(96, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(120, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(100, 1)),
                            Segment(End(100, 1), End(101, -1)),
                            Tunnel(0, Segment(End(187, 1), End(188, -1)), Segment(End(188, -1), End(190, 1))),
                        ),
                        Bridge(
                            Segment(End(101, -1), End(102, 1)),
                            Segment(End(102, 1), End(103, -1)),
                            Tunnel(0, Segment(End(167, 1), End(168, -1)), Segment(End(168, -1), End(169, 1))),
                        ),
                        Bridge(
                            Segment(End(103, -1), End(104, 1)),
                            Segment(End(104, 1), End(105, -1)),
                            Tunnel(0, Segment(End(147, 1), End(148, -1)), Segment(End(148, -1), End(149, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(127, 1), End(128, -1)), Segment(End(128, -1), End(129, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(109, 1)),
                            Segment(End(109, 1), End(110, -1)),
                            Tunnel(0, Segment(End(196, 1), End(197, -1)), Segment(End(197, -1), End(199, 1))),
                        ),
                        Bridge(
                            Segment(End(110, -1), End(111, 1)),
                            Segment(End(111, 1), End(112, -1)),
                            Tunnel(0, Segment(End(176, 1), End(177, -1)), Segment(End(177, -1), End(178, 1))),
                        ),
                        Bridge(
                            Segment(End(112, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(156, 1), End(157, -1)), Segment(End(157, -1), End(158, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(136, 1), End(137, -1)), Segment(End(137, -1), End(138, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(119, -1), End(120, 1)),
                            Segment(End(120, 1), End(121, -1)),
                            Tunnel(0, Segment(End(185, 1), End(186, -1)), Segment(End(186, -1), End(187, 1))),
                        ),
                        Bridge(
                            Segment(End(121, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(165, 1), End(166, -1)), Segment(End(166, -1), End(167, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(145, 1), End(146, -1)), Segment(End(146, -1), End(147, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(127, 1)),
                            Segment(End(127, 1), End(128, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(128, -1), End(129, 1)),
                            Segment(End(129, 1), End(130, -1)),
                            Tunnel(0, Segment(End(194, 1), End(195, -1)), Segment(End(195, -1), End(196, 1))),
                        ),
                        Bridge(
                            Segment(End(130, -1), End(131, 1)),
                            Segment(End(131, 1), End(132, -1)),
                            Tunnel(0, Segment(End(174, 1), End(175, -1)), Segment(End(175, -1), End(176, 1))),
                        ),
                        Bridge(
                            Segment(End(132, -1), End(133, 1)),
                            Segment(End(133, 1), End(134, -1)),
                            Tunnel(0, Segment(End(154, 1), End(155, -1)), Segment(End(155, -1), End(156, 1))),
                        ),
                        Bridge(
                            Segment(End(134, -1), End(136, 1)),
                            Segment(End(136, 1), End(137, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(137, -1), End(138, 1)),
                            Segment(End(138, 1), End(139, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(139, -1), End(140, 1)),
                            Segment(End(140, 1), End(141, -1)),
                            Tunnel(0, Segment(End(183, 1), End(184, -1)), Segment(End(184, -1), End(185, 1))),
                        ),
                        Bridge(
                            Segment(End(141, -1), End(142, 1)),
                            Segment(End(142, 1), End(143, -1)),
                            Tunnel(0, Segment(End(163, 1), End(164, -1)), Segment(End(164, -1), End(165, 1))),
                        ),
                        Bridge(
                            Segment(End(143, -1), End(145, 1)),
                            Segment(End(145, 1), End(146, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(146, -1), End(147, 1)),
                            Segment(End(147, 1), End(148, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(148, -1), End(149, 1)),
                            Segment(End(149, 1), End(150, -1)),
                            Tunnel(0, Segment(End(192, 1), End(193, -1)), Segment(End(193, -1), End(194, 1))),
                        ),
                        Bridge(
                            Segment(End(150, -1), End(151, 1)),
                            Segment(End(151, 1), End(152, -1)),
                            Tunnel(0, Segment(End(172, 1), End(173, -1)), Segment(End(173, -1), End(174, 1))),
                        ),
                        Bridge(
                            Segment(End(152, -1), End(154, 1)),
                            Segment(End(154, 1), End(155, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(155, -1), End(156, 1)),
                            Segment(End(156, 1), End(157, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(157, -1), End(158, 1)),
                            Segment(End(158, 1), End(159, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(159, -1), End(160, 1)),
                            Segment(End(160, 1), End(161, -1)),
                            Tunnel(0, Segment(End(181, 1), End(182, -1)), Segment(End(182, -1), End(183, 1))),
                        ),
                        Bridge(
                            Segment(End(161, -1), End(163, 1)),
                            Segment(End(163, 1), End(164, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(164, -1), End(165, 1)),
                            Segment(End(165, 1), End(166, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(166, -1), End(167, 1)),
                            Segment(End(167, 1), End(168, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(168, -1), End(169, 1)),
                            Segment(End(169, 1), End(170, -1)),
                            Tunnel(0, Segment(End(190, 1), End(191, -1)), Segment(End(191, -1), End(192, 1))),
                        ),
                        Bridge(
                            Segment(End(170, -1), End(172, 1)),
                            Segment(End(172, 1), End(173, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(173, -1), End(174, 1)),
                            Segment(End(174, 1), End(175, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(175, -1), End(176, 1)),
                            Segment(End(176, 1), End(177, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(177, -1), End(178, 1)),
                            Segment(End(178, 1), End(179, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(179, -1), End(181, 1)),
                            Segment(End(181, 1), End(182, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(182, -1), End(183, 1)),
                            Segment(End(183, 1), End(184, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(184, -1), End(185, 1)),
                            Segment(End(185, 1), End(186, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(186, -1), End(187, 1)),
                            Segment(End(187, 1), End(188, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(188, -1), End(190, 1)),
                            Segment(End(190, 1), End(191, -1)),
                            Tunnel(0, Segment(End(79, 1), End(80, -1)), Segment(End(80, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(191, -1), End(192, 1)),
                            Segment(End(192, 1), End(193, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(193, -1), End(194, 1)),
                            Segment(End(194, 1), End(195, -1)),
                            Tunnel(0, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(195, -1), End(196, 1)),
                            Segment(End(196, 1), End(197, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_12_8(self):
        knot = Knot(12, 8)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(3, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(3, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(1, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(25, -1)),
                            Tunnel(3, Segment(End(45, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(2, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(1, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(3, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(41, -1)),
                            Tunnel(3, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(2, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(2, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(27, -1)),
                        Segment(End(27, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(3, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(2, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(0, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(3, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(1, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(3, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(2, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(2, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(27, -1)),
                            Tunnel(0, Segment(End(23, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(3, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(2, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(2, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(43, -1)),
                            Tunnel(0, Segment(End(39, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(3, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(1, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(3, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(1, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(2, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(3, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(1, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(3, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(29, -1)),
                            Tunnel(1, Segment(End(25, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(3, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(3, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(45, -1)),
                            Tunnel(1, Segment(End(41, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(3, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(2, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(3, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(1, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(2, Segment(End(27, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(1, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(2, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(47, -1)),
                            Tunnel(2, Segment(End(43, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_9_12(self):
        knot = Knot(9, 12)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(2, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(2, Segment(End(63, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(2, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(1, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(2, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(1, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(37, -1)),
                            Tunnel(2, Segment(End(15, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(2, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(1, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(1, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(1, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(2, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(72, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(61, -1)),
                            Tunnel(2, Segment(End(39, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(2, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(2, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(1, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(72, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(1, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(2, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(1, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(2, Segment(End(26, 1), End(27, -1)), Segment(End(27, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(1, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(2, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(39, -1)),
                            Tunnel(0, Segment(End(35, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(2, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(2, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(2, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(1, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(72, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(2, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(1, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(63, -1)),
                            Tunnel(0, Segment(End(59, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(1, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(26, 1)),
                        Segment(End(26, 1), End(27, -1)),
                        Segment(End(27, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(72, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(2, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(26, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(2, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(17, -1)),
                            Tunnel(1, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(1, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(2, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(26, 1)),
                            Segment(End(26, 1), End(27, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(27, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(2, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(1, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(2, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(41, -1)),
                            Tunnel(1, Segment(End(37, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(1, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(1, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(1, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(2, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(65, -1)),
                            Tunnel(1, Segment(End(61, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(1, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(2, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_10_11(self):
        knot = Knot(10, 11)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(84, 1)),
                        Segment(End(84, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(89, 1)),
                        Segment(End(89, 1), End(90, -1)),
                        Segment(End(90, -1), End(91, 1)),
                        Segment(End(91, 1), End(92, -1)),
                        Segment(End(92, -1), End(93, 1)),
                        Segment(End(93, 1), End(94, -1)),
                        Segment(End(94, -1), End(95, 1)),
                        Segment(End(95, 1), End(96, -1)),
                        Segment(End(96, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(100, 1)),
                        Segment(End(100, 1), End(101, -1)),
                        Segment(End(101, -1), End(102, 1)),
                        Segment(End(102, 1), End(103, -1)),
                        Segment(End(103, -1), End(104, 1)),
                        Segment(End(104, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(111, 1)),
                        Segment(End(111, 1), End(112, -1)),
                        Segment(End(112, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(117, 1)),
                        Segment(End(117, 1), End(118, -1)),
                        Segment(End(118, -1), End(119, 1)),
                        Segment(End(119, 1), End(120, -1)),
                        Segment(End(120, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(126, 1)),
                        Segment(End(126, 1), End(127, -1)),
                        Segment(End(127, -1), End(128, 1)),
                        Segment(End(128, 1), End(129, -1)),
                        Segment(End(129, -1), End(130, 1)),
                        Segment(End(130, 1), End(131, -1)),
                        Segment(End(131, -1), End(133, 1)),
                        Segment(End(133, 1), End(134, -1)),
                        Segment(End(134, -1), End(135, 1)),
                        Segment(End(135, 1), End(136, -1)),
                        Segment(End(136, -1), End(137, 1)),
                        Segment(End(137, 1), End(138, -1)),
                        Segment(End(138, -1), End(139, 1)),
                        Segment(End(139, 1), End(140, -1)),
                        Segment(End(140, -1), End(141, 1)),
                        Segment(End(141, 1), End(142, -1)),
                        Segment(End(142, -1), End(144, 1)),
                        Segment(End(144, 1), End(145, -1)),
                        Segment(End(145, -1), End(146, 1)),
                        Segment(End(146, 1), End(147, -1)),
                        Segment(End(147, -1), End(148, 1)),
                        Segment(End(148, 1), End(149, -1)),
                        Segment(End(149, -1), End(150, 1)),
                        Segment(End(150, 1), End(151, -1)),
                        Segment(End(151, -1), End(152, 1)),
                        Segment(End(152, 1), End(153, -1)),
                        Segment(End(153, -1), End(155, 1)),
                        Segment(End(155, 1), End(156, -1)),
                        Segment(End(156, -1), End(157, 1)),
                        Segment(End(157, 1), End(158, -1)),
                        Segment(End(158, -1), End(159, 1)),
                        Segment(End(159, 1), End(160, -1)),
                        Segment(End(160, -1), End(161, 1)),
                        Segment(End(161, 1), End(162, -1)),
                        Segment(End(162, -1), End(163, 1)),
                        Segment(End(163, 1), End(164, -1)),
                        Segment(End(164, -1), End(166, 1)),
                        Segment(End(166, 1), End(167, -1)),
                        Segment(End(167, -1), End(168, 1)),
                        Segment(End(168, 1), End(169, -1)),
                        Segment(End(169, -1), End(170, 1)),
                        Segment(End(170, 1), End(171, -1)),
                        Segment(End(171, -1), End(172, 1)),
                        Segment(End(172, 1), End(173, -1)),
                        Segment(End(173, -1), End(174, 1)),
                        Segment(End(174, 1), End(175, -1)),
                        Segment(End(175, -1), End(177, 1)),
                        Segment(End(177, 1), End(178, -1)),
                        Segment(End(178, -1), End(179, 1)),
                        Segment(End(179, 1), End(180, -1)),
                        Segment(End(180, -1), End(181, 1)),
                        Segment(End(181, 1), End(182, -1)),
                        Segment(End(182, -1), End(183, 1)),
                        Segment(End(183, 1), End(184, -1)),
                        Segment(End(184, -1), End(185, 1)),
                        Segment(End(185, 1), End(186, -1)),
                        Segment(End(186, -1), End(188, 1)),
                        Segment(End(188, 1), End(189, -1)),
                        Segment(End(189, -1), End(190, 1)),
                        Segment(End(190, 1), End(191, -1)),
                        Segment(End(191, -1), End(192, 1)),
                        Segment(End(192, 1), End(193, -1)),
                        Segment(End(193, -1), End(194, 1)),
                        Segment(End(194, 1), End(195, -1)),
                        Segment(End(195, -1), End(196, 1)),
                        Segment(End(196, 1), End(197, -1)),
                        Segment(End(197, -1), End(199, 1)),
                        Segment(End(199, 1), End(200, -1)),
                        Segment(End(200, -1), End(201, 1)),
                        Segment(End(201, 1), End(202, -1)),
                        Segment(End(202, -1), End(203, 1)),
                        Segment(End(203, 1), End(204, -1)),
                        Segment(End(204, -1), End(205, 1)),
                        Segment(End(205, 1), End(206, -1)),
                        Segment(End(206, -1), End(207, 1)),
                        Segment(End(207, 1), End(208, -1)),
                        Segment(End(208, -1), End(210, 1)),
                        Segment(End(210, 1), End(211, -1)),
                        Segment(End(211, -1), End(212, 1)),
                        Segment(End(212, 1), End(213, -1)),
                        Segment(End(213, -1), End(214, 1)),
                        Segment(End(214, 1), End(215, -1)),
                        Segment(End(215, -1), End(216, 1)),
                        Segment(End(216, 1), End(217, -1)),
                        Segment(End(217, -1), End(218, 1)),
                        Segment(End(218, 1), End(219, -1)),
                        Segment(End(219, -1), End(221, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(104, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(146, 1), End(147, -1)), Segment(End(147, -1), End(148, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(188, 1), End(189, -1)), Segment(End(189, -1), End(190, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(117, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(157, 1), End(158, -1)), Segment(End(158, -1), End(159, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(199, 1), End(200, -1)), Segment(End(200, -1), End(201, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(84, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(126, 1), End(127, -1)), Segment(End(127, -1), End(128, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(168, 1), End(169, -1)), Segment(End(169, -1), End(170, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(210, 1), End(211, -1)), Segment(End(211, -1), End(212, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(95, 1), End(96, -1)), Segment(End(96, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(137, 1), End(138, -1)), Segment(End(138, -1), End(139, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(179, 1), End(180, -1)), Segment(End(180, -1), End(181, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(148, 1), End(149, -1)), Segment(End(149, -1), End(150, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(190, 1), End(191, -1)), Segment(End(191, -1), End(192, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(117, 1), End(118, -1)), Segment(End(118, -1), End(119, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(159, 1), End(160, -1)), Segment(End(160, -1), End(161, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(201, 1), End(202, -1)), Segment(End(202, -1), End(203, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(128, 1), End(129, -1)), Segment(End(129, -1), End(130, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(170, 1), End(171, -1)), Segment(End(171, -1), End(172, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(212, 1), End(213, -1)), Segment(End(213, -1), End(214, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(100, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(139, 1), End(140, -1)), Segment(End(140, -1), End(141, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(181, 1), End(182, -1)), Segment(End(182, -1), End(183, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(84, 1)),
                            Segment(End(84, 1), End(85, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(89, 1)),
                            Segment(End(89, 1), End(90, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(111, 1))),
                        ),
                        Bridge(
                            Segment(End(90, -1), End(91, 1)),
                            Segment(End(91, 1), End(92, -1)),
                            Tunnel(0, Segment(End(150, 1), End(151, -1)), Segment(End(151, -1), End(152, 1))),
                        ),
                        Bridge(
                            Segment(End(92, -1), End(93, 1)),
                            Segment(End(93, 1), End(94, -1)),
                            Tunnel(0, Segment(End(192, 1), End(193, -1)), Segment(End(193, -1), End(194, 1))),
                        ),
                        Bridge(
                            Segment(End(94, -1), End(95, 1)),
                            Segment(End(95, 1), End(96, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(96, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(100, 1)),
                            Segment(End(100, 1), End(101, -1)),
                            Tunnel(0, Segment(End(119, 1), End(120, -1)), Segment(End(120, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(101, -1), End(102, 1)),
                            Segment(End(102, 1), End(103, -1)),
                            Tunnel(0, Segment(End(161, 1), End(162, -1)), Segment(End(162, -1), End(163, 1))),
                        ),
                        Bridge(
                            Segment(End(103, -1), End(104, 1)),
                            Segment(End(104, 1), End(105, -1)),
                            Tunnel(0, Segment(End(203, 1), End(204, -1)), Segment(End(204, -1), End(205, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(111, 1)),
                            Segment(End(111, 1), End(112, -1)),
                            Tunnel(0, Segment(End(130, 1), End(131, -1)), Segment(End(131, -1), End(133, 1))),
                        ),
                        Bridge(
                            Segment(End(112, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(172, 1), End(173, -1)), Segment(End(173, -1), End(174, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(214, 1), End(215, -1)), Segment(End(215, -1), End(216, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(117, 1)),
                            Segment(End(117, 1), End(118, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(118, -1), End(119, 1)),
                            Segment(End(119, 1), End(120, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(120, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(141, 1), End(142, -1)), Segment(End(142, -1), End(144, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(183, 1), End(184, -1)), Segment(End(184, -1), End(185, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(126, 1)),
                            Segment(End(126, 1), End(127, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(127, -1), End(128, 1)),
                            Segment(End(128, 1), End(129, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(129, -1), End(130, 1)),
                            Segment(End(130, 1), End(131, -1)),
                            Tunnel(0, Segment(End(89, 1), End(90, -1)), Segment(End(90, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(131, -1), End(133, 1)),
                            Segment(End(133, 1), End(134, -1)),
                            Tunnel(0, Segment(End(152, 1), End(153, -1)), Segment(End(153, -1), End(155, 1))),
                        ),
                        Bridge(
                            Segment(End(134, -1), End(135, 1)),
                            Segment(End(135, 1), End(136, -1)),
                            Tunnel(0, Segment(End(194, 1), End(195, -1)), Segment(End(195, -1), End(196, 1))),
                        ),
                        Bridge(
                            Segment(End(136, -1), End(137, 1)),
                            Segment(End(137, 1), End(138, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(138, -1), End(139, 1)),
                            Segment(End(139, 1), End(140, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(140, -1), End(141, 1)),
                            Segment(End(141, 1), End(142, -1)),
                            Tunnel(0, Segment(End(100, 1), End(101, -1)), Segment(End(101, -1), End(102, 1))),
                        ),
                        Bridge(
                            Segment(End(142, -1), End(144, 1)),
                            Segment(End(144, 1), End(145, -1)),
                            Tunnel(0, Segment(End(163, 1), End(164, -1)), Segment(End(164, -1), End(166, 1))),
                        ),
                        Bridge(
                            Segment(End(145, -1), End(146, 1)),
                            Segment(End(146, 1), End(147, -1)),
                            Tunnel(0, Segment(End(205, 1), End(206, -1)), Segment(End(206, -1), End(207, 1))),
                        ),
                        Bridge(
                            Segment(End(147, -1), End(148, 1)),
                            Segment(End(148, 1), End(149, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(149, -1), End(150, 1)),
                            Segment(End(150, 1), End(151, -1)),
                            Tunnel(0, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(151, -1), End(152, 1)),
                            Segment(End(152, 1), End(153, -1)),
                            Tunnel(0, Segment(End(111, 1), End(112, -1)), Segment(End(112, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(153, -1), End(155, 1)),
                            Segment(End(155, 1), End(156, -1)),
                            Tunnel(0, Segment(End(174, 1), End(175, -1)), Segment(End(175, -1), End(177, 1))),
                        ),
                        Bridge(
                            Segment(End(156, -1), End(157, 1)),
                            Segment(End(157, 1), End(158, -1)),
                            Tunnel(0, Segment(End(216, 1), End(217, -1)), Segment(End(217, -1), End(218, 1))),
                        ),
                        Bridge(
                            Segment(End(158, -1), End(159, 1)),
                            Segment(End(159, 1), End(160, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(160, -1), End(161, 1)),
                            Segment(End(161, 1), End(162, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(162, -1), End(163, 1)),
                            Segment(End(163, 1), End(164, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(164, -1), End(166, 1)),
                            Segment(End(166, 1), End(167, -1)),
                            Tunnel(0, Segment(End(185, 1), End(186, -1)), Segment(End(186, -1), End(188, 1))),
                        ),
                        Bridge(
                            Segment(End(167, -1), End(168, 1)),
                            Segment(End(168, 1), End(169, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(169, -1), End(170, 1)),
                            Segment(End(170, 1), End(171, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(171, -1), End(172, 1)),
                            Segment(End(172, 1), End(173, -1)),
                            Tunnel(0, Segment(End(91, 1), End(92, -1)), Segment(End(92, -1), End(93, 1))),
                        ),
                        Bridge(
                            Segment(End(173, -1), End(174, 1)),
                            Segment(End(174, 1), End(175, -1)),
                            Tunnel(0, Segment(End(133, 1), End(134, -1)), Segment(End(134, -1), End(135, 1))),
                        ),
                        Bridge(
                            Segment(End(175, -1), End(177, 1)),
                            Segment(End(177, 1), End(178, -1)),
                            Tunnel(0, Segment(End(196, 1), End(197, -1)), Segment(End(197, -1), End(199, 1))),
                        ),
                        Bridge(
                            Segment(End(178, -1), End(179, 1)),
                            Segment(End(179, 1), End(180, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(180, -1), End(181, 1)),
                            Segment(End(181, 1), End(182, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(182, -1), End(183, 1)),
                            Segment(End(183, 1), End(184, -1)),
                            Tunnel(0, Segment(End(102, 1), End(103, -1)), Segment(End(103, -1), End(104, 1))),
                        ),
                        Bridge(
                            Segment(End(184, -1), End(185, 1)),
                            Segment(End(185, 1), End(186, -1)),
                            Tunnel(0, Segment(End(144, 1), End(145, -1)), Segment(End(145, -1), End(146, 1))),
                        ),
                        Bridge(
                            Segment(End(186, -1), End(188, 1)),
                            Segment(End(188, 1), End(189, -1)),
                            Tunnel(0, Segment(End(207, 1), End(208, -1)), Segment(End(208, -1), End(210, 1))),
                        ),
                        Bridge(
                            Segment(End(189, -1), End(190, 1)),
                            Segment(End(190, 1), End(191, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(191, -1), End(192, 1)),
                            Segment(End(192, 1), End(193, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(193, -1), End(194, 1)),
                            Segment(End(194, 1), End(195, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(195, -1), End(196, 1)),
                            Segment(End(196, 1), End(197, -1)),
                            Tunnel(0, Segment(End(155, 1), End(156, -1)), Segment(End(156, -1), End(157, 1))),
                        ),
                        Bridge(
                            Segment(End(197, -1), End(199, 1)),
                            Segment(End(199, 1), End(200, -1)),
                            Tunnel(0, Segment(End(218, 1), End(219, -1)), Segment(End(219, -1), End(221, 1))),
                        ),
                        Bridge(
                            Segment(End(200, -1), End(201, 1)),
                            Segment(End(201, 1), End(202, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(202, -1), End(203, 1)),
                            Segment(End(203, 1), End(204, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(84, 1))),
                        ),
                        Bridge(
                            Segment(End(204, -1), End(205, 1)),
                            Segment(End(205, 1), End(206, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(126, 1))),
                        ),
                        Bridge(
                            Segment(End(206, -1), End(207, 1)),
                            Segment(End(207, 1), End(208, -1)),
                            Tunnel(0, Segment(End(166, 1), End(167, -1)), Segment(End(167, -1), End(168, 1))),
                        ),
                        Bridge(
                            Segment(End(208, -1), End(210, 1)),
                            Segment(End(210, 1), End(211, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(211, -1), End(212, 1)),
                            Segment(End(212, 1), End(213, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(213, -1), End(214, 1)),
                            Segment(End(214, 1), End(215, -1)),
                            Tunnel(0, Segment(End(93, 1), End(94, -1)), Segment(End(94, -1), End(95, 1))),
                        ),
                        Bridge(
                            Segment(End(215, -1), End(216, 1)),
                            Segment(End(216, 1), End(217, -1)),
                            Tunnel(0, Segment(End(135, 1), End(136, -1)), Segment(End(136, -1), End(137, 1))),
                        ),
                        Bridge(
                            Segment(End(217, -1), End(218, 1)),
                            Segment(End(218, 1), End(219, -1)),
                            Tunnel(0, Segment(End(177, 1), End(178, -1)), Segment(End(178, -1), End(179, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_11_10(self):
        knot = Knot(11, 10)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(71, -1)),
                        Segment(End(71, -1), End(72, 1)),
                        Segment(End(72, 1), End(73, -1)),
                        Segment(End(73, -1), End(74, 1)),
                        Segment(End(74, 1), End(75, -1)),
                        Segment(End(75, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(84, -1)),
                        Segment(End(84, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(88, -1)),
                        Segment(End(88, -1), End(89, 1)),
                        Segment(End(89, 1), End(91, -1)),
                        Segment(End(91, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(96, 1)),
                        Segment(End(96, 1), End(97, -1)),
                        Segment(End(97, -1), End(98, 1)),
                        Segment(End(98, 1), End(99, -1)),
                        Segment(End(99, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(105, 1)),
                        Segment(End(105, 1), End(106, -1)),
                        Segment(End(106, -1), End(107, 1)),
                        Segment(End(107, 1), End(108, -1)),
                        Segment(End(108, -1), End(109, 1)),
                        Segment(End(109, 1), End(111, -1)),
                        Segment(End(111, -1), End(112, 1)),
                        Segment(End(112, 1), End(113, -1)),
                        Segment(End(113, -1), End(114, 1)),
                        Segment(End(114, 1), End(115, -1)),
                        Segment(End(115, -1), End(116, 1)),
                        Segment(End(116, 1), End(117, -1)),
                        Segment(End(117, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(121, 1)),
                        Segment(End(121, 1), End(122, -1)),
                        Segment(End(122, -1), End(123, 1)),
                        Segment(End(123, 1), End(124, -1)),
                        Segment(End(124, -1), End(125, 1)),
                        Segment(End(125, 1), End(126, -1)),
                        Segment(End(126, -1), End(127, 1)),
                        Segment(End(127, 1), End(128, -1)),
                        Segment(End(128, -1), End(129, 1)),
                        Segment(End(129, 1), End(131, -1)),
                        Segment(End(131, -1), End(132, 1)),
                        Segment(End(132, 1), End(133, -1)),
                        Segment(End(133, -1), End(134, 1)),
                        Segment(End(134, 1), End(135, -1)),
                        Segment(End(135, -1), End(136, 1)),
                        Segment(End(136, 1), End(137, -1)),
                        Segment(End(137, -1), End(138, 1)),
                        Segment(End(138, 1), End(139, -1)),
                        Segment(End(139, -1), End(141, 1)),
                        Segment(End(141, 1), End(142, -1)),
                        Segment(End(142, -1), End(143, 1)),
                        Segment(End(143, 1), End(144, -1)),
                        Segment(End(144, -1), End(145, 1)),
                        Segment(End(145, 1), End(146, -1)),
                        Segment(End(146, -1), End(147, 1)),
                        Segment(End(147, 1), End(148, -1)),
                        Segment(End(148, -1), End(149, 1)),
                        Segment(End(149, 1), End(151, -1)),
                        Segment(End(151, -1), End(152, 1)),
                        Segment(End(152, 1), End(153, -1)),
                        Segment(End(153, -1), End(154, 1)),
                        Segment(End(154, 1), End(155, -1)),
                        Segment(End(155, -1), End(156, 1)),
                        Segment(End(156, 1), End(157, -1)),
                        Segment(End(157, -1), End(158, 1)),
                        Segment(End(158, 1), End(159, -1)),
                        Segment(End(159, -1), End(161, 1)),
                        Segment(End(161, 1), End(162, -1)),
                        Segment(End(162, -1), End(163, 1)),
                        Segment(End(163, 1), End(164, -1)),
                        Segment(End(164, -1), End(165, 1)),
                        Segment(End(165, 1), End(166, -1)),
                        Segment(End(166, -1), End(167, 1)),
                        Segment(End(167, 1), End(168, -1)),
                        Segment(End(168, -1), End(169, 1)),
                        Segment(End(169, 1), End(171, -1)),
                        Segment(End(171, -1), End(172, 1)),
                        Segment(End(172, 1), End(173, -1)),
                        Segment(End(173, -1), End(174, 1)),
                        Segment(End(174, 1), End(175, -1)),
                        Segment(End(175, -1), End(176, 1)),
                        Segment(End(176, 1), End(177, -1)),
                        Segment(End(177, -1), End(178, 1)),
                        Segment(End(178, 1), End(179, -1)),
                        Segment(End(179, -1), End(181, 1)),
                        Segment(End(181, 1), End(182, -1)),
                        Segment(End(182, -1), End(183, 1)),
                        Segment(End(183, 1), End(184, -1)),
                        Segment(End(184, -1), End(185, 1)),
                        Segment(End(185, 1), End(186, -1)),
                        Segment(End(186, -1), End(187, 1)),
                        Segment(End(187, 1), End(188, -1)),
                        Segment(End(188, -1), End(189, 1)),
                        Segment(End(189, 1), End(191, -1)),
                        Segment(End(191, -1), End(192, 1)),
                        Segment(End(192, 1), End(193, -1)),
                        Segment(End(193, -1), End(194, 1)),
                        Segment(End(194, 1), End(195, -1)),
                        Segment(End(195, -1), End(196, 1)),
                        Segment(End(196, 1), End(197, -1)),
                        Segment(End(197, -1), End(198, 1)),
                        Segment(End(198, 1), End(199, -1)),
                        Segment(End(199, -1), End(201, 1)),
                        Segment(End(201, 1), End(202, -1)),
                        Segment(End(202, -1), End(203, 1)),
                        Segment(End(203, 1), End(204, -1)),
                        Segment(End(204, -1), End(205, 1)),
                        Segment(End(205, 1), End(206, -1)),
                        Segment(End(206, -1), End(207, 1)),
                        Segment(End(207, 1), End(208, -1)),
                        Segment(End(208, -1), End(209, 1)),
                        Segment(End(209, 1), End(211, -1)),
                        Segment(End(211, -1), End(212, 1)),
                        Segment(End(212, 1), End(213, -1)),
                        Segment(End(213, -1), End(214, 1)),
                        Segment(End(214, 1), End(215, -1)),
                        Segment(End(215, -1), End(216, 1)),
                        Segment(End(216, 1), End(217, -1)),
                        Segment(End(217, -1), End(218, 1)),
                        Segment(End(218, 1), End(219, -1)),
                        Segment(End(219, -1), End(221, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(198, 1), End(199, -1)), Segment(End(199, -1), End(201, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(156, 1), End(157, -1)), Segment(End(157, -1), End(158, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(114, 1), End(115, -1)), Segment(End(115, -1), End(116, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(72, 1), End(73, -1)), Segment(End(73, -1), End(74, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(187, 1), End(188, -1)), Segment(End(188, -1), End(189, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(145, 1), End(146, -1)), Segment(End(146, -1), End(147, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(105, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(218, 1), End(219, -1)), Segment(End(219, -1), End(221, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(176, 1), End(177, -1)), Segment(End(177, -1), End(178, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(134, 1), End(135, -1)), Segment(End(135, -1), End(136, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(0, Segment(End(49, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(207, 1), End(208, -1)), Segment(End(208, -1), End(209, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(165, 1), End(166, -1)), Segment(End(166, -1), End(167, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(123, 1), End(124, -1)), Segment(End(124, -1), End(125, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(196, 1), End(197, -1)), Segment(End(197, -1), End(198, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(154, 1), End(155, -1)), Segment(End(155, -1), End(156, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(112, 1), End(113, -1)), Segment(End(113, -1), End(114, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(51, -1)),
                            Tunnel(0, Segment(End(69, 1), End(71, -1)), Segment(End(71, -1), End(72, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(185, 1), End(186, -1)), Segment(End(186, -1), End(187, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(143, 1), End(144, -1)), Segment(End(144, -1), End(145, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(0, Segment(End(216, 1), End(217, -1)), Segment(End(217, -1), End(218, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(0, Segment(End(174, 1), End(175, -1)), Segment(End(175, -1), End(176, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(132, 1), End(133, -1)), Segment(End(133, -1), End(134, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(71, -1)),
                            Tunnel(0, Segment(End(89, 1), End(91, -1)), Segment(End(91, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(72, 1)),
                            Segment(End(72, 1), End(73, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(73, -1), End(74, 1)),
                            Segment(End(74, 1), End(75, -1)),
                            Tunnel(0, Segment(End(205, 1), End(206, -1)), Segment(End(206, -1), End(207, 1))),
                        ),
                        Bridge(
                            Segment(End(75, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(163, 1), End(164, -1)), Segment(End(164, -1), End(165, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(121, 1), End(122, -1)), Segment(End(122, -1), End(123, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(84, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(84, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(0, Segment(End(194, 1), End(195, -1)), Segment(End(195, -1), End(196, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(88, -1)),
                            Tunnel(0, Segment(End(152, 1), End(153, -1)), Segment(End(153, -1), End(154, 1))),
                        ),
                        Bridge(
                            Segment(End(88, -1), End(89, 1)),
                            Segment(End(89, 1), End(91, -1)),
                            Tunnel(0, Segment(End(109, 1), End(111, -1)), Segment(End(111, -1), End(112, 1))),
                        ),
                        Bridge(
                            Segment(End(91, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(96, 1)),
                            Segment(End(96, 1), End(97, -1)),
                            Tunnel(0, Segment(End(183, 1), End(184, -1)), Segment(End(184, -1), End(185, 1))),
                        ),
                        Bridge(
                            Segment(End(97, -1), End(98, 1)),
                            Segment(End(98, 1), End(99, -1)),
                            Tunnel(0, Segment(End(141, 1), End(142, -1)), Segment(End(142, -1), End(143, 1))),
                        ),
                        Bridge(
                            Segment(End(99, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(105, 1)),
                            Segment(End(105, 1), End(106, -1)),
                            Tunnel(0, Segment(End(214, 1), End(215, -1)), Segment(End(215, -1), End(216, 1))),
                        ),
                        Bridge(
                            Segment(End(106, -1), End(107, 1)),
                            Segment(End(107, 1), End(108, -1)),
                            Tunnel(0, Segment(End(172, 1), End(173, -1)), Segment(End(173, -1), End(174, 1))),
                        ),
                        Bridge(
                            Segment(End(108, -1), End(109, 1)),
                            Segment(End(109, 1), End(111, -1)),
                            Tunnel(0, Segment(End(129, 1), End(131, -1)), Segment(End(131, -1), End(132, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(112, 1)),
                            Segment(End(112, 1), End(113, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(113, -1), End(114, 1)),
                            Segment(End(114, 1), End(115, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(115, -1), End(116, 1)),
                            Segment(End(116, 1), End(117, -1)),
                            Tunnel(0, Segment(End(203, 1), End(204, -1)), Segment(End(204, -1), End(205, 1))),
                        ),
                        Bridge(
                            Segment(End(117, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(161, 1), End(162, -1)), Segment(End(162, -1), End(163, 1))),
                        ),
                        Bridge(
                            Segment(End(119, -1), End(121, 1)),
                            Segment(End(121, 1), End(122, -1)),
                            Tunnel(0, Segment(End(98, 1), End(99, -1)), Segment(End(99, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(122, -1), End(123, 1)),
                            Segment(End(123, 1), End(124, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(124, -1), End(125, 1)),
                            Segment(End(125, 1), End(126, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(126, -1), End(127, 1)),
                            Segment(End(127, 1), End(128, -1)),
                            Tunnel(0, Segment(End(192, 1), End(193, -1)), Segment(End(193, -1), End(194, 1))),
                        ),
                        Bridge(
                            Segment(End(128, -1), End(129, 1)),
                            Segment(End(129, 1), End(131, -1)),
                            Tunnel(0, Segment(End(149, 1), End(151, -1)), Segment(End(151, -1), End(152, 1))),
                        ),
                        Bridge(
                            Segment(End(131, -1), End(132, 1)),
                            Segment(End(132, 1), End(133, -1)),
                            Tunnel(0, Segment(End(87, 1), End(88, -1)), Segment(End(88, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(133, -1), End(134, 1)),
                            Segment(End(134, 1), End(135, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(135, -1), End(136, 1)),
                            Segment(End(136, 1), End(137, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(137, -1), End(138, 1)),
                            Segment(End(138, 1), End(139, -1)),
                            Tunnel(0, Segment(End(181, 1), End(182, -1)), Segment(End(182, -1), End(183, 1))),
                        ),
                        Bridge(
                            Segment(End(139, -1), End(141, 1)),
                            Segment(End(141, 1), End(142, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(121, 1))),
                        ),
                        Bridge(
                            Segment(End(142, -1), End(143, 1)),
                            Segment(End(143, 1), End(144, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(144, -1), End(145, 1)),
                            Segment(End(145, 1), End(146, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(146, -1), End(147, 1)),
                            Segment(End(147, 1), End(148, -1)),
                            Tunnel(0, Segment(End(212, 1), End(213, -1)), Segment(End(213, -1), End(214, 1))),
                        ),
                        Bridge(
                            Segment(End(148, -1), End(149, 1)),
                            Segment(End(149, 1), End(151, -1)),
                            Tunnel(0, Segment(End(169, 1), End(171, -1)), Segment(End(171, -1), End(172, 1))),
                        ),
                        Bridge(
                            Segment(End(151, -1), End(152, 1)),
                            Segment(End(152, 1), End(153, -1)),
                            Tunnel(0, Segment(End(107, 1), End(108, -1)), Segment(End(108, -1), End(109, 1))),
                        ),
                        Bridge(
                            Segment(End(153, -1), End(154, 1)),
                            Segment(End(154, 1), End(155, -1)),
                            Tunnel(0, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(155, -1), End(156, 1)),
                            Segment(End(156, 1), End(157, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(157, -1), End(158, 1)),
                            Segment(End(158, 1), End(159, -1)),
                            Tunnel(0, Segment(End(201, 1), End(202, -1)), Segment(End(202, -1), End(203, 1))),
                        ),
                        Bridge(
                            Segment(End(159, -1), End(161, 1)),
                            Segment(End(161, 1), End(162, -1)),
                            Tunnel(0, Segment(End(138, 1), End(139, -1)), Segment(End(139, -1), End(141, 1))),
                        ),
                        Bridge(
                            Segment(End(162, -1), End(163, 1)),
                            Segment(End(163, 1), End(164, -1)),
                            Tunnel(0, Segment(End(96, 1), End(97, -1)), Segment(End(97, -1), End(98, 1))),
                        ),
                        Bridge(
                            Segment(End(164, -1), End(165, 1)),
                            Segment(End(165, 1), End(166, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(166, -1), End(167, 1)),
                            Segment(End(167, 1), End(168, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(168, -1), End(169, 1)),
                            Segment(End(169, 1), End(171, -1)),
                            Tunnel(0, Segment(End(189, 1), End(191, -1)), Segment(End(191, -1), End(192, 1))),
                        ),
                        Bridge(
                            Segment(End(171, -1), End(172, 1)),
                            Segment(End(172, 1), End(173, -1)),
                            Tunnel(0, Segment(End(127, 1), End(128, -1)), Segment(End(128, -1), End(129, 1))),
                        ),
                        Bridge(
                            Segment(End(173, -1), End(174, 1)),
                            Segment(End(174, 1), End(175, -1)),
                            Tunnel(0, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                        Bridge(
                            Segment(End(175, -1), End(176, 1)),
                            Segment(End(176, 1), End(177, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(177, -1), End(178, 1)),
                            Segment(End(178, 1), End(179, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(179, -1), End(181, 1)),
                            Segment(End(181, 1), End(182, -1)),
                            Tunnel(0, Segment(End(158, 1), End(159, -1)), Segment(End(159, -1), End(161, 1))),
                        ),
                        Bridge(
                            Segment(End(182, -1), End(183, 1)),
                            Segment(End(183, 1), End(184, -1)),
                            Tunnel(0, Segment(End(116, 1), End(117, -1)), Segment(End(117, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(184, -1), End(185, 1)),
                            Segment(End(185, 1), End(186, -1)),
                            Tunnel(0, Segment(End(74, 1), End(75, -1)), Segment(End(75, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(186, -1), End(187, 1)),
                            Segment(End(187, 1), End(188, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(188, -1), End(189, 1)),
                            Segment(End(189, 1), End(191, -1)),
                            Tunnel(0, Segment(End(209, 1), End(211, -1)), Segment(End(211, -1), End(212, 1))),
                        ),
                        Bridge(
                            Segment(End(191, -1), End(192, 1)),
                            Segment(End(192, 1), End(193, -1)),
                            Tunnel(0, Segment(End(147, 1), End(148, -1)), Segment(End(148, -1), End(149, 1))),
                        ),
                        Bridge(
                            Segment(End(193, -1), End(194, 1)),
                            Segment(End(194, 1), End(195, -1)),
                            Tunnel(0, Segment(End(105, 1), End(106, -1)), Segment(End(106, -1), End(107, 1))),
                        ),
                        Bridge(
                            Segment(End(195, -1), End(196, 1)),
                            Segment(End(196, 1), End(197, -1)),
                            Tunnel(0, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(197, -1), End(198, 1)),
                            Segment(End(198, 1), End(199, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(199, -1), End(201, 1)),
                            Segment(End(201, 1), End(202, -1)),
                            Tunnel(0, Segment(End(178, 1), End(179, -1)), Segment(End(179, -1), End(181, 1))),
                        ),
                        Bridge(
                            Segment(End(202, -1), End(203, 1)),
                            Segment(End(203, 1), End(204, -1)),
                            Tunnel(0, Segment(End(136, 1), End(137, -1)), Segment(End(137, -1), End(138, 1))),
                        ),
                        Bridge(
                            Segment(End(204, -1), End(205, 1)),
                            Segment(End(205, 1), End(206, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(96, 1))),
                        ),
                        Bridge(
                            Segment(End(206, -1), End(207, 1)),
                            Segment(End(207, 1), End(208, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(208, -1), End(209, 1)),
                            Segment(End(209, 1), End(211, -1)),
                            Tunnel(0, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(211, -1), End(212, 1)),
                            Segment(End(212, 1), End(213, -1)),
                            Tunnel(0, Segment(End(167, 1), End(168, -1)), Segment(End(168, -1), End(169, 1))),
                        ),
                        Bridge(
                            Segment(End(213, -1), End(214, 1)),
                            Segment(End(214, 1), End(215, -1)),
                            Tunnel(0, Segment(End(125, 1), End(126, -1)), Segment(End(126, -1), End(127, 1))),
                        ),
                        Bridge(
                            Segment(End(215, -1), End(216, 1)),
                            Segment(End(216, 1), End(217, -1)),
                            Tunnel(0, Segment(End(83, 1), End(84, -1)), Segment(End(84, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(217, -1), End(218, 1)),
                            Segment(End(218, 1), End(219, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_12_9(self):
        knot = Knot(12, 9)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(28, 1)),
                        Segment(End(28, 1), End(29, -1)),
                        Segment(End(29, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(37, 1)),
                        Segment(End(37, 1), End(38, -1)),
                        Segment(End(38, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(2, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(2, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(72, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(1, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(28, 1)),
                            Segment(End(28, 1), End(29, -1)),
                            Tunnel(1, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(29, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(1, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(37, 1)),
                            Segment(End(37, 1), End(38, -1)),
                            Tunnel(1, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(38, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(2, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(1, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(1, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(2, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(1, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(2, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(1, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(1, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(2, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(1, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(30, 1)),
                        Segment(End(30, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(39, 1)),
                        Segment(End(39, 1), End(40, -1)),
                        Segment(End(40, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(72, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(2, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(30, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(2, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(1, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(39, 1), End(40, -1)), Segment(End(40, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(2, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(2, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(1, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(72, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(2, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(30, 1)),
                            Segment(End(30, 1), End(31, -1)),
                            Tunnel(2, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(2, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(39, 1)),
                            Segment(End(39, 1), End(40, -1)),
                            Tunnel(2, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(40, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(2, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(2, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(50, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(1, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(2, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(1, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(2, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(2, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(1, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(50, 1)),
                        Segment(End(50, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(72, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(2, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(2, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(1, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(1, Segment(End(50, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(28, 1), End(29, -1)), Segment(End(29, -1), End(30, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(1, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(37, 1), End(38, -1)), Segment(End(38, -1), End(39, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(2, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(1, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(50, 1)),
                            Segment(End(50, 1), End(51, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(28, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(2, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(2, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(2, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_10_12(self):
        knot = Knot(10, 12)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(77, 1)),
                        Segment(End(77, 1), End(78, -1)),
                        Segment(End(78, -1), End(79, 1)),
                        Segment(End(79, 1), End(80, -1)),
                        Segment(End(80, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(88, 1)),
                        Segment(End(88, 1), End(89, -1)),
                        Segment(End(89, -1), End(90, 1)),
                        Segment(End(90, 1), End(91, -1)),
                        Segment(End(91, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(105, 1)),
                        Segment(End(105, 1), End(106, -1)),
                        Segment(End(106, -1), End(107, 1)),
                        Segment(End(107, 1), End(109, -1)),
                        Segment(End(109, -1), End(110, 1)),
                        Segment(End(110, 1), End(111, -1)),
                        Segment(End(111, -1), End(112, 1)),
                        Segment(End(112, 1), End(113, -1)),
                        Segment(End(113, -1), End(114, 1)),
                        Segment(End(114, 1), End(115, -1)),
                        Segment(End(115, -1), End(116, 1)),
                        Segment(End(116, 1), End(117, -1)),
                        Segment(End(117, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(121, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(1, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(1, Segment(End(88, 1), End(89, -1)), Segment(End(89, -1), End(90, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(1, Segment(End(109, 1), End(111, -1)), Segment(End(111, -1), End(112, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(77, 1), End(78, -1)), Segment(End(78, -1), End(79, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(1, Segment(End(24, 1), End(25, -1)), Segment(End(25, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(1, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(48, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(1, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(1, Segment(End(90, 1), End(91, -1)), Segment(End(91, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(1, Segment(End(112, 1), End(113, -1)), Segment(End(113, -1), End(114, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(37, -1)),
                            Tunnel(1, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(79, 1), End(80, -1)), Segment(End(80, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(1, Segment(End(48, 1), End(49, -1)), Segment(End(49, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(1, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(72, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(1, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(1, Segment(End(114, 1), End(115, -1)), Segment(End(115, -1), End(116, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(61, -1)),
                            Tunnel(1, Segment(End(37, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(105, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(1, Segment(End(72, 1), End(73, -1)), Segment(End(73, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(1, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(96, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(77, 1)),
                            Segment(End(77, 1), End(78, -1)),
                            Tunnel(1, Segment(End(116, 1), End(117, -1)), Segment(End(117, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(78, -1), End(79, 1)),
                            Segment(End(79, 1), End(80, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(80, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(1, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(85, -1)),
                            Tunnel(1, Segment(End(61, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(105, 1), End(106, -1)), Segment(End(106, -1), End(107, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(88, 1)),
                            Segment(End(88, 1), End(89, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(90, 1)),
                            Segment(End(90, 1), End(91, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(91, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(1, Segment(End(96, 1), End(97, -1)), Segment(End(97, -1), End(99, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(1, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(120, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(1, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(1, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(105, 1)),
                            Segment(End(105, 1), End(106, -1)),
                            Tunnel(1, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(106, -1), End(107, 1)),
                            Segment(End(107, 1), End(109, -1)),
                            Tunnel(1, Segment(End(85, 1), End(87, -1)), Segment(End(87, -1), End(88, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(110, 1)),
                            Segment(End(110, 1), End(111, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(112, 1)),
                            Segment(End(112, 1), End(113, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(113, -1), End(114, 1)),
                            Segment(End(114, 1), End(115, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(115, -1), End(116, 1)),
                            Segment(End(116, 1), End(117, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(77, 1))),
                        ),
                        Bridge(
                            Segment(End(117, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(99, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                        Segment(End(24, 1), End(25, -1)),
                        Segment(End(25, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(36, -1)),
                        Segment(End(36, -1), End(37, 1)),
                        Segment(End(37, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(48, 1)),
                        Segment(End(48, 1), End(49, -1)),
                        Segment(End(49, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(60, -1)),
                        Segment(End(60, -1), End(61, 1)),
                        Segment(End(61, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(72, 1)),
                        Segment(End(72, 1), End(73, -1)),
                        Segment(End(73, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(77, 1)),
                        Segment(End(77, 1), End(78, -1)),
                        Segment(End(78, -1), End(79, 1)),
                        Segment(End(79, 1), End(80, -1)),
                        Segment(End(80, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(84, -1)),
                        Segment(End(84, -1), End(85, 1)),
                        Segment(End(85, 1), End(87, -1)),
                        Segment(End(87, -1), End(88, 1)),
                        Segment(End(88, 1), End(89, -1)),
                        Segment(End(89, -1), End(90, 1)),
                        Segment(End(90, 1), End(91, -1)),
                        Segment(End(91, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(96, 1)),
                        Segment(End(96, 1), End(97, -1)),
                        Segment(End(97, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(105, 1)),
                        Segment(End(105, 1), End(106, -1)),
                        Segment(End(106, -1), End(107, 1)),
                        Segment(End(107, 1), End(108, -1)),
                        Segment(End(108, -1), End(109, 1)),
                        Segment(End(109, 1), End(111, -1)),
                        Segment(End(111, -1), End(112, 1)),
                        Segment(End(112, 1), End(113, -1)),
                        Segment(End(113, -1), End(114, 1)),
                        Segment(End(114, 1), End(115, -1)),
                        Segment(End(115, -1), End(116, 1)),
                        Segment(End(116, 1), End(117, -1)),
                        Segment(End(117, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(120, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(88, 1), End(89, -1)), Segment(End(89, -1), End(90, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(0, Segment(End(110, 1), End(111, -1)), Segment(End(111, -1), End(112, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(35, 1), End(36, -1)), Segment(End(36, -1), End(37, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(1, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(1, Segment(End(79, 1), End(80, -1)), Segment(End(80, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(1, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(24, 1)),
                            Segment(End(24, 1), End(25, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(25, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(90, 1), End(91, -1)), Segment(End(91, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(112, 1), End(113, -1)), Segment(End(113, -1), End(114, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(36, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(36, -1), End(37, 1)),
                            Segment(End(37, 1), End(39, -1)),
                            Tunnel(0, Segment(End(35, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(1, Segment(End(59, 1), End(60, -1)), Segment(End(60, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(1, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(1, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(105, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(48, 1)),
                            Segment(End(48, 1), End(49, -1)),
                            Tunnel(1, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(49, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(114, 1), End(115, -1)), Segment(End(115, -1), End(116, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(60, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(60, -1), End(61, 1)),
                            Segment(End(61, 1), End(63, -1)),
                            Tunnel(0, Segment(End(59, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(1, Segment(End(83, 1), End(84, -1)), Segment(End(84, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(1, Segment(End(105, 1), End(106, -1)), Segment(End(106, -1), End(107, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(1, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(72, 1)),
                            Segment(End(72, 1), End(73, -1)),
                            Tunnel(1, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(73, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(77, 1)),
                            Segment(End(77, 1), End(78, -1)),
                            Tunnel(0, Segment(End(116, 1), End(117, -1)), Segment(End(117, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(78, -1), End(79, 1)),
                            Segment(End(79, 1), End(80, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(80, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(84, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(84, -1), End(85, 1)),
                            Segment(End(85, 1), End(87, -1)),
                            Tunnel(0, Segment(End(83, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(88, 1)),
                            Segment(End(88, 1), End(89, -1)),
                            Tunnel(1, Segment(End(107, 1), End(108, -1)), Segment(End(108, -1), End(109, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(90, 1)),
                            Segment(End(90, 1), End(91, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(91, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(1, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(1, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(96, 1)),
                            Segment(End(96, 1), End(97, -1)),
                            Tunnel(1, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(77, 1))),
                        ),
                        Bridge(
                            Segment(End(97, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(121, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(105, 1)),
                            Segment(End(105, 1), End(106, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(106, -1), End(107, 1)),
                            Segment(End(107, 1), End(108, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(88, 1))),
                        ),
                        Bridge(
                            Segment(End(108, -1), End(109, 1)),
                            Segment(End(109, 1), End(111, -1)),
                            Tunnel(0, Segment(End(107, 1), End(109, -1)), Segment(End(109, -1), End(110, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(112, 1)),
                            Segment(End(112, 1), End(113, -1)),
                            Tunnel(1, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(113, -1), End(114, 1)),
                            Segment(End(114, 1), End(115, -1)),
                            Tunnel(1, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(115, -1), End(116, 1)),
                            Segment(End(116, 1), End(117, -1)),
                            Tunnel(1, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(117, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(1, Segment(End(77, 1), End(78, -1)), Segment(End(78, -1), End(79, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_11_11(self):
        knot = Knot(11, 11)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(5, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(7, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(9, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(1, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(3, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(5, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(7, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(9, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(10, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(4, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(6, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(8, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(10, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(4, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(6, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(8, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(9, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(5, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(7, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(9, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(3, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(5, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(7, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(8, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(10, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(4, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(6, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(8, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(10, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(4, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(6, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                    ],
                ),
                String(
                    4,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(7, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(9, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(5, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(7, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(9, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(2, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(5, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(22, 1))),
                        ),
                    ],
                ),
                String(
                    5,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(22, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-2, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(6, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(8, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(10, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(6, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(8, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(10, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(3, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                    ],
                ),
                String(
                    6,
                    [
                        Segment(End(0, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(7, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(9, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(4, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(7, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(9, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(2, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(4, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                    ],
                ),
                String(
                    7,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(5, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(8, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(10, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(5, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(8, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(10, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(3, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                    ],
                ),
                String(
                    8,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(4, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(6, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(9, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(4, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(6, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(9, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(2, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                    ],
                ),
                String(
                    9,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(3, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(5, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(7, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(10, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(3, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(5, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(7, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(10, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(1, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                    ],
                ),
                String(
                    10,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(4, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(6, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(8, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(4, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(6, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(8, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_12_10(self):
        knot = Knot(12, 10)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(31, -1)),
                        Segment(End(31, -1), End(32, 1)),
                        Segment(End(32, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(41, 1)),
                        Segment(End(41, 1), End(42, -1)),
                        Segment(End(42, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(51, -1)),
                        Segment(End(51, -1), End(52, 1)),
                        Segment(End(52, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(61, 1)),
                        Segment(End(61, 1), End(62, -1)),
                        Segment(End(62, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(71, -1)),
                        Segment(End(71, -1), End(72, 1)),
                        Segment(End(72, 1), End(73, -1)),
                        Segment(End(73, -1), End(74, 1)),
                        Segment(End(74, 1), End(75, -1)),
                        Segment(End(75, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(84, -1)),
                        Segment(End(84, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(88, -1)),
                        Segment(End(88, -1), End(89, 1)),
                        Segment(End(89, 1), End(91, -1)),
                        Segment(End(91, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(96, 1)),
                        Segment(End(96, 1), End(97, -1)),
                        Segment(End(97, -1), End(98, 1)),
                        Segment(End(98, 1), End(99, -1)),
                        Segment(End(99, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(105, 1)),
                        Segment(End(105, 1), End(106, -1)),
                        Segment(End(106, -1), End(107, 1)),
                        Segment(End(107, 1), End(108, -1)),
                        Segment(End(108, -1), End(109, 1)),
                        Segment(End(109, 1), End(111, -1)),
                        Segment(End(111, -1), End(112, 1)),
                        Segment(End(112, 1), End(113, -1)),
                        Segment(End(113, -1), End(114, 1)),
                        Segment(End(114, 1), End(115, -1)),
                        Segment(End(115, -1), End(116, 1)),
                        Segment(End(116, 1), End(117, -1)),
                        Segment(End(117, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(121, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(1, Segment(End(98, 1), End(99, -1)), Segment(End(99, -1), End(100, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(1, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(1, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(1, Segment(End(31, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(107, 1), End(108, -1)), Segment(End(108, -1), End(109, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(41, 1), End(42, -1)), Segment(End(42, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(1, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(1, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(120, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(1, Segment(End(96, 1), End(97, -1)), Segment(End(97, -1), End(98, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(1, Segment(End(74, 1), End(75, -1)), Segment(End(75, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(31, -1)),
                            Tunnel(1, Segment(End(51, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(31, -1), End(32, 1)),
                            Segment(End(32, 1), End(33, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(105, 1), End(106, -1)), Segment(End(106, -1), End(107, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(83, 1), End(84, -1)), Segment(End(84, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(61, 1), End(62, -1)), Segment(End(62, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(41, 1)),
                            Segment(End(41, 1), End(42, -1)),
                            Tunnel(1, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(43, 1))),
                        ),
                        Bridge(
                            Segment(End(42, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(1, Segment(End(116, 1), End(117, -1)), Segment(End(117, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(1, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(96, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(51, -1)),
                            Tunnel(1, Segment(End(71, 1), End(73, -1)), Segment(End(73, -1), End(74, 1))),
                        ),
                        Bridge(
                            Segment(End(51, -1), End(52, 1)),
                            Segment(End(52, 1), End(53, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(105, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(61, 1)),
                            Segment(End(61, 1), End(62, -1)),
                            Tunnel(1, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(63, 1))),
                        ),
                        Bridge(
                            Segment(End(62, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(1, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(1, Segment(End(114, 1), End(115, -1)), Segment(End(115, -1), End(116, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(71, -1)),
                            Tunnel(1, Segment(End(91, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(72, 1)),
                            Segment(End(72, 1), End(73, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(73, -1), End(74, 1)),
                            Segment(End(74, 1), End(75, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(75, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(1, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(84, -1)),
                            Tunnel(1, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(84, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(1, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(88, -1)),
                            Tunnel(1, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(88, -1), End(89, 1)),
                            Segment(End(89, 1), End(91, -1)),
                            Tunnel(1, Segment(End(111, 1), End(113, -1)), Segment(End(113, -1), End(114, 1))),
                        ),
                        Bridge(
                            Segment(End(91, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(96, 1)),
                            Segment(End(96, 1), End(97, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(97, -1), End(98, 1)),
                            Segment(End(98, 1), End(99, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(99, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(1, Segment(End(100, 1), End(101, -1)), Segment(End(101, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(1, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(105, 1)),
                            Segment(End(105, 1), End(106, -1)),
                            Tunnel(1, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(106, -1), End(107, 1)),
                            Segment(End(107, 1), End(108, -1)),
                            Tunnel(1, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(108, -1), End(109, 1)),
                            Segment(End(109, 1), End(111, -1)),
                            Tunnel(1, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(112, 1)),
                            Segment(End(112, 1), End(113, -1)),
                            Tunnel(0, Segment(End(87, 1), End(88, -1)), Segment(End(88, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(113, -1), End(114, 1)),
                            Segment(End(114, 1), End(115, -1)),
                            Tunnel(0, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(115, -1), End(116, 1)),
                            Segment(End(116, 1), End(117, -1)),
                            Tunnel(0, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(117, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(33, -1)),
                        Segment(End(33, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(43, 1)),
                        Segment(End(43, 1), End(44, -1)),
                        Segment(End(44, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(53, -1)),
                        Segment(End(53, -1), End(54, 1)),
                        Segment(End(54, 1), End(55, -1)),
                        Segment(End(55, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(63, 1)),
                        Segment(End(63, 1), End(64, -1)),
                        Segment(End(64, -1), End(65, 1)),
                        Segment(End(65, 1), End(66, -1)),
                        Segment(End(66, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(73, -1)),
                        Segment(End(73, -1), End(74, 1)),
                        Segment(End(74, 1), End(75, -1)),
                        Segment(End(75, -1), End(76, 1)),
                        Segment(End(76, 1), End(77, -1)),
                        Segment(End(77, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(83, 1)),
                        Segment(End(83, 1), End(84, -1)),
                        Segment(End(84, -1), End(85, 1)),
                        Segment(End(85, 1), End(86, -1)),
                        Segment(End(86, -1), End(87, 1)),
                        Segment(End(87, 1), End(88, -1)),
                        Segment(End(88, -1), End(89, 1)),
                        Segment(End(89, 1), End(90, -1)),
                        Segment(End(90, -1), End(91, 1)),
                        Segment(End(91, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(96, 1)),
                        Segment(End(96, 1), End(97, -1)),
                        Segment(End(97, -1), End(98, 1)),
                        Segment(End(98, 1), End(99, -1)),
                        Segment(End(99, -1), End(100, 1)),
                        Segment(End(100, 1), End(101, -1)),
                        Segment(End(101, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(105, 1)),
                        Segment(End(105, 1), End(106, -1)),
                        Segment(End(106, -1), End(107, 1)),
                        Segment(End(107, 1), End(108, -1)),
                        Segment(End(108, -1), End(109, 1)),
                        Segment(End(109, 1), End(110, -1)),
                        Segment(End(110, -1), End(111, 1)),
                        Segment(End(111, 1), End(113, -1)),
                        Segment(End(113, -1), End(114, 1)),
                        Segment(End(114, 1), End(115, -1)),
                        Segment(End(115, -1), End(116, 1)),
                        Segment(End(116, 1), End(117, -1)),
                        Segment(End(117, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(120, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(1, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(98, 1), End(99, -1)), Segment(End(99, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(76, 1), End(77, -1)), Segment(End(77, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(54, 1), End(55, -1)), Segment(End(55, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(32, 1), End(33, -1)), Segment(End(33, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(1, Segment(End(109, 1), End(110, -1)), Segment(End(110, -1), End(111, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(1, Segment(End(87, 1), End(88, -1)), Segment(End(88, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(1, Segment(End(65, 1), End(66, -1)), Segment(End(66, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(1, Segment(End(43, 1), End(44, -1)), Segment(End(44, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(121, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(96, 1), End(97, -1)), Segment(End(97, -1), End(98, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(74, 1), End(75, -1)), Segment(End(75, -1), End(76, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(52, 1), End(53, -1)), Segment(End(53, -1), End(54, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(33, -1)),
                            Tunnel(0, Segment(End(29, 1), End(31, -1)), Segment(End(31, -1), End(32, 1))),
                        ),
                        Bridge(
                            Segment(End(33, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(1, Segment(End(107, 1), End(108, -1)), Segment(End(108, -1), End(109, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(1, Segment(End(85, 1), End(86, -1)), Segment(End(86, -1), End(87, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(1, Segment(End(63, 1), End(64, -1)), Segment(End(64, -1), End(65, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(43, 1)),
                            Segment(End(43, 1), End(44, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(44, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(116, 1), End(117, -1)), Segment(End(117, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(96, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(72, 1), End(73, -1)), Segment(End(73, -1), End(74, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(53, -1)),
                            Tunnel(0, Segment(End(49, 1), End(51, -1)), Segment(End(51, -1), End(52, 1))),
                        ),
                        Bridge(
                            Segment(End(53, -1), End(54, 1)),
                            Segment(End(54, 1), End(55, -1)),
                            Tunnel(1, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(55, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(1, Segment(End(105, 1), End(106, -1)), Segment(End(106, -1), End(107, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(1, Segment(End(83, 1), End(84, -1)), Segment(End(84, -1), End(85, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(63, 1)),
                            Segment(End(63, 1), End(64, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(41, 1))),
                        ),
                        Bridge(
                            Segment(End(64, -1), End(65, 1)),
                            Segment(End(65, 1), End(66, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(66, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(114, 1), End(115, -1)), Segment(End(115, -1), End(116, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(73, -1)),
                            Tunnel(0, Segment(End(69, 1), End(71, -1)), Segment(End(71, -1), End(72, 1))),
                        ),
                        Bridge(
                            Segment(End(73, -1), End(74, 1)),
                            Segment(End(74, 1), End(75, -1)),
                            Tunnel(1, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(75, -1), End(76, 1)),
                            Segment(End(76, 1), End(77, -1)),
                            Tunnel(1, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(77, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(1, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(105, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(83, 1)),
                            Segment(End(83, 1), End(84, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(61, 1))),
                        ),
                        Bridge(
                            Segment(End(84, -1), End(85, 1)),
                            Segment(End(85, 1), End(86, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(86, -1), End(87, 1)),
                            Segment(End(87, 1), End(88, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(88, -1), End(89, 1)),
                            Segment(End(89, 1), End(90, -1)),
                            Tunnel(0, Segment(End(112, 1), End(113, -1)), Segment(End(113, -1), End(114, 1))),
                        ),
                        Bridge(
                            Segment(End(90, -1), End(91, 1)),
                            Segment(End(91, 1), End(93, -1)),
                            Tunnel(0, Segment(End(89, 1), End(91, -1)), Segment(End(91, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(1, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(96, 1)),
                            Segment(End(96, 1), End(97, -1)),
                            Tunnel(1, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(97, -1), End(98, 1)),
                            Segment(End(98, 1), End(99, -1)),
                            Tunnel(1, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(99, -1), End(100, 1)),
                            Segment(End(100, 1), End(101, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(101, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(105, 1)),
                            Segment(End(105, 1), End(106, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(106, -1), End(107, 1)),
                            Segment(End(107, 1), End(108, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(108, -1), End(109, 1)),
                            Segment(End(109, 1), End(110, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(110, -1), End(111, 1)),
                            Segment(End(111, 1), End(113, -1)),
                            Tunnel(0, Segment(End(109, 1), End(111, -1)), Segment(End(111, -1), End(112, 1))),
                        ),
                        Bridge(
                            Segment(End(113, -1), End(114, 1)),
                            Segment(End(114, 1), End(115, -1)),
                            Tunnel(1, Segment(End(89, 1), End(90, -1)), Segment(End(90, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(115, -1), End(116, 1)),
                            Segment(End(116, 1), End(117, -1)),
                            Tunnel(1, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(117, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(1, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_11_12(self):
        knot = Knot(11, 12)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(33, 1)),
                        Segment(End(33, 1), End(34, -1)),
                        Segment(End(34, -1), End(35, 1)),
                        Segment(End(35, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(44, 1)),
                        Segment(End(44, 1), End(45, -1)),
                        Segment(End(45, -1), End(46, 1)),
                        Segment(End(46, 1), End(47, -1)),
                        Segment(End(47, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(55, 1)),
                        Segment(End(55, 1), End(56, -1)),
                        Segment(End(56, -1), End(57, 1)),
                        Segment(End(57, 1), End(58, -1)),
                        Segment(End(58, -1), End(59, 1)),
                        Segment(End(59, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(66, 1)),
                        Segment(End(66, 1), End(67, -1)),
                        Segment(End(67, -1), End(68, 1)),
                        Segment(End(68, 1), End(69, -1)),
                        Segment(End(69, -1), End(70, 1)),
                        Segment(End(70, 1), End(71, -1)),
                        Segment(End(71, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(77, 1)),
                        Segment(End(77, 1), End(78, -1)),
                        Segment(End(78, -1), End(79, 1)),
                        Segment(End(79, 1), End(80, -1)),
                        Segment(End(80, -1), End(81, 1)),
                        Segment(End(81, 1), End(82, -1)),
                        Segment(End(82, -1), End(83, 1)),
                        Segment(End(83, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(88, 1)),
                        Segment(End(88, 1), End(89, -1)),
                        Segment(End(89, -1), End(90, 1)),
                        Segment(End(90, 1), End(91, -1)),
                        Segment(End(91, -1), End(92, 1)),
                        Segment(End(92, 1), End(93, -1)),
                        Segment(End(93, -1), End(94, 1)),
                        Segment(End(94, 1), End(95, -1)),
                        Segment(End(95, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(99, 1)),
                        Segment(End(99, 1), End(100, -1)),
                        Segment(End(100, -1), End(101, 1)),
                        Segment(End(101, 1), End(102, -1)),
                        Segment(End(102, -1), End(103, 1)),
                        Segment(End(103, 1), End(104, -1)),
                        Segment(End(104, -1), End(105, 1)),
                        Segment(End(105, 1), End(106, -1)),
                        Segment(End(106, -1), End(107, 1)),
                        Segment(End(107, 1), End(109, -1)),
                        Segment(End(109, -1), End(110, 1)),
                        Segment(End(110, 1), End(111, -1)),
                        Segment(End(111, -1), End(112, 1)),
                        Segment(End(112, 1), End(113, -1)),
                        Segment(End(113, -1), End(114, 1)),
                        Segment(End(114, 1), End(115, -1)),
                        Segment(End(115, -1), End(116, 1)),
                        Segment(End(116, 1), End(117, -1)),
                        Segment(End(117, -1), End(118, 1)),
                        Segment(End(118, 1), End(119, -1)),
                        Segment(End(119, -1), End(121, 1)),
                        Segment(End(121, 1), End(122, -1)),
                        Segment(End(122, -1), End(123, 1)),
                        Segment(End(123, 1), End(124, -1)),
                        Segment(End(124, -1), End(125, 1)),
                        Segment(End(125, 1), End(126, -1)),
                        Segment(End(126, -1), End(127, 1)),
                        Segment(End(127, 1), End(128, -1)),
                        Segment(End(128, -1), End(129, 1)),
                        Segment(End(129, 1), End(130, -1)),
                        Segment(End(130, -1), End(131, 1)),
                        Segment(End(131, 1), End(133, -1)),
                        Segment(End(133, -1), End(134, 1)),
                        Segment(End(134, 1), End(135, -1)),
                        Segment(End(135, -1), End(136, 1)),
                        Segment(End(136, 1), End(137, -1)),
                        Segment(End(137, -1), End(138, 1)),
                        Segment(End(138, 1), End(139, -1)),
                        Segment(End(139, -1), End(140, 1)),
                        Segment(End(140, 1), End(141, -1)),
                        Segment(End(141, -1), End(142, 1)),
                        Segment(End(142, 1), End(143, -1)),
                        Segment(End(143, -1), End(145, 1)),
                        Segment(End(145, 1), End(146, -1)),
                        Segment(End(146, -1), End(147, 1)),
                        Segment(End(147, 1), End(148, -1)),
                        Segment(End(148, -1), End(149, 1)),
                        Segment(End(149, 1), End(150, -1)),
                        Segment(End(150, -1), End(151, 1)),
                        Segment(End(151, 1), End(152, -1)),
                        Segment(End(152, -1), End(153, 1)),
                        Segment(End(153, 1), End(154, -1)),
                        Segment(End(154, -1), End(155, 1)),
                        Segment(End(155, 1), End(157, -1)),
                        Segment(End(157, -1), End(158, 1)),
                        Segment(End(158, 1), End(159, -1)),
                        Segment(End(159, -1), End(160, 1)),
                        Segment(End(160, 1), End(161, -1)),
                        Segment(End(161, -1), End(162, 1)),
                        Segment(End(162, 1), End(163, -1)),
                        Segment(End(163, -1), End(164, 1)),
                        Segment(End(164, 1), End(165, -1)),
                        Segment(End(165, -1), End(166, 1)),
                        Segment(End(166, 1), End(167, -1)),
                        Segment(End(167, -1), End(169, 1)),
                        Segment(End(169, 1), End(170, -1)),
                        Segment(End(170, -1), End(171, 1)),
                        Segment(End(171, 1), End(172, -1)),
                        Segment(End(172, -1), End(173, 1)),
                        Segment(End(173, 1), End(174, -1)),
                        Segment(End(174, -1), End(175, 1)),
                        Segment(End(175, 1), End(176, -1)),
                        Segment(End(176, -1), End(177, 1)),
                        Segment(End(177, 1), End(178, -1)),
                        Segment(End(178, -1), End(179, 1)),
                        Segment(End(179, 1), End(181, -1)),
                        Segment(End(181, -1), End(182, 1)),
                        Segment(End(182, 1), End(183, -1)),
                        Segment(End(183, -1), End(184, 1)),
                        Segment(End(184, 1), End(185, -1)),
                        Segment(End(185, -1), End(186, 1)),
                        Segment(End(186, 1), End(187, -1)),
                        Segment(End(187, -1), End(188, 1)),
                        Segment(End(188, 1), End(189, -1)),
                        Segment(End(189, -1), End(190, 1)),
                        Segment(End(190, 1), End(191, -1)),
                        Segment(End(191, -1), End(193, 1)),
                        Segment(End(193, 1), End(194, -1)),
                        Segment(End(194, -1), End(195, 1)),
                        Segment(End(195, 1), End(196, -1)),
                        Segment(End(196, -1), End(197, 1)),
                        Segment(End(197, 1), End(198, -1)),
                        Segment(End(198, -1), End(199, 1)),
                        Segment(End(199, 1), End(200, -1)),
                        Segment(End(200, -1), End(201, 1)),
                        Segment(End(201, 1), End(202, -1)),
                        Segment(End(202, -1), End(203, 1)),
                        Segment(End(203, 1), End(205, -1)),
                        Segment(End(205, -1), End(206, 1)),
                        Segment(End(206, 1), End(207, -1)),
                        Segment(End(207, -1), End(208, 1)),
                        Segment(End(208, 1), End(209, -1)),
                        Segment(End(209, -1), End(210, 1)),
                        Segment(End(210, 1), End(211, -1)),
                        Segment(End(211, -1), End(212, 1)),
                        Segment(End(212, 1), End(213, -1)),
                        Segment(End(213, -1), End(214, 1)),
                        Segment(End(214, 1), End(215, -1)),
                        Segment(End(215, -1), End(217, 1)),
                        Segment(End(217, 1), End(218, -1)),
                        Segment(End(218, -1), End(219, 1)),
                        Segment(End(219, 1), End(220, -1)),
                        Segment(End(220, -1), End(221, 1)),
                        Segment(End(221, 1), End(222, -1)),
                        Segment(End(222, -1), End(223, 1)),
                        Segment(End(223, 1), End(224, -1)),
                        Segment(End(224, -1), End(225, 1)),
                        Segment(End(225, 1), End(226, -1)),
                        Segment(End(226, -1), End(227, 1)),
                        Segment(End(227, 1), End(229, -1)),
                        Segment(End(229, -1), End(230, 1)),
                        Segment(End(230, 1), End(231, -1)),
                        Segment(End(231, -1), End(232, 1)),
                        Segment(End(232, 1), End(233, -1)),
                        Segment(End(233, -1), End(234, 1)),
                        Segment(End(234, 1), End(235, -1)),
                        Segment(End(235, -1), End(236, 1)),
                        Segment(End(236, 1), End(237, -1)),
                        Segment(End(237, -1), End(238, 1)),
                        Segment(End(238, 1), End(239, -1)),
                        Segment(End(239, -1), End(241, 1)),
                        Segment(End(241, 1), End(242, -1)),
                        Segment(End(242, -1), End(243, 1)),
                        Segment(End(243, 1), End(244, -1)),
                        Segment(End(244, -1), End(245, 1)),
                        Segment(End(245, 1), End(246, -1)),
                        Segment(End(246, -1), End(247, 1)),
                        Segment(End(247, 1), End(248, -1)),
                        Segment(End(248, -1), End(249, 1)),
                        Segment(End(249, 1), End(250, -1)),
                        Segment(End(250, -1), End(251, 1)),
                        Segment(End(251, 1), End(253, -1)),
                        Segment(End(253, -1), End(254, 1)),
                        Segment(End(254, 1), End(255, -1)),
                        Segment(End(255, -1), End(256, 1)),
                        Segment(End(256, 1), End(257, -1)),
                        Segment(End(257, -1), End(258, 1)),
                        Segment(End(258, 1), End(259, -1)),
                        Segment(End(259, -1), End(260, 1)),
                        Segment(End(260, 1), End(261, -1)),
                        Segment(End(261, -1), End(262, 1)),
                        Segment(End(262, 1), End(263, -1)),
                        Segment(End(263, -1), End(265, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(68, 1), End(69, -1)), Segment(End(69, -1), End(70, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(114, 1), End(115, -1)), Segment(End(115, -1), End(116, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(160, 1), End(161, -1)), Segment(End(161, -1), End(162, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(206, 1), End(207, -1)), Segment(End(207, -1), End(208, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(0, Segment(End(251, 1), End(253, -1)), Segment(End(253, -1), End(254, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(57, 1), End(58, -1)), Segment(End(58, -1), End(59, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(103, 1), End(104, -1)), Segment(End(104, -1), End(105, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(149, 1), End(150, -1)), Segment(End(150, -1), End(151, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(195, 1), End(196, -1)), Segment(End(196, -1), End(197, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(0, Segment(End(241, 1), End(242, -1)), Segment(End(242, -1), End(243, 1))),
                        ),
                        Bridge(
                            Segment(End(23, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(46, 1), End(47, -1)), Segment(End(47, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(92, 1), End(93, -1)), Segment(End(93, -1), End(94, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(138, 1), End(139, -1)), Segment(End(139, -1), End(140, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(184, 1), End(185, -1)), Segment(End(185, -1), End(186, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(33, 1)),
                            Segment(End(33, 1), End(34, -1)),
                            Tunnel(0, Segment(End(230, 1), End(231, -1)), Segment(End(231, -1), End(232, 1))),
                        ),
                        Bridge(
                            Segment(End(34, -1), End(35, 1)),
                            Segment(End(35, 1), End(37, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(81, 1), End(82, -1)), Segment(End(82, -1), End(83, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(127, 1), End(128, -1)), Segment(End(128, -1), End(129, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(173, 1), End(174, -1)), Segment(End(174, -1), End(175, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(44, 1)),
                            Segment(End(44, 1), End(45, -1)),
                            Tunnel(0, Segment(End(219, 1), End(220, -1)), Segment(End(220, -1), End(221, 1))),
                        ),
                        Bridge(
                            Segment(End(45, -1), End(46, 1)),
                            Segment(End(46, 1), End(47, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(47, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(70, 1), End(71, -1)), Segment(End(71, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(116, 1), End(117, -1)), Segment(End(117, -1), End(118, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(162, 1), End(163, -1)), Segment(End(163, -1), End(164, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(55, 1)),
                            Segment(End(55, 1), End(56, -1)),
                            Tunnel(0, Segment(End(208, 1), End(209, -1)), Segment(End(209, -1), End(210, 1))),
                        ),
                        Bridge(
                            Segment(End(56, -1), End(57, 1)),
                            Segment(End(57, 1), End(58, -1)),
                            Tunnel(0, Segment(End(254, 1), End(255, -1)), Segment(End(255, -1), End(256, 1))),
                        ),
                        Bridge(
                            Segment(End(58, -1), End(59, 1)),
                            Segment(End(59, 1), End(61, -1)),
                            Tunnel(0, Segment(End(35, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(105, 1), End(106, -1)), Segment(End(106, -1), End(107, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(151, 1), End(152, -1)), Segment(End(152, -1), End(153, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(66, 1)),
                            Segment(End(66, 1), End(67, -1)),
                            Tunnel(0, Segment(End(197, 1), End(198, -1)), Segment(End(198, -1), End(199, 1))),
                        ),
                        Bridge(
                            Segment(End(67, -1), End(68, 1)),
                            Segment(End(68, 1), End(69, -1)),
                            Tunnel(0, Segment(End(243, 1), End(244, -1)), Segment(End(244, -1), End(245, 1))),
                        ),
                        Bridge(
                            Segment(End(69, -1), End(70, 1)),
                            Segment(End(70, 1), End(71, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(71, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(94, 1), End(95, -1)), Segment(End(95, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(140, 1), End(141, -1)), Segment(End(141, -1), End(142, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(77, 1)),
                            Segment(End(77, 1), End(78, -1)),
                            Tunnel(0, Segment(End(186, 1), End(187, -1)), Segment(End(187, -1), End(188, 1))),
                        ),
                        Bridge(
                            Segment(End(78, -1), End(79, 1)),
                            Segment(End(79, 1), End(80, -1)),
                            Tunnel(0, Segment(End(232, 1), End(233, -1)), Segment(End(233, -1), End(234, 1))),
                        ),
                        Bridge(
                            Segment(End(80, -1), End(81, 1)),
                            Segment(End(81, 1), End(82, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(82, -1), End(83, 1)),
                            Segment(End(83, 1), End(85, -1)),
                            Tunnel(0, Segment(End(59, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(129, 1), End(130, -1)), Segment(End(130, -1), End(131, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(88, 1)),
                            Segment(End(88, 1), End(89, -1)),
                            Tunnel(0, Segment(End(175, 1), End(176, -1)), Segment(End(176, -1), End(177, 1))),
                        ),
                        Bridge(
                            Segment(End(89, -1), End(90, 1)),
                            Segment(End(90, 1), End(91, -1)),
                            Tunnel(0, Segment(End(221, 1), End(222, -1)), Segment(End(222, -1), End(223, 1))),
                        ),
                        Bridge(
                            Segment(End(91, -1), End(92, 1)),
                            Segment(End(92, 1), End(93, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(93, -1), End(94, 1)),
                            Segment(End(94, 1), End(95, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(95, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(118, 1), End(119, -1)), Segment(End(119, -1), End(121, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(99, 1)),
                            Segment(End(99, 1), End(100, -1)),
                            Tunnel(0, Segment(End(164, 1), End(165, -1)), Segment(End(165, -1), End(166, 1))),
                        ),
                        Bridge(
                            Segment(End(100, -1), End(101, 1)),
                            Segment(End(101, 1), End(102, -1)),
                            Tunnel(0, Segment(End(210, 1), End(211, -1)), Segment(End(211, -1), End(212, 1))),
                        ),
                        Bridge(
                            Segment(End(102, -1), End(103, 1)),
                            Segment(End(103, 1), End(104, -1)),
                            Tunnel(0, Segment(End(256, 1), End(257, -1)), Segment(End(257, -1), End(258, 1))),
                        ),
                        Bridge(
                            Segment(End(104, -1), End(105, 1)),
                            Segment(End(105, 1), End(106, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(106, -1), End(107, 1)),
                            Segment(End(107, 1), End(109, -1)),
                            Tunnel(0, Segment(End(83, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(110, 1)),
                            Segment(End(110, 1), End(111, -1)),
                            Tunnel(0, Segment(End(153, 1), End(154, -1)), Segment(End(154, -1), End(155, 1))),
                        ),
                        Bridge(
                            Segment(End(111, -1), End(112, 1)),
                            Segment(End(112, 1), End(113, -1)),
                            Tunnel(0, Segment(End(199, 1), End(200, -1)), Segment(End(200, -1), End(201, 1))),
                        ),
                        Bridge(
                            Segment(End(113, -1), End(114, 1)),
                            Segment(End(114, 1), End(115, -1)),
                            Tunnel(0, Segment(End(245, 1), End(246, -1)), Segment(End(246, -1), End(247, 1))),
                        ),
                        Bridge(
                            Segment(End(115, -1), End(116, 1)),
                            Segment(End(116, 1), End(117, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(117, -1), End(118, 1)),
                            Segment(End(118, 1), End(119, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(119, -1), End(121, 1)),
                            Segment(End(121, 1), End(122, -1)),
                            Tunnel(0, Segment(End(142, 1), End(143, -1)), Segment(End(143, -1), End(145, 1))),
                        ),
                        Bridge(
                            Segment(End(122, -1), End(123, 1)),
                            Segment(End(123, 1), End(124, -1)),
                            Tunnel(0, Segment(End(188, 1), End(189, -1)), Segment(End(189, -1), End(190, 1))),
                        ),
                        Bridge(
                            Segment(End(124, -1), End(125, 1)),
                            Segment(End(125, 1), End(126, -1)),
                            Tunnel(0, Segment(End(234, 1), End(235, -1)), Segment(End(235, -1), End(236, 1))),
                        ),
                        Bridge(
                            Segment(End(126, -1), End(127, 1)),
                            Segment(End(127, 1), End(128, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(128, -1), End(129, 1)),
                            Segment(End(129, 1), End(130, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(130, -1), End(131, 1)),
                            Segment(End(131, 1), End(133, -1)),
                            Tunnel(0, Segment(End(107, 1), End(109, -1)), Segment(End(109, -1), End(110, 1))),
                        ),
                        Bridge(
                            Segment(End(133, -1), End(134, 1)),
                            Segment(End(134, 1), End(135, -1)),
                            Tunnel(0, Segment(End(177, 1), End(178, -1)), Segment(End(178, -1), End(179, 1))),
                        ),
                        Bridge(
                            Segment(End(135, -1), End(136, 1)),
                            Segment(End(136, 1), End(137, -1)),
                            Tunnel(0, Segment(End(223, 1), End(224, -1)), Segment(End(224, -1), End(225, 1))),
                        ),
                        Bridge(
                            Segment(End(137, -1), End(138, 1)),
                            Segment(End(138, 1), End(139, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(139, -1), End(140, 1)),
                            Segment(End(140, 1), End(141, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(141, -1), End(142, 1)),
                            Segment(End(142, 1), End(143, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(99, 1))),
                        ),
                        Bridge(
                            Segment(End(143, -1), End(145, 1)),
                            Segment(End(145, 1), End(146, -1)),
                            Tunnel(0, Segment(End(166, 1), End(167, -1)), Segment(End(167, -1), End(169, 1))),
                        ),
                        Bridge(
                            Segment(End(146, -1), End(147, 1)),
                            Segment(End(147, 1), End(148, -1)),
                            Tunnel(0, Segment(End(212, 1), End(213, -1)), Segment(End(213, -1), End(214, 1))),
                        ),
                        Bridge(
                            Segment(End(148, -1), End(149, 1)),
                            Segment(End(149, 1), End(150, -1)),
                            Tunnel(0, Segment(End(258, 1), End(259, -1)), Segment(End(259, -1), End(260, 1))),
                        ),
                        Bridge(
                            Segment(End(150, -1), End(151, 1)),
                            Segment(End(151, 1), End(152, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(152, -1), End(153, 1)),
                            Segment(End(153, 1), End(154, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(88, 1))),
                        ),
                        Bridge(
                            Segment(End(154, -1), End(155, 1)),
                            Segment(End(155, 1), End(157, -1)),
                            Tunnel(0, Segment(End(131, 1), End(133, -1)), Segment(End(133, -1), End(134, 1))),
                        ),
                        Bridge(
                            Segment(End(157, -1), End(158, 1)),
                            Segment(End(158, 1), End(159, -1)),
                            Tunnel(0, Segment(End(201, 1), End(202, -1)), Segment(End(202, -1), End(203, 1))),
                        ),
                        Bridge(
                            Segment(End(159, -1), End(160, 1)),
                            Segment(End(160, 1), End(161, -1)),
                            Tunnel(0, Segment(End(247, 1), End(248, -1)), Segment(End(248, -1), End(249, 1))),
                        ),
                        Bridge(
                            Segment(End(161, -1), End(162, 1)),
                            Segment(End(162, 1), End(163, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(163, -1), End(164, 1)),
                            Segment(End(164, 1), End(165, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(77, 1))),
                        ),
                        Bridge(
                            Segment(End(165, -1), End(166, 1)),
                            Segment(End(166, 1), End(167, -1)),
                            Tunnel(0, Segment(End(121, 1), End(122, -1)), Segment(End(122, -1), End(123, 1))),
                        ),
                        Bridge(
                            Segment(End(167, -1), End(169, 1)),
                            Segment(End(169, 1), End(170, -1)),
                            Tunnel(0, Segment(End(190, 1), End(191, -1)), Segment(End(191, -1), End(193, 1))),
                        ),
                        Bridge(
                            Segment(End(170, -1), End(171, 1)),
                            Segment(End(171, 1), End(172, -1)),
                            Tunnel(0, Segment(End(236, 1), End(237, -1)), Segment(End(237, -1), End(238, 1))),
                        ),
                        Bridge(
                            Segment(End(172, -1), End(173, 1)),
                            Segment(End(173, 1), End(174, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(174, -1), End(175, 1)),
                            Segment(End(175, 1), End(176, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(66, 1))),
                        ),
                        Bridge(
                            Segment(End(176, -1), End(177, 1)),
                            Segment(End(177, 1), End(178, -1)),
                            Tunnel(0, Segment(End(110, 1), End(111, -1)), Segment(End(111, -1), End(112, 1))),
                        ),
                        Bridge(
                            Segment(End(178, -1), End(179, 1)),
                            Segment(End(179, 1), End(181, -1)),
                            Tunnel(0, Segment(End(155, 1), End(157, -1)), Segment(End(157, -1), End(158, 1))),
                        ),
                        Bridge(
                            Segment(End(181, -1), End(182, 1)),
                            Segment(End(182, 1), End(183, -1)),
                            Tunnel(0, Segment(End(225, 1), End(226, -1)), Segment(End(226, -1), End(227, 1))),
                        ),
                        Bridge(
                            Segment(End(183, -1), End(184, 1)),
                            Segment(End(184, 1), End(185, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(185, -1), End(186, 1)),
                            Segment(End(186, 1), End(187, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(55, 1))),
                        ),
                        Bridge(
                            Segment(End(187, -1), End(188, 1)),
                            Segment(End(188, 1), End(189, -1)),
                            Tunnel(0, Segment(End(99, 1), End(100, -1)), Segment(End(100, -1), End(101, 1))),
                        ),
                        Bridge(
                            Segment(End(189, -1), End(190, 1)),
                            Segment(End(190, 1), End(191, -1)),
                            Tunnel(0, Segment(End(145, 1), End(146, -1)), Segment(End(146, -1), End(147, 1))),
                        ),
                        Bridge(
                            Segment(End(191, -1), End(193, 1)),
                            Segment(End(193, 1), End(194, -1)),
                            Tunnel(0, Segment(End(214, 1), End(215, -1)), Segment(End(215, -1), End(217, 1))),
                        ),
                        Bridge(
                            Segment(End(194, -1), End(195, 1)),
                            Segment(End(195, 1), End(196, -1)),
                            Tunnel(0, Segment(End(260, 1), End(261, -1)), Segment(End(261, -1), End(262, 1))),
                        ),
                        Bridge(
                            Segment(End(196, -1), End(197, 1)),
                            Segment(End(197, 1), End(198, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(44, 1))),
                        ),
                        Bridge(
                            Segment(End(198, -1), End(199, 1)),
                            Segment(End(199, 1), End(200, -1)),
                            Tunnel(0, Segment(End(88, 1), End(89, -1)), Segment(End(89, -1), End(90, 1))),
                        ),
                        Bridge(
                            Segment(End(200, -1), End(201, 1)),
                            Segment(End(201, 1), End(202, -1)),
                            Tunnel(0, Segment(End(134, 1), End(135, -1)), Segment(End(135, -1), End(136, 1))),
                        ),
                        Bridge(
                            Segment(End(202, -1), End(203, 1)),
                            Segment(End(203, 1), End(205, -1)),
                            Tunnel(0, Segment(End(179, 1), End(181, -1)), Segment(End(181, -1), End(182, 1))),
                        ),
                        Bridge(
                            Segment(End(205, -1), End(206, 1)),
                            Segment(End(206, 1), End(207, -1)),
                            Tunnel(0, Segment(End(249, 1), End(250, -1)), Segment(End(250, -1), End(251, 1))),
                        ),
                        Bridge(
                            Segment(End(207, -1), End(208, 1)),
                            Segment(End(208, 1), End(209, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(33, 1))),
                        ),
                        Bridge(
                            Segment(End(209, -1), End(210, 1)),
                            Segment(End(210, 1), End(211, -1)),
                            Tunnel(0, Segment(End(77, 1), End(78, -1)), Segment(End(78, -1), End(79, 1))),
                        ),
                        Bridge(
                            Segment(End(211, -1), End(212, 1)),
                            Segment(End(212, 1), End(213, -1)),
                            Tunnel(0, Segment(End(123, 1), End(124, -1)), Segment(End(124, -1), End(125, 1))),
                        ),
                        Bridge(
                            Segment(End(213, -1), End(214, 1)),
                            Segment(End(214, 1), End(215, -1)),
                            Tunnel(0, Segment(End(169, 1), End(170, -1)), Segment(End(170, -1), End(171, 1))),
                        ),
                        Bridge(
                            Segment(End(215, -1), End(217, 1)),
                            Segment(End(217, 1), End(218, -1)),
                            Tunnel(0, Segment(End(238, 1), End(239, -1)), Segment(End(239, -1), End(241, 1))),
                        ),
                        Bridge(
                            Segment(End(218, -1), End(219, 1)),
                            Segment(End(219, 1), End(220, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(220, -1), End(221, 1)),
                            Segment(End(221, 1), End(222, -1)),
                            Tunnel(0, Segment(End(66, 1), End(67, -1)), Segment(End(67, -1), End(68, 1))),
                        ),
                        Bridge(
                            Segment(End(222, -1), End(223, 1)),
                            Segment(End(223, 1), End(224, -1)),
                            Tunnel(0, Segment(End(112, 1), End(113, -1)), Segment(End(113, -1), End(114, 1))),
                        ),
                        Bridge(
                            Segment(End(224, -1), End(225, 1)),
                            Segment(End(225, 1), End(226, -1)),
                            Tunnel(0, Segment(End(158, 1), End(159, -1)), Segment(End(159, -1), End(160, 1))),
                        ),
                        Bridge(
                            Segment(End(226, -1), End(227, 1)),
                            Segment(End(227, 1), End(229, -1)),
                            Tunnel(0, Segment(End(203, 1), End(205, -1)), Segment(End(205, -1), End(206, 1))),
                        ),
                        Bridge(
                            Segment(End(229, -1), End(230, 1)),
                            Segment(End(230, 1), End(231, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(231, -1), End(232, 1)),
                            Segment(End(232, 1), End(233, -1)),
                            Tunnel(0, Segment(End(55, 1), End(56, -1)), Segment(End(56, -1), End(57, 1))),
                        ),
                        Bridge(
                            Segment(End(233, -1), End(234, 1)),
                            Segment(End(234, 1), End(235, -1)),
                            Tunnel(0, Segment(End(101, 1), End(102, -1)), Segment(End(102, -1), End(103, 1))),
                        ),
                        Bridge(
                            Segment(End(235, -1), End(236, 1)),
                            Segment(End(236, 1), End(237, -1)),
                            Tunnel(0, Segment(End(147, 1), End(148, -1)), Segment(End(148, -1), End(149, 1))),
                        ),
                        Bridge(
                            Segment(End(237, -1), End(238, 1)),
                            Segment(End(238, 1), End(239, -1)),
                            Tunnel(0, Segment(End(193, 1), End(194, -1)), Segment(End(194, -1), End(195, 1))),
                        ),
                        Bridge(
                            Segment(End(239, -1), End(241, 1)),
                            Segment(End(241, 1), End(242, -1)),
                            Tunnel(0, Segment(End(262, 1), End(263, -1)), Segment(End(263, -1), End(265, 1))),
                        ),
                        Bridge(
                            Segment(End(242, -1), End(243, 1)),
                            Segment(End(243, 1), End(244, -1)),
                            Tunnel(0, Segment(End(44, 1), End(45, -1)), Segment(End(45, -1), End(46, 1))),
                        ),
                        Bridge(
                            Segment(End(244, -1), End(245, 1)),
                            Segment(End(245, 1), End(246, -1)),
                            Tunnel(0, Segment(End(90, 1), End(91, -1)), Segment(End(91, -1), End(92, 1))),
                        ),
                        Bridge(
                            Segment(End(246, -1), End(247, 1)),
                            Segment(End(247, 1), End(248, -1)),
                            Tunnel(0, Segment(End(136, 1), End(137, -1)), Segment(End(137, -1), End(138, 1))),
                        ),
                        Bridge(
                            Segment(End(248, -1), End(249, 1)),
                            Segment(End(249, 1), End(250, -1)),
                            Tunnel(0, Segment(End(182, 1), End(183, -1)), Segment(End(183, -1), End(184, 1))),
                        ),
                        Bridge(
                            Segment(End(250, -1), End(251, 1)),
                            Segment(End(251, 1), End(253, -1)),
                            Tunnel(0, Segment(End(227, 1), End(229, -1)), Segment(End(229, -1), End(230, 1))),
                        ),
                        Bridge(
                            Segment(End(253, -1), End(254, 1)),
                            Segment(End(254, 1), End(255, -1)),
                            Tunnel(0, Segment(End(33, 1), End(34, -1)), Segment(End(34, -1), End(35, 1))),
                        ),
                        Bridge(
                            Segment(End(255, -1), End(256, 1)),
                            Segment(End(256, 1), End(257, -1)),
                            Tunnel(0, Segment(End(79, 1), End(80, -1)), Segment(End(80, -1), End(81, 1))),
                        ),
                        Bridge(
                            Segment(End(257, -1), End(258, 1)),
                            Segment(End(258, 1), End(259, -1)),
                            Tunnel(0, Segment(End(125, 1), End(126, -1)), Segment(End(126, -1), End(127, 1))),
                        ),
                        Bridge(
                            Segment(End(259, -1), End(260, 1)),
                            Segment(End(260, 1), End(261, -1)),
                            Tunnel(0, Segment(End(171, 1), End(172, -1)), Segment(End(172, -1), End(173, 1))),
                        ),
                        Bridge(
                            Segment(End(261, -1), End(262, 1)),
                            Segment(End(262, 1), End(263, -1)),
                            Tunnel(0, Segment(End(217, 1), End(218, -1)), Segment(End(218, -1), End(219, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_12_11(self):
        knot = Knot(12, 11)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                        Segment(End(24, -1), End(25, 1)),
                        Segment(End(25, 1), End(26, -1)),
                        Segment(End(26, -1), End(27, 1)),
                        Segment(End(27, 1), End(28, -1)),
                        Segment(End(28, -1), End(29, 1)),
                        Segment(End(29, 1), End(30, -1)),
                        Segment(End(30, -1), End(31, 1)),
                        Segment(End(31, 1), End(32, -1)),
                        Segment(End(32, -1), End(34, 1)),
                        Segment(End(34, 1), End(35, -1)),
                        Segment(End(35, -1), End(36, 1)),
                        Segment(End(36, 1), End(37, -1)),
                        Segment(End(37, -1), End(38, 1)),
                        Segment(End(38, 1), End(39, -1)),
                        Segment(End(39, -1), End(40, 1)),
                        Segment(End(40, 1), End(41, -1)),
                        Segment(End(41, -1), End(42, 1)),
                        Segment(End(42, 1), End(43, -1)),
                        Segment(End(43, -1), End(45, 1)),
                        Segment(End(45, 1), End(46, -1)),
                        Segment(End(46, -1), End(47, 1)),
                        Segment(End(47, 1), End(48, -1)),
                        Segment(End(48, -1), End(49, 1)),
                        Segment(End(49, 1), End(50, -1)),
                        Segment(End(50, -1), End(51, 1)),
                        Segment(End(51, 1), End(52, -1)),
                        Segment(End(52, -1), End(53, 1)),
                        Segment(End(53, 1), End(54, -1)),
                        Segment(End(54, -1), End(56, 1)),
                        Segment(End(56, 1), End(57, -1)),
                        Segment(End(57, -1), End(58, 1)),
                        Segment(End(58, 1), End(59, -1)),
                        Segment(End(59, -1), End(60, 1)),
                        Segment(End(60, 1), End(61, -1)),
                        Segment(End(61, -1), End(62, 1)),
                        Segment(End(62, 1), End(63, -1)),
                        Segment(End(63, -1), End(64, 1)),
                        Segment(End(64, 1), End(65, -1)),
                        Segment(End(65, -1), End(67, 1)),
                        Segment(End(67, 1), End(68, -1)),
                        Segment(End(68, -1), End(69, 1)),
                        Segment(End(69, 1), End(70, -1)),
                        Segment(End(70, -1), End(71, 1)),
                        Segment(End(71, 1), End(72, -1)),
                        Segment(End(72, -1), End(73, 1)),
                        Segment(End(73, 1), End(74, -1)),
                        Segment(End(74, -1), End(75, 1)),
                        Segment(End(75, 1), End(76, -1)),
                        Segment(End(76, -1), End(78, 1)),
                        Segment(End(78, 1), End(79, -1)),
                        Segment(End(79, -1), End(80, 1)),
                        Segment(End(80, 1), End(81, -1)),
                        Segment(End(81, -1), End(82, 1)),
                        Segment(End(82, 1), End(83, -1)),
                        Segment(End(83, -1), End(84, 1)),
                        Segment(End(84, 1), End(85, -1)),
                        Segment(End(85, -1), End(86, 1)),
                        Segment(End(86, 1), End(87, -1)),
                        Segment(End(87, -1), End(89, 1)),
                        Segment(End(89, 1), End(90, -1)),
                        Segment(End(90, -1), End(91, 1)),
                        Segment(End(91, 1), End(92, -1)),
                        Segment(End(92, -1), End(93, 1)),
                        Segment(End(93, 1), End(94, -1)),
                        Segment(End(94, -1), End(95, 1)),
                        Segment(End(95, 1), End(96, -1)),
                        Segment(End(96, -1), End(97, 1)),
                        Segment(End(97, 1), End(98, -1)),
                        Segment(End(98, -1), End(100, 1)),
                        Segment(End(100, 1), End(101, -1)),
                        Segment(End(101, -1), End(102, 1)),
                        Segment(End(102, 1), End(103, -1)),
                        Segment(End(103, -1), End(104, 1)),
                        Segment(End(104, 1), End(105, -1)),
                        Segment(End(105, -1), End(106, 1)),
                        Segment(End(106, 1), End(107, -1)),
                        Segment(End(107, -1), End(108, 1)),
                        Segment(End(108, 1), End(109, -1)),
                        Segment(End(109, -1), End(111, 1)),
                        Segment(End(111, 1), End(112, -1)),
                        Segment(End(112, -1), End(113, 1)),
                        Segment(End(113, 1), End(114, -1)),
                        Segment(End(114, -1), End(115, 1)),
                        Segment(End(115, 1), End(116, -1)),
                        Segment(End(116, -1), End(117, 1)),
                        Segment(End(117, 1), End(118, -1)),
                        Segment(End(118, -1), End(119, 1)),
                        Segment(End(119, 1), End(120, -1)),
                        Segment(End(120, -1), End(122, 1)),
                        Segment(End(122, 1), End(123, -1)),
                        Segment(End(123, -1), End(124, 1)),
                        Segment(End(124, 1), End(125, -1)),
                        Segment(End(125, -1), End(126, 1)),
                        Segment(End(126, 1), End(127, -1)),
                        Segment(End(127, -1), End(128, 1)),
                        Segment(End(128, 1), End(129, -1)),
                        Segment(End(129, -1), End(130, 1)),
                        Segment(End(130, 1), End(131, -1)),
                        Segment(End(131, -1), End(133, 1)),
                        Segment(End(133, 1), End(134, -1)),
                        Segment(End(134, -1), End(135, 1)),
                        Segment(End(135, 1), End(136, -1)),
                        Segment(End(136, -1), End(137, 1)),
                        Segment(End(137, 1), End(138, -1)),
                        Segment(End(138, -1), End(139, 1)),
                        Segment(End(139, 1), End(140, -1)),
                        Segment(End(140, -1), End(141, 1)),
                        Segment(End(141, 1), End(142, -1)),
                        Segment(End(142, -1), End(144, 1)),
                        Segment(End(144, 1), End(145, -1)),
                        Segment(End(145, -1), End(146, 1)),
                        Segment(End(146, 1), End(147, -1)),
                        Segment(End(147, -1), End(148, 1)),
                        Segment(End(148, 1), End(149, -1)),
                        Segment(End(149, -1), End(150, 1)),
                        Segment(End(150, 1), End(151, -1)),
                        Segment(End(151, -1), End(152, 1)),
                        Segment(End(152, 1), End(153, -1)),
                        Segment(End(153, -1), End(155, 1)),
                        Segment(End(155, 1), End(156, -1)),
                        Segment(End(156, -1), End(157, 1)),
                        Segment(End(157, 1), End(158, -1)),
                        Segment(End(158, -1), End(159, 1)),
                        Segment(End(159, 1), End(160, -1)),
                        Segment(End(160, -1), End(161, 1)),
                        Segment(End(161, 1), End(162, -1)),
                        Segment(End(162, -1), End(163, 1)),
                        Segment(End(163, 1), End(164, -1)),
                        Segment(End(164, -1), End(166, 1)),
                        Segment(End(166, 1), End(167, -1)),
                        Segment(End(167, -1), End(168, 1)),
                        Segment(End(168, 1), End(169, -1)),
                        Segment(End(169, -1), End(170, 1)),
                        Segment(End(170, 1), End(171, -1)),
                        Segment(End(171, -1), End(172, 1)),
                        Segment(End(172, 1), End(173, -1)),
                        Segment(End(173, -1), End(174, 1)),
                        Segment(End(174, 1), End(175, -1)),
                        Segment(End(175, -1), End(177, 1)),
                        Segment(End(177, 1), End(178, -1)),
                        Segment(End(178, -1), End(179, 1)),
                        Segment(End(179, 1), End(180, -1)),
                        Segment(End(180, -1), End(181, 1)),
                        Segment(End(181, 1), End(182, -1)),
                        Segment(End(182, -1), End(183, 1)),
                        Segment(End(183, 1), End(184, -1)),
                        Segment(End(184, -1), End(185, 1)),
                        Segment(End(185, 1), End(186, -1)),
                        Segment(End(186, -1), End(188, 1)),
                        Segment(End(188, 1), End(189, -1)),
                        Segment(End(189, -1), End(190, 1)),
                        Segment(End(190, 1), End(191, -1)),
                        Segment(End(191, -1), End(192, 1)),
                        Segment(End(192, 1), End(193, -1)),
                        Segment(End(193, -1), End(194, 1)),
                        Segment(End(194, 1), End(195, -1)),
                        Segment(End(195, -1), End(196, 1)),
                        Segment(End(196, 1), End(197, -1)),
                        Segment(End(197, -1), End(199, 1)),
                        Segment(End(199, 1), End(200, -1)),
                        Segment(End(200, -1), End(201, 1)),
                        Segment(End(201, 1), End(202, -1)),
                        Segment(End(202, -1), End(203, 1)),
                        Segment(End(203, 1), End(204, -1)),
                        Segment(End(204, -1), End(205, 1)),
                        Segment(End(205, 1), End(206, -1)),
                        Segment(End(206, -1), End(207, 1)),
                        Segment(End(207, 1), End(208, -1)),
                        Segment(End(208, -1), End(210, 1)),
                        Segment(End(210, 1), End(211, -1)),
                        Segment(End(211, -1), End(212, 1)),
                        Segment(End(212, 1), End(213, -1)),
                        Segment(End(213, -1), End(214, 1)),
                        Segment(End(214, 1), End(215, -1)),
                        Segment(End(215, -1), End(216, 1)),
                        Segment(End(216, 1), End(217, -1)),
                        Segment(End(217, -1), End(218, 1)),
                        Segment(End(218, 1), End(219, -1)),
                        Segment(End(219, -1), End(221, 1)),
                        Segment(End(221, 1), End(222, -1)),
                        Segment(End(222, -1), End(223, 1)),
                        Segment(End(223, 1), End(224, -1)),
                        Segment(End(224, -1), End(225, 1)),
                        Segment(End(225, 1), End(226, -1)),
                        Segment(End(226, -1), End(227, 1)),
                        Segment(End(227, 1), End(228, -1)),
                        Segment(End(228, -1), End(229, 1)),
                        Segment(End(229, 1), End(230, -1)),
                        Segment(End(230, -1), End(232, 1)),
                        Segment(End(232, 1), End(233, -1)),
                        Segment(End(233, -1), End(234, 1)),
                        Segment(End(234, 1), End(235, -1)),
                        Segment(End(235, -1), End(236, 1)),
                        Segment(End(236, 1), End(237, -1)),
                        Segment(End(237, -1), End(238, 1)),
                        Segment(End(238, 1), End(239, -1)),
                        Segment(End(239, -1), End(240, 1)),
                        Segment(End(240, 1), End(241, -1)),
                        Segment(End(241, -1), End(243, 1)),
                        Segment(End(243, 1), End(244, -1)),
                        Segment(End(244, -1), End(245, 1)),
                        Segment(End(245, 1), End(246, -1)),
                        Segment(End(246, -1), End(247, 1)),
                        Segment(End(247, 1), End(248, -1)),
                        Segment(End(248, -1), End(249, 1)),
                        Segment(End(249, 1), End(250, -1)),
                        Segment(End(250, -1), End(251, 1)),
                        Segment(End(251, 1), End(252, -1)),
                        Segment(End(252, -1), End(254, 1)),
                        Segment(End(254, 1), End(255, -1)),
                        Segment(End(255, -1), End(256, 1)),
                        Segment(End(256, 1), End(257, -1)),
                        Segment(End(257, -1), End(258, 1)),
                        Segment(End(258, 1), End(259, -1)),
                        Segment(End(259, -1), End(260, 1)),
                        Segment(End(260, 1), End(261, -1)),
                        Segment(End(261, -1), End(262, 1)),
                        Segment(End(262, 1), End(263, -1)),
                        Segment(End(263, -1), End(265, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(0, Segment(End(240, 1), End(241, -1)), Segment(End(241, -1), End(243, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(0, Segment(End(194, 1), End(195, -1)), Segment(End(195, -1), End(196, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(0, Segment(End(148, 1), End(149, -1)), Segment(End(149, -1), End(150, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(0, Segment(End(102, 1), End(103, -1)), Segment(End(103, -1), End(104, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(0, Segment(End(56, 1), End(57, -1)), Segment(End(57, -1), End(58, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(0, Segment(End(251, 1), End(252, -1)), Segment(End(252, -1), End(254, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(0, Segment(End(205, 1), End(206, -1)), Segment(End(206, -1), End(207, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(0, Segment(End(159, 1), End(160, -1)), Segment(End(160, -1), End(161, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(0, Segment(End(113, 1), End(114, -1)), Segment(End(114, -1), End(115, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(0, Segment(End(67, 1), End(68, -1)), Segment(End(68, -1), End(69, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(262, 1), End(263, -1)), Segment(End(263, -1), End(265, 1))),
                        ),
                        Bridge(
                            Segment(End(24, -1), End(25, 1)),
                            Segment(End(25, 1), End(26, -1)),
                            Tunnel(0, Segment(End(216, 1), End(217, -1)), Segment(End(217, -1), End(218, 1))),
                        ),
                        Bridge(
                            Segment(End(26, -1), End(27, 1)),
                            Segment(End(27, 1), End(28, -1)),
                            Tunnel(0, Segment(End(170, 1), End(171, -1)), Segment(End(171, -1), End(172, 1))),
                        ),
                        Bridge(
                            Segment(End(28, -1), End(29, 1)),
                            Segment(End(29, 1), End(30, -1)),
                            Tunnel(0, Segment(End(124, 1), End(125, -1)), Segment(End(125, -1), End(126, 1))),
                        ),
                        Bridge(
                            Segment(End(30, -1), End(31, 1)),
                            Segment(End(31, 1), End(32, -1)),
                            Tunnel(0, Segment(End(78, 1), End(79, -1)), Segment(End(79, -1), End(80, 1))),
                        ),
                        Bridge(
                            Segment(End(32, -1), End(34, 1)),
                            Segment(End(34, 1), End(35, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(35, -1), End(36, 1)),
                            Segment(End(36, 1), End(37, -1)),
                            Tunnel(0, Segment(End(227, 1), End(228, -1)), Segment(End(228, -1), End(229, 1))),
                        ),
                        Bridge(
                            Segment(End(37, -1), End(38, 1)),
                            Segment(End(38, 1), End(39, -1)),
                            Tunnel(0, Segment(End(181, 1), End(182, -1)), Segment(End(182, -1), End(183, 1))),
                        ),
                        Bridge(
                            Segment(End(39, -1), End(40, 1)),
                            Segment(End(40, 1), End(41, -1)),
                            Tunnel(0, Segment(End(135, 1), End(136, -1)), Segment(End(136, -1), End(137, 1))),
                        ),
                        Bridge(
                            Segment(End(41, -1), End(42, 1)),
                            Segment(End(42, 1), End(43, -1)),
                            Tunnel(0, Segment(End(89, 1), End(90, -1)), Segment(End(90, -1), End(91, 1))),
                        ),
                        Bridge(
                            Segment(End(43, -1), End(45, 1)),
                            Segment(End(45, 1), End(46, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(46, -1), End(47, 1)),
                            Segment(End(47, 1), End(48, -1)),
                            Tunnel(0, Segment(End(238, 1), End(239, -1)), Segment(End(239, -1), End(240, 1))),
                        ),
                        Bridge(
                            Segment(End(48, -1), End(49, 1)),
                            Segment(End(49, 1), End(50, -1)),
                            Tunnel(0, Segment(End(192, 1), End(193, -1)), Segment(End(193, -1), End(194, 1))),
                        ),
                        Bridge(
                            Segment(End(50, -1), End(51, 1)),
                            Segment(End(51, 1), End(52, -1)),
                            Tunnel(0, Segment(End(146, 1), End(147, -1)), Segment(End(147, -1), End(148, 1))),
                        ),
                        Bridge(
                            Segment(End(52, -1), End(53, 1)),
                            Segment(End(53, 1), End(54, -1)),
                            Tunnel(0, Segment(End(100, 1), End(101, -1)), Segment(End(101, -1), End(102, 1))),
                        ),
                        Bridge(
                            Segment(End(54, -1), End(56, 1)),
                            Segment(End(56, 1), End(57, -1)),
                            Tunnel(0, Segment(End(31, 1), End(32, -1)), Segment(End(32, -1), End(34, 1))),
                        ),
                        Bridge(
                            Segment(End(57, -1), End(58, 1)),
                            Segment(End(58, 1), End(59, -1)),
                            Tunnel(0, Segment(End(249, 1), End(250, -1)), Segment(End(250, -1), End(251, 1))),
                        ),
                        Bridge(
                            Segment(End(59, -1), End(60, 1)),
                            Segment(End(60, 1), End(61, -1)),
                            Tunnel(0, Segment(End(203, 1), End(204, -1)), Segment(End(204, -1), End(205, 1))),
                        ),
                        Bridge(
                            Segment(End(61, -1), End(62, 1)),
                            Segment(End(62, 1), End(63, -1)),
                            Tunnel(0, Segment(End(157, 1), End(158, -1)), Segment(End(158, -1), End(159, 1))),
                        ),
                        Bridge(
                            Segment(End(63, -1), End(64, 1)),
                            Segment(End(64, 1), End(65, -1)),
                            Tunnel(0, Segment(End(111, 1), End(112, -1)), Segment(End(112, -1), End(113, 1))),
                        ),
                        Bridge(
                            Segment(End(65, -1), End(67, 1)),
                            Segment(End(67, 1), End(68, -1)),
                            Tunnel(0, Segment(End(42, 1), End(43, -1)), Segment(End(43, -1), End(45, 1))),
                        ),
                        Bridge(
                            Segment(End(68, -1), End(69, 1)),
                            Segment(End(69, 1), End(70, -1)),
                            Tunnel(0, Segment(End(260, 1), End(261, -1)), Segment(End(261, -1), End(262, 1))),
                        ),
                        Bridge(
                            Segment(End(70, -1), End(71, 1)),
                            Segment(End(71, 1), End(72, -1)),
                            Tunnel(0, Segment(End(214, 1), End(215, -1)), Segment(End(215, -1), End(216, 1))),
                        ),
                        Bridge(
                            Segment(End(72, -1), End(73, 1)),
                            Segment(End(73, 1), End(74, -1)),
                            Tunnel(0, Segment(End(168, 1), End(169, -1)), Segment(End(169, -1), End(170, 1))),
                        ),
                        Bridge(
                            Segment(End(74, -1), End(75, 1)),
                            Segment(End(75, 1), End(76, -1)),
                            Tunnel(0, Segment(End(122, 1), End(123, -1)), Segment(End(123, -1), End(124, 1))),
                        ),
                        Bridge(
                            Segment(End(76, -1), End(78, 1)),
                            Segment(End(78, 1), End(79, -1)),
                            Tunnel(0, Segment(End(53, 1), End(54, -1)), Segment(End(54, -1), End(56, 1))),
                        ),
                        Bridge(
                            Segment(End(79, -1), End(80, 1)),
                            Segment(End(80, 1), End(81, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(81, -1), End(82, 1)),
                            Segment(End(82, 1), End(83, -1)),
                            Tunnel(0, Segment(End(225, 1), End(226, -1)), Segment(End(226, -1), End(227, 1))),
                        ),
                        Bridge(
                            Segment(End(83, -1), End(84, 1)),
                            Segment(End(84, 1), End(85, -1)),
                            Tunnel(0, Segment(End(179, 1), End(180, -1)), Segment(End(180, -1), End(181, 1))),
                        ),
                        Bridge(
                            Segment(End(85, -1), End(86, 1)),
                            Segment(End(86, 1), End(87, -1)),
                            Tunnel(0, Segment(End(133, 1), End(134, -1)), Segment(End(134, -1), End(135, 1))),
                        ),
                        Bridge(
                            Segment(End(87, -1), End(89, 1)),
                            Segment(End(89, 1), End(90, -1)),
                            Tunnel(0, Segment(End(64, 1), End(65, -1)), Segment(End(65, -1), End(67, 1))),
                        ),
                        Bridge(
                            Segment(End(90, -1), End(91, 1)),
                            Segment(End(91, 1), End(92, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(92, -1), End(93, 1)),
                            Segment(End(93, 1), End(94, -1)),
                            Tunnel(0, Segment(End(236, 1), End(237, -1)), Segment(End(237, -1), End(238, 1))),
                        ),
                        Bridge(
                            Segment(End(94, -1), End(95, 1)),
                            Segment(End(95, 1), End(96, -1)),
                            Tunnel(0, Segment(End(190, 1), End(191, -1)), Segment(End(191, -1), End(192, 1))),
                        ),
                        Bridge(
                            Segment(End(96, -1), End(97, 1)),
                            Segment(End(97, 1), End(98, -1)),
                            Tunnel(0, Segment(End(144, 1), End(145, -1)), Segment(End(145, -1), End(146, 1))),
                        ),
                        Bridge(
                            Segment(End(98, -1), End(100, 1)),
                            Segment(End(100, 1), End(101, -1)),
                            Tunnel(0, Segment(End(75, 1), End(76, -1)), Segment(End(76, -1), End(78, 1))),
                        ),
                        Bridge(
                            Segment(End(101, -1), End(102, 1)),
                            Segment(End(102, 1), End(103, -1)),
                            Tunnel(0, Segment(End(29, 1), End(30, -1)), Segment(End(30, -1), End(31, 1))),
                        ),
                        Bridge(
                            Segment(End(103, -1), End(104, 1)),
                            Segment(End(104, 1), End(105, -1)),
                            Tunnel(0, Segment(End(247, 1), End(248, -1)), Segment(End(248, -1), End(249, 1))),
                        ),
                        Bridge(
                            Segment(End(105, -1), End(106, 1)),
                            Segment(End(106, 1), End(107, -1)),
                            Tunnel(0, Segment(End(201, 1), End(202, -1)), Segment(End(202, -1), End(203, 1))),
                        ),
                        Bridge(
                            Segment(End(107, -1), End(108, 1)),
                            Segment(End(108, 1), End(109, -1)),
                            Tunnel(0, Segment(End(155, 1), End(156, -1)), Segment(End(156, -1), End(157, 1))),
                        ),
                        Bridge(
                            Segment(End(109, -1), End(111, 1)),
                            Segment(End(111, 1), End(112, -1)),
                            Tunnel(0, Segment(End(86, 1), End(87, -1)), Segment(End(87, -1), End(89, 1))),
                        ),
                        Bridge(
                            Segment(End(112, -1), End(113, 1)),
                            Segment(End(113, 1), End(114, -1)),
                            Tunnel(0, Segment(End(40, 1), End(41, -1)), Segment(End(41, -1), End(42, 1))),
                        ),
                        Bridge(
                            Segment(End(114, -1), End(115, 1)),
                            Segment(End(115, 1), End(116, -1)),
                            Tunnel(0, Segment(End(258, 1), End(259, -1)), Segment(End(259, -1), End(260, 1))),
                        ),
                        Bridge(
                            Segment(End(116, -1), End(117, 1)),
                            Segment(End(117, 1), End(118, -1)),
                            Tunnel(0, Segment(End(212, 1), End(213, -1)), Segment(End(213, -1), End(214, 1))),
                        ),
                        Bridge(
                            Segment(End(118, -1), End(119, 1)),
                            Segment(End(119, 1), End(120, -1)),
                            Tunnel(0, Segment(End(166, 1), End(167, -1)), Segment(End(167, -1), End(168, 1))),
                        ),
                        Bridge(
                            Segment(End(120, -1), End(122, 1)),
                            Segment(End(122, 1), End(123, -1)),
                            Tunnel(0, Segment(End(97, 1), End(98, -1)), Segment(End(98, -1), End(100, 1))),
                        ),
                        Bridge(
                            Segment(End(123, -1), End(124, 1)),
                            Segment(End(124, 1), End(125, -1)),
                            Tunnel(0, Segment(End(51, 1), End(52, -1)), Segment(End(52, -1), End(53, 1))),
                        ),
                        Bridge(
                            Segment(End(125, -1), End(126, 1)),
                            Segment(End(126, 1), End(127, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(127, -1), End(128, 1)),
                            Segment(End(128, 1), End(129, -1)),
                            Tunnel(0, Segment(End(223, 1), End(224, -1)), Segment(End(224, -1), End(225, 1))),
                        ),
                        Bridge(
                            Segment(End(129, -1), End(130, 1)),
                            Segment(End(130, 1), End(131, -1)),
                            Tunnel(0, Segment(End(177, 1), End(178, -1)), Segment(End(178, -1), End(179, 1))),
                        ),
                        Bridge(
                            Segment(End(131, -1), End(133, 1)),
                            Segment(End(133, 1), End(134, -1)),
                            Tunnel(0, Segment(End(108, 1), End(109, -1)), Segment(End(109, -1), End(111, 1))),
                        ),
                        Bridge(
                            Segment(End(134, -1), End(135, 1)),
                            Segment(End(135, 1), End(136, -1)),
                            Tunnel(0, Segment(End(62, 1), End(63, -1)), Segment(End(63, -1), End(64, 1))),
                        ),
                        Bridge(
                            Segment(End(136, -1), End(137, 1)),
                            Segment(End(137, 1), End(138, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(138, -1), End(139, 1)),
                            Segment(End(139, 1), End(140, -1)),
                            Tunnel(0, Segment(End(234, 1), End(235, -1)), Segment(End(235, -1), End(236, 1))),
                        ),
                        Bridge(
                            Segment(End(140, -1), End(141, 1)),
                            Segment(End(141, 1), End(142, -1)),
                            Tunnel(0, Segment(End(188, 1), End(189, -1)), Segment(End(189, -1), End(190, 1))),
                        ),
                        Bridge(
                            Segment(End(142, -1), End(144, 1)),
                            Segment(End(144, 1), End(145, -1)),
                            Tunnel(0, Segment(End(119, 1), End(120, -1)), Segment(End(120, -1), End(122, 1))),
                        ),
                        Bridge(
                            Segment(End(145, -1), End(146, 1)),
                            Segment(End(146, 1), End(147, -1)),
                            Tunnel(0, Segment(End(73, 1), End(74, -1)), Segment(End(74, -1), End(75, 1))),
                        ),
                        Bridge(
                            Segment(End(147, -1), End(148, 1)),
                            Segment(End(148, 1), End(149, -1)),
                            Tunnel(0, Segment(End(27, 1), End(28, -1)), Segment(End(28, -1), End(29, 1))),
                        ),
                        Bridge(
                            Segment(End(149, -1), End(150, 1)),
                            Segment(End(150, 1), End(151, -1)),
                            Tunnel(0, Segment(End(245, 1), End(246, -1)), Segment(End(246, -1), End(247, 1))),
                        ),
                        Bridge(
                            Segment(End(151, -1), End(152, 1)),
                            Segment(End(152, 1), End(153, -1)),
                            Tunnel(0, Segment(End(199, 1), End(200, -1)), Segment(End(200, -1), End(201, 1))),
                        ),
                        Bridge(
                            Segment(End(153, -1), End(155, 1)),
                            Segment(End(155, 1), End(156, -1)),
                            Tunnel(0, Segment(End(130, 1), End(131, -1)), Segment(End(131, -1), End(133, 1))),
                        ),
                        Bridge(
                            Segment(End(156, -1), End(157, 1)),
                            Segment(End(157, 1), End(158, -1)),
                            Tunnel(0, Segment(End(84, 1), End(85, -1)), Segment(End(85, -1), End(86, 1))),
                        ),
                        Bridge(
                            Segment(End(158, -1), End(159, 1)),
                            Segment(End(159, 1), End(160, -1)),
                            Tunnel(0, Segment(End(38, 1), End(39, -1)), Segment(End(39, -1), End(40, 1))),
                        ),
                        Bridge(
                            Segment(End(160, -1), End(161, 1)),
                            Segment(End(161, 1), End(162, -1)),
                            Tunnel(0, Segment(End(256, 1), End(257, -1)), Segment(End(257, -1), End(258, 1))),
                        ),
                        Bridge(
                            Segment(End(162, -1), End(163, 1)),
                            Segment(End(163, 1), End(164, -1)),
                            Tunnel(0, Segment(End(210, 1), End(211, -1)), Segment(End(211, -1), End(212, 1))),
                        ),
                        Bridge(
                            Segment(End(164, -1), End(166, 1)),
                            Segment(End(166, 1), End(167, -1)),
                            Tunnel(0, Segment(End(141, 1), End(142, -1)), Segment(End(142, -1), End(144, 1))),
                        ),
                        Bridge(
                            Segment(End(167, -1), End(168, 1)),
                            Segment(End(168, 1), End(169, -1)),
                            Tunnel(0, Segment(End(95, 1), End(96, -1)), Segment(End(96, -1), End(97, 1))),
                        ),
                        Bridge(
                            Segment(End(169, -1), End(170, 1)),
                            Segment(End(170, 1), End(171, -1)),
                            Tunnel(0, Segment(End(49, 1), End(50, -1)), Segment(End(50, -1), End(51, 1))),
                        ),
                        Bridge(
                            Segment(End(171, -1), End(172, 1)),
                            Segment(End(172, 1), End(173, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(173, -1), End(174, 1)),
                            Segment(End(174, 1), End(175, -1)),
                            Tunnel(0, Segment(End(221, 1), End(222, -1)), Segment(End(222, -1), End(223, 1))),
                        ),
                        Bridge(
                            Segment(End(175, -1), End(177, 1)),
                            Segment(End(177, 1), End(178, -1)),
                            Tunnel(0, Segment(End(152, 1), End(153, -1)), Segment(End(153, -1), End(155, 1))),
                        ),
                        Bridge(
                            Segment(End(178, -1), End(179, 1)),
                            Segment(End(179, 1), End(180, -1)),
                            Tunnel(0, Segment(End(106, 1), End(107, -1)), Segment(End(107, -1), End(108, 1))),
                        ),
                        Bridge(
                            Segment(End(180, -1), End(181, 1)),
                            Segment(End(181, 1), End(182, -1)),
                            Tunnel(0, Segment(End(60, 1), End(61, -1)), Segment(End(61, -1), End(62, 1))),
                        ),
                        Bridge(
                            Segment(End(182, -1), End(183, 1)),
                            Segment(End(183, 1), End(184, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(184, -1), End(185, 1)),
                            Segment(End(185, 1), End(186, -1)),
                            Tunnel(0, Segment(End(232, 1), End(233, -1)), Segment(End(233, -1), End(234, 1))),
                        ),
                        Bridge(
                            Segment(End(186, -1), End(188, 1)),
                            Segment(End(188, 1), End(189, -1)),
                            Tunnel(0, Segment(End(163, 1), End(164, -1)), Segment(End(164, -1), End(166, 1))),
                        ),
                        Bridge(
                            Segment(End(189, -1), End(190, 1)),
                            Segment(End(190, 1), End(191, -1)),
                            Tunnel(0, Segment(End(117, 1), End(118, -1)), Segment(End(118, -1), End(119, 1))),
                        ),
                        Bridge(
                            Segment(End(191, -1), End(192, 1)),
                            Segment(End(192, 1), End(193, -1)),
                            Tunnel(0, Segment(End(71, 1), End(72, -1)), Segment(End(72, -1), End(73, 1))),
                        ),
                        Bridge(
                            Segment(End(193, -1), End(194, 1)),
                            Segment(End(194, 1), End(195, -1)),
                            Tunnel(0, Segment(End(25, 1), End(26, -1)), Segment(End(26, -1), End(27, 1))),
                        ),
                        Bridge(
                            Segment(End(195, -1), End(196, 1)),
                            Segment(End(196, 1), End(197, -1)),
                            Tunnel(0, Segment(End(243, 1), End(244, -1)), Segment(End(244, -1), End(245, 1))),
                        ),
                        Bridge(
                            Segment(End(197, -1), End(199, 1)),
                            Segment(End(199, 1), End(200, -1)),
                            Tunnel(0, Segment(End(174, 1), End(175, -1)), Segment(End(175, -1), End(177, 1))),
                        ),
                        Bridge(
                            Segment(End(200, -1), End(201, 1)),
                            Segment(End(201, 1), End(202, -1)),
                            Tunnel(0, Segment(End(128, 1), End(129, -1)), Segment(End(129, -1), End(130, 1))),
                        ),
                        Bridge(
                            Segment(End(202, -1), End(203, 1)),
                            Segment(End(203, 1), End(204, -1)),
                            Tunnel(0, Segment(End(82, 1), End(83, -1)), Segment(End(83, -1), End(84, 1))),
                        ),
                        Bridge(
                            Segment(End(204, -1), End(205, 1)),
                            Segment(End(205, 1), End(206, -1)),
                            Tunnel(0, Segment(End(36, 1), End(37, -1)), Segment(End(37, -1), End(38, 1))),
                        ),
                        Bridge(
                            Segment(End(206, -1), End(207, 1)),
                            Segment(End(207, 1), End(208, -1)),
                            Tunnel(0, Segment(End(254, 1), End(255, -1)), Segment(End(255, -1), End(256, 1))),
                        ),
                        Bridge(
                            Segment(End(208, -1), End(210, 1)),
                            Segment(End(210, 1), End(211, -1)),
                            Tunnel(0, Segment(End(185, 1), End(186, -1)), Segment(End(186, -1), End(188, 1))),
                        ),
                        Bridge(
                            Segment(End(211, -1), End(212, 1)),
                            Segment(End(212, 1), End(213, -1)),
                            Tunnel(0, Segment(End(139, 1), End(140, -1)), Segment(End(140, -1), End(141, 1))),
                        ),
                        Bridge(
                            Segment(End(213, -1), End(214, 1)),
                            Segment(End(214, 1), End(215, -1)),
                            Tunnel(0, Segment(End(93, 1), End(94, -1)), Segment(End(94, -1), End(95, 1))),
                        ),
                        Bridge(
                            Segment(End(215, -1), End(216, 1)),
                            Segment(End(216, 1), End(217, -1)),
                            Tunnel(0, Segment(End(47, 1), End(48, -1)), Segment(End(48, -1), End(49, 1))),
                        ),
                        Bridge(
                            Segment(End(217, -1), End(218, 1)),
                            Segment(End(218, 1), End(219, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(219, -1), End(221, 1)),
                            Segment(End(221, 1), End(222, -1)),
                            Tunnel(0, Segment(End(196, 1), End(197, -1)), Segment(End(197, -1), End(199, 1))),
                        ),
                        Bridge(
                            Segment(End(222, -1), End(223, 1)),
                            Segment(End(223, 1), End(224, -1)),
                            Tunnel(0, Segment(End(150, 1), End(151, -1)), Segment(End(151, -1), End(152, 1))),
                        ),
                        Bridge(
                            Segment(End(224, -1), End(225, 1)),
                            Segment(End(225, 1), End(226, -1)),
                            Tunnel(0, Segment(End(104, 1), End(105, -1)), Segment(End(105, -1), End(106, 1))),
                        ),
                        Bridge(
                            Segment(End(226, -1), End(227, 1)),
                            Segment(End(227, 1), End(228, -1)),
                            Tunnel(0, Segment(End(58, 1), End(59, -1)), Segment(End(59, -1), End(60, 1))),
                        ),
                        Bridge(
                            Segment(End(228, -1), End(229, 1)),
                            Segment(End(229, 1), End(230, -1)),
                            Tunnel(0, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(230, -1), End(232, 1)),
                            Segment(End(232, 1), End(233, -1)),
                            Tunnel(0, Segment(End(207, 1), End(208, -1)), Segment(End(208, -1), End(210, 1))),
                        ),
                        Bridge(
                            Segment(End(233, -1), End(234, 1)),
                            Segment(End(234, 1), End(235, -1)),
                            Tunnel(0, Segment(End(161, 1), End(162, -1)), Segment(End(162, -1), End(163, 1))),
                        ),
                        Bridge(
                            Segment(End(235, -1), End(236, 1)),
                            Segment(End(236, 1), End(237, -1)),
                            Tunnel(0, Segment(End(115, 1), End(116, -1)), Segment(End(116, -1), End(117, 1))),
                        ),
                        Bridge(
                            Segment(End(237, -1), End(238, 1)),
                            Segment(End(238, 1), End(239, -1)),
                            Tunnel(0, Segment(End(69, 1), End(70, -1)), Segment(End(70, -1), End(71, 1))),
                        ),
                        Bridge(
                            Segment(End(239, -1), End(240, 1)),
                            Segment(End(240, 1), End(241, -1)),
                            Tunnel(0, Segment(End(23, 1), End(24, -1)), Segment(End(24, -1), End(25, 1))),
                        ),
                        Bridge(
                            Segment(End(241, -1), End(243, 1)),
                            Segment(End(243, 1), End(244, -1)),
                            Tunnel(0, Segment(End(218, 1), End(219, -1)), Segment(End(219, -1), End(221, 1))),
                        ),
                        Bridge(
                            Segment(End(244, -1), End(245, 1)),
                            Segment(End(245, 1), End(246, -1)),
                            Tunnel(0, Segment(End(172, 1), End(173, -1)), Segment(End(173, -1), End(174, 1))),
                        ),
                        Bridge(
                            Segment(End(246, -1), End(247, 1)),
                            Segment(End(247, 1), End(248, -1)),
                            Tunnel(0, Segment(End(126, 1), End(127, -1)), Segment(End(127, -1), End(128, 1))),
                        ),
                        Bridge(
                            Segment(End(248, -1), End(249, 1)),
                            Segment(End(249, 1), End(250, -1)),
                            Tunnel(0, Segment(End(80, 1), End(81, -1)), Segment(End(81, -1), End(82, 1))),
                        ),
                        Bridge(
                            Segment(End(250, -1), End(251, 1)),
                            Segment(End(251, 1), End(252, -1)),
                            Tunnel(0, Segment(End(34, 1), End(35, -1)), Segment(End(35, -1), End(36, 1))),
                        ),
                        Bridge(
                            Segment(End(252, -1), End(254, 1)),
                            Segment(End(254, 1), End(255, -1)),
                            Tunnel(0, Segment(End(229, 1), End(230, -1)), Segment(End(230, -1), End(232, 1))),
                        ),
                        Bridge(
                            Segment(End(255, -1), End(256, 1)),
                            Segment(End(256, 1), End(257, -1)),
                            Tunnel(0, Segment(End(183, 1), End(184, -1)), Segment(End(184, -1), End(185, 1))),
                        ),
                        Bridge(
                            Segment(End(257, -1), End(258, 1)),
                            Segment(End(258, 1), End(259, -1)),
                            Tunnel(0, Segment(End(137, 1), End(138, -1)), Segment(End(138, -1), End(139, 1))),
                        ),
                        Bridge(
                            Segment(End(259, -1), End(260, 1)),
                            Segment(End(260, 1), End(261, -1)),
                            Tunnel(0, Segment(End(91, 1), End(92, -1)), Segment(End(92, -1), End(93, 1))),
                        ),
                        Bridge(
                            Segment(End(261, -1), End(262, 1)),
                            Segment(End(262, 1), End(263, -1)),
                            Tunnel(0, Segment(End(45, 1), End(46, -1)), Segment(End(46, -1), End(47, 1))),
                        ),
                    ],
                ),
            ]
        )

    def test_12_12(self):
        knot = Knot(12, 12)
        self.assertEqual(
            knot.strings,
            [
                String(
                    0,
                    [
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(25, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(1, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(3, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(5, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(7, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(9, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(13, -1)),
                            Tunnel(11, Segment(End(9, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(2, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(4, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(6, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(8, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(10, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    1,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(11, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(2, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(4, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(6, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(8, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(10, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(15, -1)),
                            Tunnel(0, Segment(End(11, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(3, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(5, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(7, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(9, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    2,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(10, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(0, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(3, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(5, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(7, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(9, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(11, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(17, -1)),
                            Tunnel(1, Segment(End(13, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(4, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(6, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(8, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    3,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(9, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(11, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(1, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(4, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(6, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(8, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(10, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(0, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(19, -1)),
                            Tunnel(2, Segment(End(15, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(5, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(7, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    4,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(9, 1)),
                        Segment(End(9, 1), End(10, -1)),
                        Segment(End(10, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(21, -1)),
                        Segment(End(21, -1), End(22, 1)),
                        Segment(End(22, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(8, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(10, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(0, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(2, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(9, 1)),
                            Segment(End(9, 1), End(10, -1)),
                            Tunnel(5, Segment(End(8, 1), End(9, -1)), Segment(End(9, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(10, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(7, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(12, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(9, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(11, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(1, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(21, -1)),
                            Tunnel(3, Segment(End(17, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(22, 1)),
                            Segment(End(22, 1), End(23, -1)),
                            Tunnel(6, Segment(End(21, 1), End(22, -1)), Segment(End(22, -1), End(23, 1))),
                        ),
                    ],
                ),
                String(
                    5,
                    [
                        Segment(End(0, 1), End(1, -1)),
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(11, 1)),
                        Segment(End(11, 1), End(12, -1)),
                        Segment(End(12, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(23, -1)),
                        Segment(End(23, -1), End(24, 1)),
                    ],
                    [
                        Bridge(
                            Segment(End(-1, -1), End(0, 1)),
                            Segment(End(0, 1), End(1, -1)),
                            Tunnel(7, Segment(End(-1, 1), End(0, -1)), Segment(End(0, -1), End(1, 1))),
                        ),
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(9, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(11, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(1, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(3, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(11, 1)),
                            Segment(End(11, 1), End(12, -1)),
                            Tunnel(6, Segment(End(10, 1), End(11, -1)), Segment(End(11, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(12, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(8, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(14, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(10, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(0, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(2, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(23, -1)),
                            Tunnel(4, Segment(End(19, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                    ],
                ),
                String(
                    6,
                    [
                        Segment(End(1, -1), End(2, 1)),
                        Segment(End(2, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(13, 1)),
                        Segment(End(13, 1), End(14, -1)),
                        Segment(End(14, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(25, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(1, -1), End(2, 1)),
                            Segment(End(2, 1), End(3, -1)),
                            Tunnel(8, Segment(End(1, 1), End(2, -1)), Segment(End(2, -1), End(3, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(10, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(0, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(2, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(4, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(13, 1)),
                            Segment(End(13, 1), End(14, -1)),
                            Tunnel(7, Segment(End(12, 1), End(13, -1)), Segment(End(13, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(14, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(9, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(16, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(11, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(1, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(3, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(25, -1)),
                            Tunnel(5, Segment(End(21, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                    ],
                ),
                String(
                    7,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(3, -1)),
                        Segment(End(3, -1), End(4, 1)),
                        Segment(End(4, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(15, 1)),
                        Segment(End(15, 1), End(16, -1)),
                        Segment(End(16, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(3, -1)),
                            Tunnel(6, Segment(End(-1, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(3, -1), End(4, 1)),
                            Segment(End(4, 1), End(5, -1)),
                            Tunnel(9, Segment(End(3, 1), End(4, -1)), Segment(End(4, -1), End(5, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(11, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(1, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(3, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(5, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(15, 1)),
                            Segment(End(15, 1), End(16, -1)),
                            Tunnel(8, Segment(End(14, 1), End(15, -1)), Segment(End(15, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(16, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(10, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(18, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(0, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(2, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(4, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                    ],
                ),
                String(
                    8,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(5, -1)),
                        Segment(End(5, -1), End(6, 1)),
                        Segment(End(6, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(17, 1)),
                        Segment(End(17, 1), End(18, -1)),
                        Segment(End(18, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(5, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(5, -1)),
                            Tunnel(7, Segment(End(1, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(5, -1), End(6, 1)),
                            Segment(End(6, 1), End(7, -1)),
                            Tunnel(10, Segment(End(5, 1), End(6, -1)), Segment(End(6, -1), End(7, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(0, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(2, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(4, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(6, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(17, 1)),
                            Segment(End(17, 1), End(18, -1)),
                            Tunnel(9, Segment(End(16, 1), End(17, -1)), Segment(End(17, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(18, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(11, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(20, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(1, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(3, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                    ],
                ),
                String(
                    9,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(7, -1)),
                        Segment(End(7, -1), End(8, 1)),
                        Segment(End(8, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(19, 1)),
                        Segment(End(19, 1), End(20, -1)),
                        Segment(End(20, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(4, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(6, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(7, -1)),
                            Tunnel(8, Segment(End(3, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(7, -1), End(8, 1)),
                            Segment(End(8, 1), End(9, -1)),
                            Tunnel(11, Segment(End(7, 1), End(8, -1)), Segment(End(8, -1), End(9, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(1, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(3, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(5, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(7, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(19, 1)),
                            Segment(End(19, 1), End(20, -1)),
                            Tunnel(10, Segment(End(18, 1), End(19, -1)), Segment(End(19, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(20, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(0, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(22, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(2, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                    ],
                ),
                String(
                    10,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(9, -1)),
                        Segment(End(9, -1), End(10, 1)),
                        Segment(End(10, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(21, 1)),
                        Segment(End(21, 1), End(22, -1)),
                        Segment(End(22, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(3, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(5, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(7, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(9, -1)),
                            Tunnel(9, Segment(End(5, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(9, -1), End(10, 1)),
                            Segment(End(10, 1), End(11, -1)),
                            Tunnel(0, Segment(End(9, 1), End(10, -1)), Segment(End(10, -1), End(11, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(2, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(4, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(6, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(8, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(21, 1)),
                            Segment(End(21, 1), End(22, -1)),
                            Tunnel(11, Segment(End(20, 1), End(21, -1)), Segment(End(21, -1), End(23, 1))),
                        ),
                        Bridge(
                            Segment(End(22, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(1, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(24, 1))),
                        ),
                    ],
                ),
                String(
                    11,
                    [
                        Segment(End(0, -1), End(1, 1)),
                        Segment(End(1, 1), End(2, -1)),
                        Segment(End(2, -1), End(3, 1)),
                        Segment(End(3, 1), End(4, -1)),
                        Segment(End(4, -1), End(5, 1)),
                        Segment(End(5, 1), End(6, -1)),
                        Segment(End(6, -1), End(7, 1)),
                        Segment(End(7, 1), End(8, -1)),
                        Segment(End(8, -1), End(9, 1)),
                        Segment(End(9, 1), End(11, -1)),
                        Segment(End(11, -1), End(12, 1)),
                        Segment(End(12, 1), End(13, -1)),
                        Segment(End(13, -1), End(14, 1)),
                        Segment(End(14, 1), End(15, -1)),
                        Segment(End(15, -1), End(16, 1)),
                        Segment(End(16, 1), End(17, -1)),
                        Segment(End(17, -1), End(18, 1)),
                        Segment(End(18, 1), End(19, -1)),
                        Segment(End(19, -1), End(20, 1)),
                        Segment(End(20, 1), End(21, -1)),
                        Segment(End(21, -1), End(23, 1)),
                        Segment(End(23, 1), End(24, -1)),
                    ],
                    [
                        Bridge(
                            Segment(End(0, -1), End(1, 1)),
                            Segment(End(1, 1), End(2, -1)),
                            Tunnel(2, Segment(End(0, 1), End(1, -1)), Segment(End(1, -1), End(2, 1))),
                        ),
                        Bridge(
                            Segment(End(2, -1), End(3, 1)),
                            Segment(End(3, 1), End(4, -1)),
                            Tunnel(4, Segment(End(2, 1), End(3, -1)), Segment(End(3, -1), End(4, 1))),
                        ),
                        Bridge(
                            Segment(End(4, -1), End(5, 1)),
                            Segment(End(5, 1), End(6, -1)),
                            Tunnel(6, Segment(End(4, 1), End(5, -1)), Segment(End(5, -1), End(6, 1))),
                        ),
                        Bridge(
                            Segment(End(6, -1), End(7, 1)),
                            Segment(End(7, 1), End(8, -1)),
                            Tunnel(8, Segment(End(6, 1), End(7, -1)), Segment(End(7, -1), End(8, 1))),
                        ),
                        Bridge(
                            Segment(End(8, -1), End(9, 1)),
                            Segment(End(9, 1), End(11, -1)),
                            Tunnel(10, Segment(End(7, 1), End(9, -1)), Segment(End(9, -1), End(10, 1))),
                        ),
                        Bridge(
                            Segment(End(11, -1), End(12, 1)),
                            Segment(End(12, 1), End(13, -1)),
                            Tunnel(1, Segment(End(11, 1), End(12, -1)), Segment(End(12, -1), End(13, 1))),
                        ),
                        Bridge(
                            Segment(End(13, -1), End(14, 1)),
                            Segment(End(14, 1), End(15, -1)),
                            Tunnel(3, Segment(End(13, 1), End(14, -1)), Segment(End(14, -1), End(15, 1))),
                        ),
                        Bridge(
                            Segment(End(15, -1), End(16, 1)),
                            Segment(End(16, 1), End(17, -1)),
                            Tunnel(5, Segment(End(15, 1), End(16, -1)), Segment(End(16, -1), End(17, 1))),
                        ),
                        Bridge(
                            Segment(End(17, -1), End(18, 1)),
                            Segment(End(18, 1), End(19, -1)),
                            Tunnel(7, Segment(End(17, 1), End(18, -1)), Segment(End(18, -1), End(19, 1))),
                        ),
                        Bridge(
                            Segment(End(19, -1), End(20, 1)),
                            Segment(End(20, 1), End(21, -1)),
                            Tunnel(9, Segment(End(19, 1), End(20, -1)), Segment(End(20, -1), End(21, 1))),
                        ),
                        Bridge(
                            Segment(End(21, -1), End(23, 1)),
                            Segment(End(23, 1), End(24, -1)),
                            Tunnel(0, Segment(End(22, 1), End(23, -1)), Segment(End(23, -1), End(25, 1))),
                        ),
                    ],
                ),
            ]
        )
